class AuthorizationResponse(object):
    def __init__(
            self,
            success,                # type: bool
            authorized_by,          # type: str
            authorized_by_level     # type: str
    ):
        # type: (...) -> None

        self.success = success
        self.authorized_by = authorized_by
        self.authorized_by_level = authorized_by_level


