from typing import Optional

from sysactions import action
from custom_sysactions import user_authorized_to_execute_action


@action
@user_authorized_to_execute_action
def clear_hold_item(
    pos_id,             # type: str
    sale_line_json,     # type: Optional[str]
):
    # type: (...) -> bool

    from table_actions import set_custom_production_tag

    return set_custom_production_tag(
        pos_id=pos_id,
        sale_line_json=sale_line_json,
    )
