from typing import Dict


class ConfiguredAlert(object):
    def __init__(
        self,
        alert_id,    # type: str
        alert_json,  # type: Dict
    ):
        # type: (...) -> None

        self.id = alert_id
        self.alert_json = alert_json
