from actions.events import event_publisher


def force_send_updated_model_to_ui(
    pos_id,         # type: str
    event_data="",  # type: str
):
    # type: (...) -> None

    event_subject = "POS{}".format(pos_id)
    event_publisher.publish_event(
        event_subject=event_subject,
        action_type="RESYNC_MODEL",
        event_type="RESYNC_MODEL",
        event_data=str(event_data),
    )
