class SitefParams(object):
    TRANSACTION_DATE = 105
    RECEIPT_CUSTOMER = 121
    RECEIPT_MERCHANT = 122
    TRANSACTION_PROCESSOR = 131
    CARD_TYPE = 132
    NSU = 133
    NSU_HOST = 134
    AUTH_CODE = (135, 952)
    CARD_BIN = 136
    INSTITUTION_NAME = 156
    AUTHORIZER_NAME = 158
    EXPIRATION_DATE = 513
    QR_CODE = 584
    AUTHORIZER_CNPJ = 950
    ID_MERCADO_PAGO = 4077
    DIGITAL_WALLET_ID = 106
    LAST_CARD_DIGITS = 1190
    SAT_ACQUIRER = 953
    NFCE_BRAND = 951
    MESSAGE_CODE = 2967
    MIN_SIZE = 2968
    MAX_SIZE = 2969
    TIME_OUT = 2970
    CANCEL_OPERATION_REASON = 111
