import os

from enum import Enum

from systools import sys_parsefilepath


class SystemPaths(Enum):

    data_path = sys_parsefilepath("{HVDATADIR}")
    genesis_data_path = os.path.join(data_path, "../../genesis/data")
    genesis_server_path = os.path.join(data_path, "../../genesis/data/server")
    genesis_databases_path = os.path.join(data_path, "../../genesis/data/server/databases")
    genesis_bundles_path = os.path.join(data_path, "../../genesis/data/server/bundles")
    static_configuration_folder = os.path.join(data_path, "../../genesis", "static")
