""" Python Character Mapping Codec generated from 'VENDORS/MICSFT/PC/CP860.TXT' with gencodec.py.
"""

import codecs

from typing import (
    Union,
    Optional,
)


def custom_encode(
    text,             # type: Union[str, unicode]
    errors='strict',  # type: Optional[str]
):
    return codecs.charmap_encode(
        text,
        errors,
        encoding_map
    )[0]


encoding_map = {
    0x0000: 0x0000,  # NULL
    0x0001: 0x0001,  # START OF HEADING
    0x0002: 0x0002,  # START OF TEXT
    0x0003: 0x0003,  # END OF TEXT
    0x0004: 0x0004,  # END OF TRANSMISSION
    0x0005: 0x0005,  # ENQUIRY
    0x0006: 0x0006,  # ACKNOWLEDGE
    0x0007: 0x0007,  # BELL
    0x0008: 0x0008,  # BACKSPACE
    0x0009: 0x0009,  # HORIZONTAL TABULATION
    0x000a: 0x000a,  # LINE FEED
    0x000b: 0x000b,  # VERTICAL TABULATION
    0x000c: 0x000c,  # FORM FEED
    0x000d: 0x000d,  # CARRIAGE RETURN
    0x000e: 0x000e,  # SHIFT OUT
    0x000f: 0x000f,  # SHIFT IN
    0x0010: 0x0010,  # DATA LINK ESCAPE
    0x0011: 0x0011,  # DEVICE CONTROL ONE
    0x0012: 0x0012,  # DEVICE CONTROL TWO
    0x0013: 0x0013,  # DEVICE CONTROL THREE
    0x0014: 0x0014,  # DEVICE CONTROL FOUR
    0x0015: 0x0015,  # NEGATIVE ACKNOWLEDGE
    0x0016: 0x0016,  # SYNCHRONOUS IDLE
    0x0017: 0x0017,  # END OF TRANSMISSION BLOCK
    0x0018: 0x0018,  # CANCEL
    0x0019: 0x0019,  # END OF MEDIUM
    0x001a: 0x001a,  # SUBSTITUTE
    0x001b: 0x001b,  # ESCAPE
    0x001c: 0x001c,  # FILE SEPARATOR
    0x001d: 0x001d,  # GROUP SEPARATOR
    0x001e: 0x001e,  # RECORD SEPARATOR
    0x001f: 0x001f,  # UNIT SEPARATOR
    0x0020: 0x0020,  # SPACE
    0x0021: 0x0021,  # EXCLAMATION MARK
    0x0022: 0x0022,  # QUOTATION MARK
    0x0023: 0x0023,  # NUMBER SIGN
    0x0024: 0x0024,  # DOLLAR SIGN
    0x0025: 0x0025,  # PERCENT SIGN
    0x0026: 0x0026,  # AMPERSAND
    0x0027: 0x0027,  # APOSTROPHE
    0x0028: 0x0028,  # LEFT PARENTHESIS
    0x0029: 0x0029,  # RIGHT PARENTHESIS
    0x002a: 0x002a,  # ASTERISK
    0x002b: 0x002b,  # PLUS SIGN
    0x002c: 0x002c,  # COMMA
    0x002d: 0x002d,  # HYPHEN-MINUS
    0x002e: 0x002e,  # FULL STOP
    0x002f: 0x002f,  # SOLIDUS
    0x0030: 0x0030,  # DIGIT ZERO
    0x0031: 0x0031,  # DIGIT ONE
    0x0032: 0x0032,  # DIGIT TWO
    0x0033: 0x0033,  # DIGIT THREE
    0x0034: 0x0034,  # DIGIT FOUR
    0x0035: 0x0035,  # DIGIT FIVE
    0x0036: 0x0036,  # DIGIT SIX
    0x0037: 0x0037,  # DIGIT SEVEN
    0x0038: 0x0038,  # DIGIT EIGHT
    0x0039: 0x0039,  # DIGIT NINE
    0x003a: 0x003a,  # COLON
    0x003b: 0x003b,  # SEMICOLON
    0x003c: 0x003c,  # LESS-THAN SIGN
    0x003d: 0x003d,  # EQUALS SIGN
    0x003e: 0x003e,  # GREATER-THAN SIGN
    0x003f: 0x003f,  # QUESTION MARK
    0x0040: 0x0040,  # COMMERCIAL AT
    0x0041: 0x0041,  # LATIN CAPITAL LETTER A
    0x0042: 0x0042,  # LATIN CAPITAL LETTER B
    0x0043: 0x0043,  # LATIN CAPITAL LETTER C
    0x0044: 0x0044,  # LATIN CAPITAL LETTER D
    0x0045: 0x0045,  # LATIN CAPITAL LETTER E
    0x0046: 0x0046,  # LATIN CAPITAL LETTER F
    0x0047: 0x0047,  # LATIN CAPITAL LETTER G
    0x0048: 0x0048,  # LATIN CAPITAL LETTER H
    0x0049: 0x0049,  # LATIN CAPITAL LETTER I
    0x004a: 0x004a,  # LATIN CAPITAL LETTER J
    0x004b: 0x004b,  # LATIN CAPITAL LETTER K
    0x004c: 0x004c,  # LATIN CAPITAL LETTER L
    0x004d: 0x004d,  # LATIN CAPITAL LETTER M
    0x004e: 0x004e,  # LATIN CAPITAL LETTER N
    0x004f: 0x004f,  # LATIN CAPITAL LETTER O
    0x0050: 0x0050,  # LATIN CAPITAL LETTER P
    0x0051: 0x0051,  # LATIN CAPITAL LETTER Q
    0x0052: 0x0052,  # LATIN CAPITAL LETTER R
    0x0053: 0x0053,  # LATIN CAPITAL LETTER S
    0x0054: 0x0054,  # LATIN CAPITAL LETTER T
    0x0055: 0x0055,  # LATIN CAPITAL LETTER U
    0x0056: 0x0056,  # LATIN CAPITAL LETTER V
    0x0057: 0x0057,  # LATIN CAPITAL LETTER W
    0x0058: 0x0058,  # LATIN CAPITAL LETTER X
    0x0059: 0x0059,  # LATIN CAPITAL LETTER Y
    0x005a: 0x005a,  # LATIN CAPITAL LETTER Z
    0x005b: 0x005b,  # LEFT SQUARE BRACKET
    0x005c: 0x005c,  # REVERSE SOLIDUS
    0x005d: 0x005d,  # RIGHT SQUARE BRACKET
    0x005e: 0x005e,  # CIRCUMFLEX ACCENT
    0x005f: 0x005f,  # LOW LINE
    0x0060: 0x0060,  # GRAVE ACCENT
    0x0061: 0x0061,  # LATIN SMALL LETTER A
    0x0062: 0x0062,  # LATIN SMALL LETTER B
    0x0063: 0x0063,  # LATIN SMALL LETTER C
    0x0064: 0x0064,  # LATIN SMALL LETTER D
    0x0065: 0x0065,  # LATIN SMALL LETTER E
    0x0066: 0x0066,  # LATIN SMALL LETTER F
    0x0067: 0x0067,  # LATIN SMALL LETTER G
    0x0068: 0x0068,  # LATIN SMALL LETTER H
    0x0069: 0x0069,  # LATIN SMALL LETTER I
    0x006a: 0x006a,  # LATIN SMALL LETTER J
    0x006b: 0x006b,  # LATIN SMALL LETTER K
    0x006c: 0x006c,  # LATIN SMALL LETTER L
    0x006d: 0x006d,  # LATIN SMALL LETTER M
    0x006e: 0x006e,  # LATIN SMALL LETTER N
    0x006f: 0x006f,  # LATIN SMALL LETTER O
    0x0070: 0x0070,  # LATIN SMALL LETTER P
    0x0071: 0x0071,  # LATIN SMALL LETTER Q
    0x0072: 0x0072,  # LATIN SMALL LETTER R
    0x0073: 0x0073,  # LATIN SMALL LETTER S
    0x0074: 0x0074,  # LATIN SMALL LETTER T
    0x0075: 0x0075,  # LATIN SMALL LETTER U
    0x0076: 0x0076,  # LATIN SMALL LETTER V
    0x0077: 0x0077,  # LATIN SMALL LETTER W
    0x0078: 0x0078,  # LATIN SMALL LETTER X
    0x0079: 0x0079,  # LATIN SMALL LETTER Y
    0x007a: 0x007a,  # LATIN SMALL LETTER Z
    0x007b: 0x007b,  # LEFT CURLY BRACKET
    0x007c: 0x007c,  # VERTICAL LINE
    0x007d: 0x007d,  # RIGHT CURLY BRACKET
    0x007e: 0x007e,  # TILDE
    0x007f: 0x007f,  # DELETE
    0x00a0: 0x00ff,  # NO-BREAK SPACE
    0x00a1: 0x00ad,  # INVERTED EXCLAMATION MARK
    0x00a3: 0x009c,  # POUND SIGN
    0x00ab: 0x00ae,  # LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
    0x00ac: 0x00aa,  # NOT SIGN
    0x00b0: 0x00f8,  # DEGREE SIGN
    0x00b1: 0x00f1,  # PLUS-MINUS SIGN
    0x00b2: 0x00fd,  # SUPERSCRIPT TWO
    0x00b5: 0x00e6,  # MICRO SIGN
    0x00b7: 0x00fa,  # MIDDLE DOT
    0x00bb: 0x00af,  # RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
    0x00bc: 0x00ac,  # VULGAR FRACTION ONE QUARTER
    0x00bd: 0x00ab,  # VULGAR FRACTION ONE HALF
    0x00bf: 0x00a8,  # INVERTED QUESTION MARK
    0x00c1: 0x00a4,  # LATIN CAPITAL LETTER A WITH ACUTE
    0x00c4: 0x008e,  # LATIN CAPITAL LETTER A WITH DIAERESIS
    0x00c3: 0x008e,  # LATIN CAPITAL LETTER A WITH TILDE
    0x00c5: 0x008f,  # LATIN CAPITAL LETTER A WITH RING ABOVE
    0x00c6: 0x0092,  # LATIN CAPITAL LIGATURE AE
    0x00c7: 0x0080,  # LATIN CAPITAL LETTER C WITH CEDILLA
    0x00c9: 0x0090,  # LATIN CAPITAL LETTER E WITH ACUTE
    0x00cd: 0x00a5,  # LATIN CAPITAL LETTER I WITH ACUTE
    0x00d0: 0x008b,  # LATIN CAPITAL LETTER ETH
    0x00d3: 0x00a6,  # LATIN CAPITAL LETTER O WITH ACUTE
    0x00d6: 0x0099,  # LATIN CAPITAL LETTER O WITH DIAERESIS
    0x00d8: 0x009d,  # LATIN CAPITAL LETTER O WITH STROKE
    0x00da: 0x00a7,  # LATIN CAPITAL LETTER U WITH ACUTE
    0x00dc: 0x009a,  # LATIN CAPITAL LETTER U WITH DIAERESIS
    0x00dd: 0x0097,  # LATIN CAPITAL LETTER Y WITH ACUTE
    0x00de: 0x008d,  # LATIN CAPITAL LETTER THORN
    0x00df: 0x00e1,  # LATIN SMALL LETTER SHARP S
    0x00e0: 0x0085,  # LATIN SMALL LETTER A WITH GRAVE
    0x00e1: 0x00a0,  # LATIN SMALL LETTER A WITH ACUTE
    0x00e2: 0x0083,  # LATIN SMALL LETTER A WITH CIRCUMFLEX
    0x00e4: 0x0084,  # LATIN SMALL LETTER A WITH DIAERESIS
    0x00e5: 0x0086,  # LATIN SMALL LETTER A WITH RING ABOVE
    0x00e6: 0x0091,  # LATIN SMALL LIGATURE AE
    0x00e7: 0x0087,  # LATIN SMALL LETTER C WITH CEDILLA
    0x00e8: 0x008a,  # LATIN SMALL LETTER E WITH GRAVE
    0x00e9: 0x0082,  # LATIN SMALL LETTER E WITH ACUTE
    0x00ea: 0x0088,  # LATIN SMALL LETTER E WITH CIRCUMFLEX
    0x00eb: 0x0089,  # LATIN SMALL LETTER E WITH DIAERESIS
    0x00ed: 0x00a1,  # LATIN SMALL LETTER I WITH ACUTE
    0x00f0: 0x008c,  # LATIN SMALL LETTER ETH
    0x00f3: 0x00a2,  # LATIN SMALL LETTER O WITH ACUTE
    0x00f4: 0x0093,  # LATIN SMALL LETTER O WITH CIRCUMFLEX
    0x00f6: 0x0094,  # LATIN SMALL LETTER O WITH DIAERESIS
    0x00f7: 0x00f6,  # DIVISION SIGN
    0x00f8: 0x009b,  # LATIN SMALL LETTER O WITH STROKE
    0x00fa: 0x00a3,  # LATIN SMALL LETTER U WITH ACUTE
    0x00fc: 0x0081,  # LATIN SMALL LETTER U WITH DIAERESIS
    0x00fd: 0x0098,  # LATIN SMALL LETTER Y WITH ACUTE
    0x00fe: 0x0095,  # LATIN SMALL LETTER THORN
    0x0192: 0x009f,  # LATIN SMALL LETTER F WITH HOOK
    0x0393: 0x00e2,  # GREEK CAPITAL LETTER GAMMA
    0x0398: 0x00e9,  # GREEK CAPITAL LETTER THETA
    0x03a3: 0x00e4,  # GREEK CAPITAL LETTER SIGMA
    0x03a6: 0x00e8,  # GREEK CAPITAL LETTER PHI
    0x03a9: 0x00ea,  # GREEK CAPITAL LETTER OMEGA
    0x03b1: 0x00e0,  # GREEK SMALL LETTER ALPHA
    0x03b4: 0x00eb,  # GREEK SMALL LETTER DELTA
    0x03b5: 0x00ee,  # GREEK SMALL LETTER EPSILON
    0x03c0: 0x00e3,  # GREEK SMALL LETTER PI
    0x03c3: 0x00e5,  # GREEK SMALL LETTER SIGMA
    0x03c4: 0x00e7,  # GREEK SMALL LETTER TAU
    0x03c6: 0x00ed,  # GREEK SMALL LETTER PHI
    0x0404: 0x00A4,  # CYRILLIC CAPITAL LETTER UKRAINIAN
    0x207f: 0x00fc,  # SUPERSCRIPT LATIN SMALL LETTER N
    0x20a7: 0x009e,  # PESETA SIGN
    0x2219: 0x00f9,  # BULLET OPERATOR
    0x221a: 0x00fb,  # SQUARE ROOT
    0x221e: 0x00ec,  # INFINITY
    0x2229: 0x00ef,  # INTERSECTION
    0x2248: 0x00f7,  # ALMOST EQUAL TO
    0x2261: 0x00f0,  # IDENTICAL TO
    0x2264: 0x00f3,  # LESS-THAN OR EQUAL TO
    0x2265: 0x00f2,  # GREATER-THAN OR EQUAL TO
    0x2310: 0x00a9,  # REVERSED NOT SIGN
    0x2320: 0x00f4,  # TOP HALF INTEGRAL
    0x2321: 0x00f5,  # BOTTOM HALF INTEGRAL
    0x2500: 0x00c4,  # BOX DRAWINGS LIGHT HORIZONTAL
    0x2502: 0x00b3,  # BOX DRAWINGS LIGHT VERTICAL
    0x250c: 0x00da,  # BOX DRAWINGS LIGHT DOWN AND RIGHT
    0x2510: 0x00bf,  # BOX DRAWINGS LIGHT DOWN AND LEFT
    0x2514: 0x00c0,  # BOX DRAWINGS LIGHT UP AND RIGHT
    0x2518: 0x00d9,  # BOX DRAWINGS LIGHT UP AND LEFT
    0x251c: 0x00c3,  # BOX DRAWINGS LIGHT VERTICAL AND RIGHT
    0x2524: 0x00b4,  # BOX DRAWINGS LIGHT VERTICAL AND LEFT
    0x252c: 0x00c2,  # BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
    0x2534: 0x00c1,  # BOX DRAWINGS LIGHT UP AND HORIZONTAL
    0x253c: 0x00c5,  # BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
    0x2550: 0x00cd,  # BOX DRAWINGS DOUBLE HORIZONTAL
    0x2551: 0x00ba,  # BOX DRAWINGS DOUBLE VERTICAL
    0x2552: 0x00d5,  # BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE
    0x2553: 0x00d6,  # BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE
    0x2554: 0x00c9,  # BOX DRAWINGS DOUBLE DOWN AND RIGHT
    0x2555: 0x00b8,  # BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE
    0x2556: 0x00b7,  # BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE
    0x2557: 0x00bb,  # BOX DRAWINGS DOUBLE DOWN AND LEFT
    0x2558: 0x00d4,  # BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE
    0x2559: 0x00d3,  # BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE
    0x255a: 0x00c8,  # BOX DRAWINGS DOUBLE UP AND RIGHT
    0x255b: 0x00be,  # BOX DRAWINGS UP SINGLE AND LEFT DOUBLE
    0x255c: 0x00bd,  # BOX DRAWINGS UP DOUBLE AND LEFT SINGLE
    0x255d: 0x00bc,  # BOX DRAWINGS DOUBLE UP AND LEFT
    0x255e: 0x00c6,  # BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE
    0x255f: 0x00c7,  # BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE
    0x2560: 0x00cc,  # BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
    0x2561: 0x00b5,  # BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE
    0x2562: 0x00b6,  # BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE
    0x2563: 0x00b9,  # BOX DRAWINGS DOUBLE VERTICAL AND LEFT
    0x2564: 0x00d1,  # BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE
    0x2565: 0x00d2,  # BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE
    0x2566: 0x00cb,  # BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
    0x2567: 0x00cf,  # BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE
    0x2568: 0x00d0,  # BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE
    0x2569: 0x00ca,  # BOX DRAWINGS DOUBLE UP AND HORIZONTAL
    0x256a: 0x00d8,  # BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE
    0x256b: 0x00d7,  # BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE
    0x256c: 0x00ce,  # BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
    0x2580: 0x00df,  # UPPER HALF BLOCK
    0x2584: 0x00dc,  # LOWER HALF BLOCK
    0x2588: 0x00db,  # FULL BLOCK
    0x258c: 0x00dd,  # LEFT HALF BLOCK
    0x2590: 0x00de,  # RIGHT HALF BLOCK
    0x2591: 0x00b0,  # LIGHT SHADE
    0x2592: 0x00b1,  # MEDIUM SHADE
    0x2593: 0x00b2,  # DARK SHADE
    0x25a0: 0x00fe,  # BLACK SQUARE

    # Manualy
    0x00fb: 0x0096,  # LATIN SMALL LETTER U WITH CIRCUMFLEX
    0x00db: 0x0096,  # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
    0x20ac: 0x00d5,  # EURO SIGN
    0x201c: 0x00f2,  # LEFT DOUBLE QUOTATION MARK
    0x201d: 0x00a6,  # RIGHT DOUBLE QUOTATION MARK
    0x2013: 0x002d,  # BIG DASH

    # CP860
    0x00a2: 0x009b,  # CENT SIGN
    0x00aa: 0x00a6,  # FEMININE ORDINAL INDICATOR
    0x00ba: 0x00a7,  # MASCULINE ORDINAL INDICATOR
    0x00c0: 0x0091,  # LATIN CAPITAL LETTER A WITH GRAVE
    0x00c2: 0x008f,  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
    0x00c8: 0x0092,  # LATIN CAPITAL LETTER E WITH GRAVE
    0x00ca: 0x0089,  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
    0x00cc: 0x0098,  # LATIN CAPITAL LETTER I WITH GRAVE
    0x00d1: 0x00a5,  # LATIN CAPITAL LETTER N WITH TILDE
    0x00d2: 0x00a9,  # LATIN CAPITAL LETTER O WITH GRAVE
    0x00d4: 0x008c,  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
    0x00d5: 0x0099,  # LATIN CAPITAL LETTER O WITH TILDE
    0x00d9: 0x009d,  # LATIN CAPITAL LETTER U WITH GRAVE
    0x00e3: 0x0084,  # LATIN SMALL LETTER A WITH TILDE
    0x00ec: 0x008d,  # LATIN SMALL LETTER I WITH GRAVE
    0x00f1: 0x00a4,  # LATIN SMALL LETTER N WITH TILDE
    0x00f2: 0x0095,  # LATIN SMALL LETTER O WITH GRAVE
    0x00f5: 0x0094,  # LATIN SMALL LETTER O WITH TILDE
    0x00f9: 0x0097,  # LATIN SMALL LETTER U WITH GRAVE

}
