-- noinspection SqlNoDataSourceInspectionForFile

-- Schema: i18ncustom
-- Brief: customized internationalization and localization message tables
-- Created by: amerolli
-- Table schema_version: must be defined for database update through the Persistence Component

PRAGMA journal_mode = OFF;

CREATE TABLE schema_version AS
SELECT "$Author: eztranslate$" AS Author,
       "$Date: 2023-08-25T14:09:34Z$" AS LastModifiedAt,
       "$Revision: 3b0b540b1b9045aa9ffad68e90a871cf$" AS Revision;


CREATE TABLE Languages (
    LangCode        INTEGER PRIMARY KEY AUTOINCREMENT,
    LangName        VARCHAR NOT NULL UNIQUE,
    ParentLangCode  INTEGER DEFAULT NULL
    CONSTRAINT FK_Languages_ParentLangCode REFERENCES Languages(LangCode)
);
CREATE TABLE Messages (
    LangCode    INTEGER NOT NULL
    CONSTRAINT FK_Messages_LangCode REFERENCES Languages(LangCode),
    MessageKey  VARCHAR,
    Message     VARCHAR,
    PRIMARY KEY(LangCode, MessageKey)
);

INSERT INTO LANGUAGES(LangCode, LangName, ParentLangCode) VALUES (4, 'pt', NULL);
INSERT INTO LANGUAGES(LangCode, LangName, ParentLangCode) VALUES (1, 'en', 4);
INSERT INTO LANGUAGES(LangCode, LangName, ParentLangCode) VALUES (6, 'es', 4);

BEGIN TRANSACTION;

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ABANDON', "Abandonar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ABANDON', "Abandon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ABANDON', "Abandonar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ABANDONED', "Abandonada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ABANDONED', "Abandoned");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ABANDONED', "Abandonada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ABANDON_TABLE_CONFIRMATION', "Tem certeza que deseja abandonar o serviço?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ABANDON_TABLE_CONFIRMATION', "Are you sure you want to abandon the service?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ABANDON_TABLE_CONFIRMATION', "¿Está seguro de abandonar el servicio?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ACCESS_DENIED', "Acesso negado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ACCESS_DENIED', "Access denied");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ACCESS_DENIED', "Acceso denegado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ACTION', "Ação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ACTION', "Action");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ACTION', "Acción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ACTIONS', "Ações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ACTIONS', "Actions");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ACTIONS', "Acciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ACTIVE', "Ativo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ACTIVE', "Active");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ACTIVE', "Activo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADD', "Adicionar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADD', "Add");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADD', "Añadir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADDITIONAL', "(Extra)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADDITIONAL', "(Extra)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADDITIONAL', "(Extra)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADDRESS', "Endereço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADDRESS', "Address");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADDRESS', "Dirección");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADDRESS_DELIVERY_REPORT', "Endereço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADDRESS_DELIVERY_REPORT', "Address");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADDRESS_DELIVERY_REPORT', "Dirección");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADDRESS_REFERENCE', "Referência");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADDRESS_REFERENCE', "Address reference");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADDRESS_REFERENCE', "Referencia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADD_COMMENT', "Nova Instrução");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADD_COMMENT', "Add comment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADD_COMMENT', "Añadir comentario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADD_ITEM_DISCOUNT_NOT_REMOVED', "Descontos que adicionam itens não podem ser removidos. Demais descontos removidos com sucesso.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADD_ITEM_DISCOUNT_NOT_REMOVED', "Discounts that add items cannot be removed. Too many discounts removed successfully.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADD_ITEM_DISCOUNT_NOT_REMOVED', "Los descuentos que agregan artículos no se pueden eliminar. Se eliminaron con éxito demasiados descuentos.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADD_ORDER', "Adicionar Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADD_ORDER', "Add Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADD_ORDER', "Agregar Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ADMINISTRATOR', "Administrador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ADMINISTRATOR', "Administrator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ADMINISTRATOR', "Administrador");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_CERTIFICATE_EXPIRATION_DAYS', "O Certificado Digital da loja vai vencer em {0} dias");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_CERTIFICATE_EXPIRATION_DAYS', "PENDING_TRADUCTION_ALERT_CERTIFICATE_EXPIRATION_DAYS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_CERTIFICATE_EXPIRATION_DAYS', "El Certificado Digital de la tienda va a vencer en {0} días");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_NOTIFY_ON_CANCEL', "Cancelamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_NOTIFY_ON_CANCEL', "Cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_NOTIFY_ON_CANCEL', "Cancelación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_NOTIFY_ON_DISCOUNT', "Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_NOTIFY_ON_DISCOUNT', "Discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_NOTIFY_ON_DISCOUNT', "Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_NOTIFY_ON_UNKNOW', "(desconhecido)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_NOTIFY_ON_UNKNOW', "(unknown)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_NOTIFY_ON_UNKNOW', "(desconocido)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_TYPE', "Tipo do Alerta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_TYPE', "Alert Type");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_TYPE', "Tipo de alerta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_USER', "Usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_USER', "User");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_USER', "Usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALERT_VALUE', "Valor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALERT_VALUE', "Value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALERT_VALUE', "Valor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL', "Todas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL', "All");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL', "Todas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_COUPONS', "Todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_COUPONS', "All");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_COUPONS', "Todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_OPERATORS', "Todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_OPERATORS', "All");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_OPERATORS', "Todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_POS_STATUS', "PDV Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_POS_STATUS', "POS Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_POS_STATUS', "TDV Status");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_PRODUCTS', "Todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_PRODUCTS', "All");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_PRODUCTS', "Todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_SEATS', "Todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_SEATS', "All");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_SEATS', "Todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_SECTORS', "Todos Setores");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_SECTORS', "All Sectors");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_SECTORS', "Todos los Sectores");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_STORE', "Toda Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_STORE', "All Store");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_STORE', "Toda la Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ALL_TENDERS', "Todos os pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ALL_TENDERS', "$PENDING_TRADUCTION_ALL_TENDERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ALL_TENDERS', "Todos los pagos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AMOUNT_DISCOUNT_BODY_REPORT', "Valor Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AMOUNT_DISCOUNT_BODY_REPORT', "Discount amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AMOUNT_DISCOUNT_BODY_REPORT', "Importe de descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AMOUNT_DUE_NOT_INFORMED', "Valor devido deve ser informado!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AMOUNT_DUE_NOT_INFORMED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AMOUNT_DUE_NOT_INFORMED', "¡Valor de la deuda debe ser informado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AMOUNT_ORDER_BODY_REPORT', "Valor Faturado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AMOUNT_ORDER_BODY_REPORT', "Invoiced Amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AMOUNT_ORDER_BODY_REPORT', "Importe facturado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AMOUNT_PAYABLE', "Valor a pagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AMOUNT_PAYABLE', "Amount to pay");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AMOUNT_PAYABLE', "Valor a pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APP', "APP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APP', "APP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APP', "APP");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLIED_COUPONS', "Promoções Aplicadas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLIED_COUPONS', "Applied Promotions");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLIED_COUPONS', "Promociones Aplicadas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLYING_CORE', "Aplicando CORE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLYING_CORE', "APPLYING_CORE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLYING_CORE', "Aplicando CORE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLYING_NEW_LOADERS', "Aplicando Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLYING_NEW_LOADERS', "APPLYING_NEW_LOADERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLYING_NEW_LOADERS', "Aplicando Loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLYING_SRC', "Aplicando SRC");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLYING_SRC', "APPLYING_SRC");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLYING_SRC', "Aplicando SRC");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLYING_UPDATE_MENU_VERSION_PENDING', "A atualização {0} de cardápio está pendente.\\Ela será aplicada e o sistema será reiniciado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLYING_UPDATE_MENU_VERSION_PENDING', "PENDING_TRADUCTION_APPLYING_UPDATE_MENU_VERSION_PENDING");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLYING_UPDATE_MENU_VERSION_PENDING', "La actualización {0} del menú está pendiente.\\La actualización será aplicada y el sistema se reiniciará");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLY_COUPONS', "Aplicar Cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLY_COUPONS', "Apply Coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLY_COUPONS', "Aplicar Cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'APPLY_DISCOUNTS', "Aplicar Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'APPLY_DISCOUNTS', "Apply Discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'APPLY_DISCOUNTS', "Aplicar Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ASK_BARCODE', "Digitar Código de Barras");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ASK_BARCODE', "Inform barcode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ASK_BARCODE', "Informar Código de Barras");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ASK_CPF_CNPJ', "O CPF será informado?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ASK_CPF_CNPJ', "$PENDING_TRADUCTION_ASK_CPF_CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ASK_CPF_CNPJ', "¿El DOCUMENTO será informado?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ASSOCIATED_FINGERPRINT_ERROR', "Impressão digital associada a usuário que não esta cadastrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ASSOCIATED_FINGERPRINT_ERROR', "Fingerprint associated to unkown user");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ASSOCIATED_FINGERPRINT_ERROR', "Huella digital asociada a otro usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTHENTICATION_FAILED', "Falha na autenticação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTHENTICATION_FAILED', "Authentication failed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTHENTICATION_FAILED', "Autenticación fallida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTHORIZE', "Autorizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTHORIZE', "Authoraze");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTHORIZE', "Autorizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTHORIZED_BY', "Autorizado Por");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTHORIZED_BY', "Authorized by");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTHORIZED_BY', "Autorizado por");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTH_ENTER_USER_ID', "Autorização - Gerente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTH_ENTER_USER_ID', "Authorization - Manager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTH_ENTER_USER_ID', "Autorización - Gerente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTOMATIC_BENEFIT', "Automático");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTOMATIC_BENEFIT', "Automatic");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTOMATIC_BENEFIT', "Automático");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AUTOMATIC_BENEFITS_TO_MANUAL', "Descontos automáticos precisarão ser aplicados manualmente. Por favor, verifique a lista de descontos.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AUTOMATIC_BENEFITS_TO_MANUAL', "Automatic discounts will need to be applied manually. Please check the discount list.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AUTOMATIC_BENEFITS_TO_MANUAL', "Los descuentos automáticos deberán aplicarse manualmente. Por favor, consulte la lista de descuentos.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AVAILABLE', "Disponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AVAILABLE', "Available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AVAILABLE', "Disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AVAILABLE_COUPONS', "Promoções Disponíveis");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AVAILABLE_COUPONS', "Promotions Available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AVAILABLE_COUPONS', "Promociones disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AVAILABLE_PRODUCTS', "Produtos Disponíveis");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AVAILABLE_PRODUCTS', "Available Products");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AVAILABLE_PRODUCTS', "Productos Disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'AVERAGE_TICKET', "TM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'AVERAGE_TICKET', "AT:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'AVERAGE_TICKET', "TM");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BACK', "Voltar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BACK', "Back");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BACK', "Atrás");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BACK_FROM_SOURCE_TABLE', "Voltar a mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BACK_FROM_SOURCE_TABLE', "Back to table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BACK_FROM_SOURCE_TABLE', "Volver a la mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BACK_TO_ORDER', "Voltar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BACK_TO_ORDER', "Back to Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BACK_TO_ORDER', "Atrás");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BARCODE_NOT_FOUND', "Código de barras não encontrado no banco de dados: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BARCODE_NOT_FOUND', "Barcode not found: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BARCODE_NOT_FOUND', "Código de Barras no encontrado: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BARCODE_PRODUCT_RUPTURED', "O produto {0} está indicado como indisponível.\\Deseja realizar a venda?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BARCODE_PRODUCT_RUPTURED', "The product {0} is unavailable.\\Sell it anyway?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BARCODE_PRODUCT_RUPTURED', "El producto {0} no está disponible.\\¿Vender de todas maneras?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BARCODE_WRONG', "Código de Barras não é um EAN13 válido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BARCODE_WRONG', "Invalid barcode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BARCODE_WRONG', "Código de Barras inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BEGIN_DATE', "Data Início");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BEGIN_DATE', "PENDING_TRADUCTION_BEGIN_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BEGIN_DATE', "Fecha Inicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT', "Relatório de Promoções");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT', "PENDING_TRADUCTION_BENEFIT_REPORT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT', "PENDING_TRADUCTION_BENEFIT_REPORT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_CONSOLIDATED', "Relatório de Promoções - Consolidado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_CONSOLIDATED', "PENDING_TRADUCTION_BENEFIT_REPORT_CONSOLIDATED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_CONSOLIDATED', "PENDING_TRADUCTION_BENEFIT_REPORT_CONSOLIDATED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_DATE_TIME', "Data/Hora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_DATE_TIME', "PENDING_TRADUCTION_BENEFIT_REPORT_PERIOD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_DATE_TIME', "PENDING_TRADUCTION_BENEFIT_REPORT_PERIOD");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_DETAILED', "Relatório de Promoções - Detalhado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_DETAILED', "PENDING_TRADUCTION_BENEFIT_REPORT_DETAILED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_DETAILED', "PENDING_TRADUCTION_BENEFIT_REPORT_DETAILED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_HEADER_DEFAULT_VALUE', "Padrão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_HEADER_DEFAULT_VALUE', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_DEFAULT_VALUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_HEADER_DEFAULT_VALUE', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_DEFAULT_VALUE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_HEADER_DISCOUNT_VALUE', "Desc");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_HEADER_DISCOUNT_VALUE', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_DEFAULT_VALUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_HEADER_DISCOUNT_VALUE', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_DEFAULT_VALUE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_HEADER_ITEMS', "Itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_HEADER_ITEMS', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_ITEMS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_HEADER_ITEMS', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_ITEMS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_HEADER_ORDER', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_HEADER_ORDER', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_ORDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_HEADER_ORDER', "PENDING_TRADUCTION_BENEFIT_REPORT_HEADER_ORDER");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_HEADER_PROMOTION_NAME', "Promoção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_HEADER_PROMOTION_NAME', "PENDING_TRADUCTION_PROMOTION_NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_HEADER_PROMOTION_NAME', "PENDING_TRADUCTION_PROMOTION_NAME");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_PERIOD', "Período");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_PERIOD', "PENDING_TRADUCTION_BENEFIT_REPORT_PERIOD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_PERIOD', "PENDING_TRADUCTION_BENEFIT_REPORT_PERIOD");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BENEFIT_REPORT_PROMOTION_NAME', "Promoção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BENEFIT_REPORT_PROMOTION_NAME', "PENDING_TRADUCTION_BENEFIT_REPORT_PROMOTION_NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BENEFIT_REPORT_PROMOTION_NAME', "PENDING_TRADUCTION_BENEFIT_REPORT_PROMOTION_NAME");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BINARIES', "Binários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BINARIES', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BINARIES', "Binarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BINARIES_NOT_FOUND', "Não existe uma versão de CORE / SRC disponível para a Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BINARIES_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BINARIES_NOT_FOUND', "No existe una versión de CORE / SRC disponible para la Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BLOCKED_NAME_MESSAGE', "O nome ou termo digitado não é válido.\Por favor escreva novamente.\\Nós repudiamos todo e qualquer ato de discriminação racial de gênero classe social\ou qualquer outro tipo. Nossa empresa preza pela igualdade e diversidade.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BLOCKED_NAME_MESSAGE', "The typed name or term is not valid.\Please type it again.\\We condemns any act of racial gender social class or any other kind of discrimination.\Our company values equality and diversity.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BLOCKED_NAME_MESSAGE', "El nombre o término escrito no es válido.\Por favor escríbalo nuevamente.\\ Condenamos cualquier acto de racismo racia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BLOCKED_NAME_MESSAGE_TITLE', "AVISO: NOME/TERMO BLOQUEADO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BLOCKED_NAME_MESSAGE_TITLE', "WARNING: BLOCKED NAME/TERM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BLOCKED_NAME_MESSAGE_TITLE', "ADVERTENCIA: NOMBRE BLOQUEADO/TÉRMINO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BLOCKING_DAYS_FOR_OPENING_BUSINESS_DAY', "Existem PDV com dia de negócio aberto a mais de {0} dias: {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BLOCKING_DAYS_FOR_OPENING_BUSINESS_DAY', "There are POS with business days open for more than {0} days: {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BLOCKING_DAYS_FOR_OPENING_BUSINESS_DAY', "Hay TPV con días hábiles abiertos por más de {0} días: {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BORDEREAU', "Borderô");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BORDEREAU', "Bordereau");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BORDEREAU', "Recaudación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BRAND', "E-DEPLOY PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BRAND', "E-DEPLOY POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BRAND', "E-DEPLOY TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'BY_SEAT', "Por Assento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'BY_SEAT', "By seat");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'BY_SEAT', "Por asiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CALL_PANEL', "PAINEL DE RETIRADA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CALL_PANEL', "CALL PANEL");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CALL_PANEL', "PANEL DE RETIRO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCEL', "Cancelar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCEL', "CANCEL");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCEL', "Cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED', "Cancelado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED', "Canceled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED', "Cancelado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED_CHANGE_PROFILE', "Operação cancelada. Perfil não alterado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED_CHANGE_PROFILE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED_CHANGE_PROFILE', "Operación cancelada. Perfil no alterado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED_DISABLE_USER', "Operação cancelada. Usuário não desativado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED_DISABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED_DISABLE_USER', "Operación cancelada. Usuario no desactivado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED_ENABLE_USER', "Operação cancelada. Usuário não ativado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED_ENABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED_ENABLE_USER', "Operación cancelada. Usuario no activado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED_ORDER', "VENDA CANCELADA NO PARCEIRO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED_ORDER', "ORDER CANCELED ON PARTNER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED_ORDER', "ORDEN CANCELADA EN SOCIO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELED_SALE', "A VENDA FOI CANCELADA\\\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELED_SALE', "The sale was canceled\\\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELED_SALE', "La venta ha sido cancelada\\\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELLATION', "Cancelamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELLATION', "Cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELLATION', "Cancelamiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCELLATION_VIA_SYSTEM', "Cancelamento via sistema");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCELLATION_VIA_SYSTEM', "PENDING_TRADUCTION_CANCELLATION_VIA_SYSTEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCELLATION_VIA_SYSTEM', "Cancelamiento via sistema");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANCEL_WITHOUT_ICON', "Cancelar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANCEL_WITHOUT_ICON', "CANCEL");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANCEL_WITHOUT_ICON', "Cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_ADD_ITEMS_IN_AN_OPENED_ORDER', "Não é possível adicionar itens em um pedido já salvo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_ADD_ITEMS_IN_AN_OPENED_ORDER', "It is not possible to add items to a saved order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_ADD_ITEMS_IN_AN_OPENED_ORDER', "No fue posible añadir artículos a una orden guardada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_CHANGE_ORDER_WITH_PAYMENTS', "Não é possível modificar um pedido após inserir um pagamento.\\Deseja cancelar o pedido atual?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_CHANGE_ORDER_WITH_PAYMENTS', "It is not possible to change an order when payments were added. Do you want to void the current order?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_CHANGE_ORDER_WITH_PAYMENTS', "No es posible cambiar una orden con pagos aplicados. ¿Desea anular la cuenta actual?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_CHANGE_PASSWORD', "Não é permitido alterar a senha desse usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_CHANGE_PASSWORD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_CHANGE_PASSWORD', "No está permitido cambiar la contraseña de este usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_CLOSE_DAYS', "Não é possível fechar o dia para os seguintes PDV:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_CLOSE_DAYS', "It is not possible to close the day in the following POS:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_CLOSE_DAYS', "No es posible cerrar el día en el siguiente TPV:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_CLOSE_OPERATOR', "Não foi possível fechar o Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_CLOSE_OPERATOR', "Not possible to close operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_CLOSE_OPERATOR', "No fue posible cerrar mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_FRACTION_TWICE', "Não é possível dividir um item já dividido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_FRACTION_TWICE', "Cannot split an item already splited");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_FRACTION_TWICE', "No es posible dividir un artículo ya dividido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_LOGIN_OPERATOR', "Não foi possível logar o Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_LOGIN_OPERATOR', "Not possible to log in operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_LOGIN_OPERATOR', "No fue posible iniciar sesión de mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_MERGE', "Não é possível juntar itens diferentes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_MERGE', "Cannot merge diferent items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_MERGE', "No es posible juntar dos artículos diferentes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_OPEN_OPERATOR', "Não foi possível abrir o Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_OPEN_OPERATOR', "Not possible to log in operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_OPEN_OPERATOR', "No fue posible iniciar sesión de mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_RECALL_ORDER_TOO_OLD', "Pedido muito antigo para ser recuperado. Faça outro pedido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_RECALL_ORDER_TOO_OLD', "This order is too old to be recalled. Start another sale.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_RECALL_ORDER_TOO_OLD', "Esta orden es muy antigua para ser recuperada. Abra otra cuenta.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_REMOVE_ITEMS_IN_AN_OPENED_ORDER', "Não é possível remover itens em um pedido já salvo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_REMOVE_ITEMS_IN_AN_OPENED_ORDER', "It is not possible to remove items from a saved order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_REMOVE_ITEMS_IN_AN_OPENED_ORDER', "No fue posible borrar artículos de una orden guardada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_REMOVE_ITEMS_IN_A_TOTALED_ORDER', "Não é possivel remover itens de um pedido totalizada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_REMOVE_ITEMS_IN_A_TOTALED_ORDER', "Cannot remove items in a totaled order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_REMOVE_ITEMS_IN_A_TOTALED_ORDER', "No se puede remover artículos de un pedido totalizado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_REMOVE_LOYALTY_ON_TOTALED_SCREEN', "Não é permitido remover dados de fidelidade na tela de pagamento com descontos aplicados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_REMOVE_LOYALTY_ON_TOTALED_SCREEN', "It is not allowed to remove loyalty data on the payment screen with discounts applied");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_REMOVE_LOYALTY_ON_TOTALED_SCREEN', "No está permitido eliminar datos de fidelización en la pantalla de pago con descuentos aplicados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_TRANSFER_TABLE_WITH_PARTIAL_AMOUNT', "Não é possível transferir uma mesa com pagamento parcial!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_TRANSFER_TABLE_WITH_PARTIAL_AMOUNT', "It is not possible to transfer a table with partial payment!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_TRANSFER_TABLE_WITH_PARTIAL_AMOUNT', "¡No es posible transferir una mesa con pago parcial!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANNOT_USE_TIP_BY_AMOUNT_AND_PERCENTAGE', "Porcentagem e valor não podem ser passados ao mesmo tempo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANNOT_USE_TIP_BY_AMOUNT_AND_PERCENTAGE', "Percentage and amount can not be passed in same time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANNOT_USE_TIP_BY_AMOUNT_AND_PERCENTAGE', "El porcentaje y la cantidad no se pueden pasar al mismo tiempo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANOT_OPEN_DAYS', "Não é possível abrir o dia para os seguintes PDV:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANOT_OPEN_DAYS', "It is not possible to open the day in the following POS:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANOT_OPEN_DAYS', "No es posible abrir el día en el siguiente TPV:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CANT_SAVE_REMOTE_ORDER_JSON', "Não foi possível salvar a propriedade Remote Order Json: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CANT_SAVE_REMOTE_ORDER_JSON', "Couldn't save Remote Order Json custom property: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CANT_SAVE_REMOTE_ORDER_JSON', "PENDING_TRADUCTION_CANT_SAVE_REMOTE_ORDER_JSON {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CARD', "Cartão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CARD', "Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CARD', "Terjeta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CARDS', "CARTÕES");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CARDS', "Cards");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CARDS', "Tarjetas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CARD_BRAND', "Bandeira do Cartão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CARD_BRAND', "Card Brand");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CARD_BRAND', "Marca de Tarjeta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CARD_PAYMENT_AMOUNT_HIGHER_THAN_DUE', "Valor para pagamento com cartão não pode ser superior ao devido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CARD_PAYMENT_AMOUNT_HIGHER_THAN_DUE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CARD_PAYMENT_AMOUNT_HIGHER_THAN_DUE', "¡El monto del pago con tarjeta no puede exceder al monto de la deuda!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH', "Dinheiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH', "Cash");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH', "Efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_BREAK_MESSAGE', "Justifique a quebra de caixa:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_BREAK_MESSAGE', "Justify cash break:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_BREAK_MESSAGE', "Justifique el retiro en efectivo:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_IN', "Suprimento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_IN', "Supply");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_IN', "Suministro");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_OUT', "Sangria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_OUT', "Skim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_OUT', "Efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_OUT_MAX_VALUE', "Valor Inválido. Digite um valor válido menor que R$ {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_OUT_MAX_VALUE', "Invalid value. Inform a value minor than ${0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_OUT_MAX_VALUE', "Valor inválido. Ingrese un valor menor que ${0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_PAID_IN', "Colocar Dinheiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_PAID_IN', "Cash Paid In");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_PAID_IN', "Ingresar Dinero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CASH_PAID_OUT', "Tirar Dinheiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CASH_PAID_OUT', "Cash Paid Out");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CASH_PAID_OUT', "Retirar Dinero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CATALOG', "Catálogo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CATALOG', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CATALOG', "Catálogo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CATALOG_NOT_FOUND', "Não existe Cardápio publicado para essa Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CATALOG_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CATALOG_NOT_FOUND', "No hay Menú publicada para esta tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CATALOG_START_UPDATE', "Iniciando atualização de cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CATALOG_START_UPDATE', "$PENDING_TRADUCTION_CATALOG_START_UPDATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CATALOG_START_UPDATE', "$PENDING_TRADUCTION_CATALOG_START_UPDATE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CATEGORY', "Catergoria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CATEGORY', "Category");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CATEGORY', "Categoría");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CERTIFICATE_EXPIRATION_READING_FAILURE', "Falha na Leitura da Validade do Certificado NFCE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CERTIFICATE_EXPIRATION_READING_FAILURE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CERTIFICATE_EXPIRATION_READING_FAILURE', "Falla en la Lectura de Vencimiento del Certificado NFCE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHALLENGE', "Desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHALLENGE', "PENDING_TRADUCTION_CHALLENGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHALLENGE', "Desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHALLENGES', "Desafios");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHALLENGES', "PENDING_TRADUCTION_CHALLENGES");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHALLENGES', "Desafíos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHALLENGE_PRODUCTS', "Produtos desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHALLENGE_PRODUCTS', "PENDING_TRADUCTION_CHALLENGE_PRODUCTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHALLENGE_PRODUCTS', "Productos desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHALLENGE_REPORT', "Relatório Desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHALLENGE_REPORT', "PENDING_TRADUCTION_CHALLENGE_REPORT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHALLENGE_REPORT', "Reporte Desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE', "Troco");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE', "Change");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE', "Cambio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGED_YOUR_MIND', "Mudou de Ideia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGED_YOUR_MIND', "Changed your mind");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGED_YOUR_MIND', "Cambió de idea");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_COURSE', "Curso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_COURSE', "Course");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_COURSE', "Curso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_INFORMATION', "Alterar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_INFORMATION', "Change");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_INFORMATION', "Cambiar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_LANGUAGE', "Trocar Idioma");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_LANGUAGE', "Change Language");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_LANGUAGE', "Cambiar Idioma");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_NEEDED', "LEVAR TROCO valor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_NEEDED', "Take change amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_NEEDED', "Tomar valor de cambio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_OPERATOR', "Mudar Garçom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_OPERATOR', "Change Operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_OPERATOR', "Cambiar Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_OR_KEEP_OPERATOR', "Deseja manter ou alterar o operador atual?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_OR_KEEP_OPERATOR', "Do you want to keep or change the current operator?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_OR_KEEP_OPERATOR', "¿Quieres mantener o cambiar el operador actual?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_SEATS', "# Assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_SEATS', "Change Number of Seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_SEATS', "Cambiar número de asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_TIP_TYPE', "A gorjeta será alterada de que forma?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_TIP_TYPE', "How ill the tip be changed?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_TIP_TYPE', "¿Cómo se cambiará la propina?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHANGE_USER_FUNCTION', "Trocar função do Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHANGE_USER_FUNCTION', "Change operator function");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHANGE_USER_FUNCTION', "Cambiar funciones de Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_AUTHENTICATION_TYPE', "Escolha o tipo de autenticação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_AUTHENTICATION_TYPE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_AUTHENTICATION_TYPE', "Elija el tipo de autenticación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_DESELECT_TYPE', "Escolha a opção de deseleção:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_DESELECT_TYPE', "Choose deselection type:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_DESELECT_TYPE', "Elegir tipo de deselección");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_ORDER_TYPE', "Escolha o tipo de pedido:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_ORDER_TYPE', "Choose the order type:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_ORDER_TYPE', "Elija el tipo de pedido:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_PMIX_PRODUCT_TYPE', "Escolha o tipo de listagem de produtos:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_PMIX_PRODUCT_TYPE', "Choose PMIX type:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_PMIX_PRODUCT_TYPE', "Elija tipo de PMIX");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_PROFILE', "Escolha o tipo de perfil");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_PROFILE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_PROFILE', "Elija el tipo de perfil");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CHOOSE_REOPEN_TYPE', "Escolha a opção de reabertura:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CHOOSE_REOPEN_TYPE', "Choose reopen type:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CHOOSE_REOPEN_TYPE', "Elegir tipo de reapertura de mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CITY', "Cidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CITY', "City");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CITY', "Ciudad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CITY_DELIVERY_REPORT', "Cidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CITY_DELIVERY_REPORT', "City");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CITY_DELIVERY_REPORT', "Ciudad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN', "Limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN', "Clean");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN', "Limpiar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_CLIENT', "Limpar Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_CLIENT', "Clear Client");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_CLIENT', "Limpiar Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_DISCOUNTS', "Limpar Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_DISCOUNTS', "Clear Discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_DISCOUNTS', "Limpiar Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "Para voltar é necessário limpar os descontos manuais aplicados existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "PENDING_TRADUCTION_CLEAN_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "Para regresar es necesario borrar descuentos manuales existentes.\\Confirmar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_MANUAL_DISCOUNTS_TO_STORE_ORDER', "Para salvar o pedido é necessário limpar os descontos manuais existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_MANUAL_DISCOUNTS_TO_STORE_ORDER', "PENDING_TRADUCTION_CLEAN_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_MANUAL_DISCOUNTS_TO_STORE_ORDER', "Para guardar el pedido es necesario borrar descuentos manuales existentes.\\Confirmar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_NUMPAD', "Limpeza de dados efetuada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_NUMPAD', "Data cleaning performed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_NUMPAD', "Limpieza de datos realizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_ORDERS', "Limpar carrinho");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_ORDERS', "Clear cart");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_ORDERS', "Limpiar Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "Para voltar é necessário limpar os pagamentos e descontos manuais existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "PENDING_TRADUCTION_CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION', "Para regresar es necesario borrar los pagos y descuentos manuales existentes.\\Confirmar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_STORE_ORDER', "Para salvar o pedido é necessário limpar os pagamentos e descontos manuais existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_STORE_ORDER', "PENDING_TRADUCTION_CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_EXECUTE_ACTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_AND_MANUAL_DISCOUNTS_TO_STORE_ORDER', "Para guardar el pedido es necesario borrar los pagos y descuentos manuales existentes.\\Confirmar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_TO_APPLY_DISCOUNT', "Para aplicação de desconto é necessário limpar os pagamentos existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_TO_APPLY_DISCOUNT', "For discounts it is necessary to clear the payments.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_TO_APPLY_DISCOUNT', "Para aplicar Descuentos es necesario borrar los pagos.\\¿Confirma la operación?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_TO_DESELECT', "Para deselecionar a mesa é necessário limpar os pagamentos existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_TO_DESELECT', "To unselect table it is necessary to clear the payments.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_TO_DESELECT', "Para deseleccionar mesa es necesario borrar los pagos existentes.\\¿Confirma la operación?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_TO_EXECUTE_ACTION', "Para voltar é necessário limpar os pagamentos existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_TO_EXECUTE_ACTION', "To go back is necessary to clear the payments.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_TO_EXECUTE_ACTION', "Para regresar es necesario borrar los pagos existentes.\\¿Confirma la operación?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAN_TENDERS_TO_STORE_ORDER', "Para salvar o pedido é necessário limpar os pagamentos existentes.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAN_TENDERS_TO_STORE_ORDER', "To store order is necessary to clear the payments.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAN_TENDERS_TO_STORE_ORDER', "Para guardar el pedido es necesario borrar los pagos existentes.\\Confirmar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_BARCODE', "Deseja limpar o código de barras?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_BARCODE', "Do you want to clear the barcode?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_BARCODE', "¿Quieres borrar el código de barras?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_HOLD_FIRE', "Limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_HOLD_FIRE', "Clear Fire");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_HOLD_FIRE', "Limpiar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_MANUAL_DISCOUNTS_CONFIRMATION', "Para continuar é necessário remover todos os descontos manuais.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_MANUAL_DISCOUNTS_CONFIRMATION', "PENDING_TRADUCTION_CLEAR_MANUAL_DISCOUNTS_CONFIRMATION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_MANUAL_DISCOUNTS_CONFIRMATION', "Para continuar se deben remover los descuentos manuales.\\¿Confirma la operación?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_OPTION', "Limpar Opções");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_OPTION', "Clear Options");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_OPTION', "Descartar Opciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_ORDER', "Cancelar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_ORDER', "Cancel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_ORDER', "Cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_TENDERS', "Limpar Pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_TENDERS', "Clear tenders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_TENDERS', "Limpiar Formas de Pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_TENDER_AND_MANUAL_DISCOUNTS_CONFIRMATION', "Para continuar é necessário remover todos os pagamentos e descontos manuais.\\Confirmar operação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_TENDER_AND_MANUAL_DISCOUNTS_CONFIRMATION', "PENDING_TRADUCTION_CLEAR_TENDER_AND_MANUAL_DISCOUNTS_CONFIRMATION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_TENDER_AND_MANUAL_DISCOUNTS_CONFIRMATION', "Para continuar se deben remover los pagos y descuentos manuales.\\¿Confirma la operación?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLEAR_TENDER_CONFIRMATION', "Tem certeza que deseja remover todos os pagamentos?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLEAR_TENDER_CONFIRMATION', "Are you sure do you want to remove all tenders?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLEAR_TENDER_CONFIRMATION', "¿Está seguro de borrar las forma de pago?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLICK_TO_START', "Toque na tela para fazer seu pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLICK_TO_START', "Click to start");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLICK_TO_START', "Dé clic para comenzar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLIENT_NAME', "Cliente: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLIENT_NAME', "Client: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLIENT_NAME', "Cliente: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSED', "Fechado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSED', "Closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSED', "Cerrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSED_ORDERS', "Pedidos Fechados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSED_ORDERS', "Closed Orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSED_ORDERS', "Pedidos Cerrados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_ALL_TABLES_OR_TABS', "Fechar todas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_ALL_TABLES_OR_TABS', "PENDING_TRADUCTION_CLOSE_ALL_TABLES_OR_TABS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_ALL_TABLES_OR_TABS', "Cerrar todas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_ALL_TABLES_OR_TABS_TITLE', "Fechar Comandas ou Mesas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_ALL_TABLES_OR_TABS_TITLE', "PENDING_TRADUCTION_CLOSE_ALL_TABLES_OR_TABS_TITLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_ALL_TABLES_OR_TABS_TITLE', "Cerrar Comandas o Mesas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_APPLICATION', "Fechar Aplicação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_APPLICATION', "Close Application");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_APPLICATION', "Cerrar Aplicación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_DAY', "Fechar o Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_DAY', "Close Day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_DAY', "Cerrar el Día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_DAY_DATABASE_CLEANUP_CONFIRMATION', "Necessário executar processo de backups do PDV e será necessário manter o servidor ligado. Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_DAY_DATABASE_CLEANUP_CONFIRMATION', "Need to run POS backups process and it will be necessary to keep the server powered on. Do you wish to continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_DAY_DATABASE_CLEANUP_CONFIRMATION', "Necesita ejecutar el proceso de copias de seguridad de TPV y será necesario mantener el servidor encendido. ¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_DELIVERY_ORDER_QUESTION', "Deseja finalizar o pedido?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_DELIVERY_ORDER_QUESTION', "Do you want to finalize the order?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_DELIVERY_ORDER_QUESTION', "¿Desea finalizar el pedido?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_OPERATORS', "Fechar Operadores");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_OPERATORS', "Close Operators");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_OPERATORS', "Cerrar Meseros");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_POS', "Fechar Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_POS', "Close the Day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_POS', "Cerrar el Dia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_TABLE', "Finalizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_TABLE', "Close Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_TABLE', "Cerrar Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_TABLE_LAST_SEAT', "Não existem outros assentos ocupados na mesa. A mesa será fechada após a separação deste assento.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_TABLE_LAST_SEAT', "There are no other seats occupied at the table. The table will be closed after separating this seat.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_TABLE_LAST_SEAT', "No hay otros asientos ocupados en la mesa. La mesa se cerrará después de separar este asiento.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_USER', "Fechar Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_USER', "Close Operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_USER', "Cerrar Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CLOSE_USER_VOID_ORDER', "Existe um pedido em aberto, deseja cancelar o pedido para continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CLOSE_USER_VOID_ORDER', "There is an open order, do you want to cancel the order to continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CLOSE_USER_VOID_ORDER', "Hay un pedido abierto, ¿quieres cancelar el pedido para continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COMPLEMENT_DELIVERY_REPORT', "Complemento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COMPLEMENT_DELIVERY_REPORT', "Address 2");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COMPLEMENT_DELIVERY_REPORT', "Complemento de Dirección");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COMPONENT_NOT_FOUND', "Componente {0} não encontrado no PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COMPONENT_NOT_FOUND', "Component {0} not found in POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COMPONENT_NOT_FOUND', "Componente {0} no encontrado en el TDV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS', "Configurações de Notificações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS', "Notifications Settings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS', "Configuracion de notificaciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS_DIALOG_TITLE', "Configurações de Notificações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS_DIALOG_TITLE', "Notifications Settings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS_DIALOG_TITLE', "Configuracion de notificaciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS_LAST_UPDATED', "Última atualização: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS_LAST_UPDATED', "Last update: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS_LAST_UPDATED', "Última actualización: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS_LAST_UPDATED_UNKNOW', "(desconhecido)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS_LAST_UPDATED_UNKNOW', "(unknown)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS_LAST_UPDATED_UNKNOW', "(desconocido)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS_SUCCESSFULLY_UPDATED', "As configurações de alertas foram atualizadas com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS_SUCCESSFULLY_UPDATED', "Alerts settings have been updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS_SUCCESSFULLY_UPDATED', "La configuración de las alertas se ha actualizado correctamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIGURED_ALERTS_UPDATE_FAILED', "Erro ao atualizar as configurações de alertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIGURED_ALERTS_UPDATE_FAILED', "Error updating alerts settings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIGURED_ALERTS_UPDATE_FAILED', "Error al actualizar la configuración de alertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM', "Confirmar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM', "Confirm");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM', "Confirmar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMATION', "Confirmação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMATION', "Confirmation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMATION', "Confirmación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMATION_MESSAGE_NEXT_PRINT_RECEIPT_MERCHANT', "Clique em OK para imprimir a próxima via do comerciante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMATION_MESSAGE_NEXT_PRINT_RECEIPT_MERCHANT', "Click OK to print the next Merchant Via");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMATION_MESSAGE_NEXT_PRINT_RECEIPT_MERCHANT', "Haga clic en Aceptar para imprimir el siguiente comerciante Vía");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMATION_MESSAGE_PRINT_FISCAL_COUPON', "Clique em OK para imprimir o cupom fiscal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMATION_MESSAGE_PRINT_FISCAL_COUPON', "Click OK to print the invoice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMATION_MESSAGE_PRINT_FISCAL_COUPON', "Haga clic en Aceptar para imprimir la factura");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_CUSTOMER', "Clique em OK para imprimir a via do cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_CUSTOMER', "Click OK to print the customer&#39;s copy");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_CUSTOMER', "Haga clic en Aceptar para imprimir la copia del cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_MERCHANT', "Clique em OK para imprimir a via do comerciante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_MERCHANT', "Click OK to print the Merchant Via");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMATION_MESSAGE_PRINT_RECEIPT_MERCHANT', "Haga clic en Aceptar para imprimir el Vía del comerciante");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMS_USER_DISABLEMENT', "Tem certeza que deseja desativar o usuário?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMS_USER_DISABLEMENT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMS_USER_DISABLEMENT', "¿Está seguro que desea desactivar este usuario?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRMS_USER_ENABLE', "Tem certeza que deseja reativar o usuário?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRMS_USER_ENABLE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRMS_USER_ENABLE', "¿Está seguro que desea reactivar este usuario?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_ACTIVE_OPERATOR', "Confirme operador ativo\{0} - {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_ACTIVE_OPERATOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_ACTIVE_OPERATOR', "Confirme operador activo\{0} - {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CHANGE_OPERATOR_TO_SUPERVISOR', "Tem certeza que deseja alterar o perfil de operador para supervisor?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CHANGE_OPERATOR_TO_SUPERVISOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CHANGE_OPERATOR_TO_SUPERVISOR', "¿Está seguro que desea cambiar el perfil de operador para supervisor?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CHANGE_SUPERVISOR_TO_OPERATOR', "Tem certeza que deseja alterar o perfil de supervisor para operador?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CHANGE_SUPERVISOR_TO_OPERATOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CHANGE_SUPERVISOR_TO_OPERATOR', "¿Está seguro que desea cambiar el perfil de supervisor para operador?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLEAN_DISCOUNT', "Descontos já aplicados serão removidos. Continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLEAN_DISCOUNT', "Applied discounts will be removed. Continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLEAN_DISCOUNT', "Los Descuentos aplicados serán removidos. Continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLOSE_DAY', "O dia {0} será fechado para os seguintes PDV:\\{1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLOSE_DAY', "The day {0} will be closed in the following POS:\\{1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLOSE_DAY', "El día {0} será cerrado en el siguiente TPV:\\{1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLOSE_POS', "Você tem certeza que deseja fechar o PDV{0}?\n OBS: Essa operação pode demorar alguns segundos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLOSE_POS', "Are you sure you want to close POS{0}?\n NOTE: This operation may take a few seconds");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLOSE_POS', "¿Está seguro de que desea cerrar el TPV{0}?\n NOTA: Esta operación puede tardar unos segundos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLOSE_TABLE', "Confirmar fechamento de mesa?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLOSE_TABLE', "Confirm table closure?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLOSE_TABLE', "¿Confirmar cierre de mesa?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLOSE_USER', "O usuário {0} será fechado.\\Confirmar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLOSE_USER', "The user {0} will be closed.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLOSE_USER', "El usuario {0} se cerrará.\\Confirma?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CLOSE_USER_FROM_ANOTHER_POS', "Permitir login do operador em outro PDV?\\Pedidos não enviados para produção serão canceladas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CLOSE_USER_FROM_ANOTHER_POS', "Allow operator login in another POS?\\Orders not sent to production will be canceled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CLOSE_USER_FROM_ANOTHER_POS', "¿Permitir el inicio de sesión del operador desde otro TPV?\\Los pedidos no enviados a producción serán cancelados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_CPF_CNPJ', "Deseja utilizar o CPF/CNPJ no cupom fiscal?\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_CPF_CNPJ', "Do you want to use the CPF/CNPJ in the fiscal data?\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_CPF_CNPJ', "PENDING_TRADUCTION_CONFIRM_CPF_CNPJ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_DELIVERY', "Confirmar entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_DELIVERY', "Confirm delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_DELIVERY', "Confirmar pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_DELIVERY_PAYMENT', "Confirmar pagamento do pedido?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_DELIVERY_PAYMENT', "Confirm order payment?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_DELIVERY_PAYMENT', "¿Confirmar pago de pedido?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_EMPTY_CPF_CNPJ', "Confirmar fiscalização sem CPF/CNPJ?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_EMPTY_CPF_CNPJ', "Confirm fiscalization without documents?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_EMPTY_CPF_CNPJ', "Confirmar fiscalización sin documentos?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_MANUAL_ORDER_ADDITION', "O valor do pedido atual, {0},\está diferente do valor vendido no parceiro, {1}.\Confirma a aplicação de adição no valor de {2},\correspondente a diferença de valor?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_MANUAL_ORDER_ADDITION', "CONFIRM_MANUAL_ORDER_ADDITION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_MANUAL_ORDER_ADDITION', "El valor del pedido actual, {0},\es diferente del valor vendido por el socio, {1}.\Confirma la aplicación de la adición de {2} en la suma del valor,\correspondiente a la diferencia de los valores?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_MANUAL_ORDER_DISCOUNT', "O valor do pedido atual, {0},\está diferente do valor vendido no parceiro, {1}.\Confirma a aplicação de desconto no valor de {2},\correspondente a diferença de valor?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_MANUAL_ORDER_DISCOUNT', "CONFIRM_MANUAL_ORDER_DISCOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_MANUAL_ORDER_DISCOUNT', "El valor del pedido actual, {0},\es diferente del valor vendido por el socio, {1}.\Confirma la aplicación del descuento de {2} en la suma del valor,\correspondiente a la diferencia de los valores?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_MANUAL_ORDER_ITEM_QUANTITY_DIFF', "A quantidade de itens na venda, {0},\está diferente da quantidade no parceiro, {1}.\Deseja prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_MANUAL_ORDER_ITEM_QUANTITY_DIFF', "CONFIRM_MANUAL_ORDER_ITEM_QUANTITY_DIFF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_MANUAL_ORDER_ITEM_QUANTITY_DIFF', "La cantidad de artículos en la venta, {0},\es diferente de la cantiad en el socio, {1}.\¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_OPENING_OPERATOR_LOGGED_IN_ANOTHER_POS', "Usuário já logado no POS ({0}). O usuário será deslogado, deseja prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_OPENING_OPERATOR_LOGGED_IN_ANOTHER_POS', "User already logged in at the POS ({0}). User will be logged out, do you want to proceed?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_OPENING_OPERATOR_LOGGED_IN_ANOTHER_POS', "Usuario ya logueado en el TPV ({0}). Se cerrará la sesión del usuario, ¿quieres continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_OPEN_CURRENT_DAY', "Você tem certeza que deseja abrir o dia de negócio {0}?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_OPEN_CURRENT_DAY', "$PENDING_TRADUCTION {0}?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_OPEN_CURRENT_DAY', "¿Está seguo que desea abrir el día laboral {0}?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_OPEN_DAY', "O dia será aberto para os seguintes PDV:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_OPEN_DAY', "The day will be opened in the following POS:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_OPEN_DAY', "El día se abrirá en el siguientes TPV:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_OPEN_USER', "O usuário {0} será aberto.\\Confirmar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_OPEN_USER', "The user {0} will be opened.\\Confirm?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_OPEN_USER', "El usuario {0} se abrirá.\\Confirma?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_ORDER', "Confirmar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_ORDER', "Confirm");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_ORDER', "Confirmar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_PAYMENT_RECEIVED', "Confirma recebimento em {0}: {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_PAYMENT_RECEIVED', "Confirm payment received in {0}: {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_PAYMENT_RECEIVED', "Confirmar pago recibido en {0}: {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_RUPTURE', "Deseja confirmar a ruptura?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_RUPTURE', "Confirm rupture?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_RUPTURE', "¿Confirmar Ruptura?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_RUPTURE_BY_QUANTITY', "Deseja confirmar a ruptura por quantidades?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_RUPTURE_BY_QUANTITY', "Confirm rupture by quantity?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_RUPTURE_BY_QUANTITY', "¿Confirmar ruptura por cantidades?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_SCALE_WEIGHT', "Confirme o peso: {0} KG");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_SCALE_WEIGHT', "Confirm the weight: {0} KG");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_SCALE_WEIGHT', "Confirmar peso: {0} KG");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_SET_AVAILABLE', "Tem certeza que deseja liberar a mesa?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_SET_AVAILABLE', "Are you sure you want to set the table available?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_SET_AVAILABLE', "¿Está seguro de liberar la mesa?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_TRANSFER_TABLE', "Tem certeza que deseja transferir a mesa para {0}?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_TRANSFER_TABLE', "Are you sure you want to transfer the table to {0}?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_TRANSFER_TABLE', "¿Está seguro de transferir la mesa a {0}?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_UPDATE_OPERATORS_BY_SECTOR', "Deseja confirmar a atualização de operadores por setor?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_UPDATE_OPERATORS_BY_SECTOR', "Confirm sector operators update?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_UPDATE_OPERATORS_BY_SECTOR', "¿Confirma la actualización de operadores por sector?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_VOID_LINE', "Deseja apagar a linha?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_VOID_LINE', "Do you want to delete the line?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_VOID_LINE', "¿Quieres eliminar la línea?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONFIRM_VOID_ORDER', "Deseja apagar o pedido?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONFIRM_VOID_ORDER', "Delete order?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONFIRM_VOID_ORDER', "Eliminar orden?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONNECTED', "Ligado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONNECTED', "Connected");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONNECTED', "Conectado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONNECTING_TO_SERVER', "Conectando com o servidor. Por favor aguarde");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONNECTING_TO_SERVER', "Connecting to server. Please wait.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONNECTING_TO_SERVER', "Conectando con el servidor. Por favor espere.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONNECTION_NOT_FOUND', "Loja sem conexão com o Servidor do BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONNECTION_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONNECTION_NOT_FOUND', "Tienda sin conexión con el servidor de BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONNECT_PRINTER_ERROR', "Erro ao conectar com o dispositivo, verifique as configurações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONNECT_PRINTER_ERROR', "Error connecting with device, check settings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONNECT_PRINTER_ERROR', "Error al conectarse con el dispositivo, verifique la configuración");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONSULT_OPEN_TABS', "Consultar Comandas abertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONSULT_OPEN_TABS', "$PENDING_TRADUCTION_CONSULT_OPEN_COMMANDS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONSULT_OPEN_TABS', "Consultar Comandas abiertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CONTINUE_REFUND', "Continuar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CONTINUE_REFUND', "Continue");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CONTINUE_REFUND', "Continuar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COULD_NOT_DESELECT_TABLE', "Mesa/Comanda não pode ser deselecionada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COULD_NOT_DESELECT_TABLE', "Table/Pager cannot be selected");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COULD_NOT_DESELECT_TABLE', "Mesa/Comanda no seleccionada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COULD_NOT_GET_OPERATOR_BY_SECTOR_INFO', "Erro ao consultar operadores por setores.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COULD_NOT_GET_OPERATOR_BY_SECTOR_INFO', "Error getting operators by sectors.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COULD_NOT_GET_OPERATOR_BY_SECTOR_INFO', "Error al consultar operadores por sectores.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COULD_NOT_GET_SECTORS_FROM_STORE', "Erro ao consultar setores cadastrados no restaurante.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COULD_NOT_GET_SECTORS_FROM_STORE', "Error getting registered sectors in restaurant.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COULD_NOT_GET_SECTORS_FROM_STORE', "Error al consultar los sectores registrados del restaurante.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON', "Cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON', "Cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON', "Cupon");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_CANCELED', "Cupom Cancelado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_CANCELED', "PENDING_TRADUCTION_COUPON_CANCELED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_CANCELED', "Cupón Cancelado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_DISCOUNT_DELIVERY_FEE_IFOOD', "Desconto Taxa de Entrega iFood:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_DISCOUNT_DELIVERY_FEE_IFOOD', "iFood Delivery Fee Discount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_DISCOUNT_DELIVERY_FEE_IFOOD', "Descuento de entrega Gratis iFood:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_DISCOUNT_DELIVERY_FEE_MERCHANT', "Desconto Taxa de Entrega Restaurante:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_DISCOUNT_DELIVERY_FEE_MERCHANT', "Delivery fee store discount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_DISCOUNT_DELIVERY_FEE_MERCHANT', "Tarifa de entrega de restaurante con Descuento:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_DISCOUNT_DELIVERY_FEE_PARTNER', "Desconto Taxa de Entrega do Parceiro:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_DISCOUNT_DELIVERY_FEE_PARTNER', "PENDING_TRADUCTION_COUPON_DISCOUNT_DELIVERY_FEE_PARTNER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_DISCOUNT_DELIVERY_FEE_PARTNER', "PENDING_TRADUCTION_COUPON_DISCOUNT_DELIVERY_FEE_PARTNER");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_DISCOUNT_IFOOD', "Desconto iFood:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_DISCOUNT_IFOOD', "iFood Discount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_DISCOUNT_IFOOD', "Descuento iFood:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_DISCOUNT_MERCHANT', "Desconto Restaurante:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_DISCOUNT_MERCHANT', "Store Discount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_DISCOUNT_MERCHANT', "Descuento de Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_ERROR', "Erro de cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_ERROR', "Erro de cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_ERROR', "Error de cupon");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'COUPON_PREREQ_FAILED', "Nem todos requisitos para o cupom [{0}] foram satisfeitos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'COUPON_PREREQ_FAILED', "Not all requirements for the [{0}] coupon were met");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'COUPON_PREREQ_FAILED', "No se reconocieron todos los requisitos para el cupon [{0}]");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CPF_CNPJ', "CPF/CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CPF_CNPJ', "CPF/CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CPF_CNPJ', "CPF/CNPJ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATE_CHALLENGE', "Criar Desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATE_CHALLENGE', "PENDING_TRADUCTION_CREATE_CHALLENGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATE_CHALLENGE', "Crear Desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATE_FISCAL_XML', "Criar XML Fiscal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATE_FISCAL_XML', "Create Fiscal XML");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATE_FISCAL_XML', "Crear XML Fiscal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATE_NEW', "Criar Novo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATE_NEW', "PENDING_TRADUCTION_CREATE_NEW");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATE_NEW', "Crear Nuevo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATE_TAB', "Abrir Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATE_TAB', "Open Pager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATE_TAB', "Abrir Comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATING_MENU_BACKUP', "Criando backup do Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATING_MENU_BACKUP', "CREATING_MENU_BACKUP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATING_MENU_BACKUP', "Creando backup del Menú");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREATING_POS_BACKUP', "Criando backup do PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREATING_POS_BACKUP', "Creating POS backup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREATING_POS_BACKUP', "Creando backup del TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CREDIT_CARD', "Cartão de Crédito");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CREDIT_CARD', "Credit Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CREDIT_CARD', "Tarjeta de Crédito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER', "Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER', "Customer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER', "Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_DATA', "Dados do Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_DATA', "Customer Data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_DATA', "Datos del Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_DOC', "Documento Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_DOC', "Customer Document");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_DOC', "Documento del Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_ERROR_ON_UPDATE', "Erro ao atualizar dados do Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_ERROR_ON_UPDATE', "Error adding new customer.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_ERROR_ON_UPDATE', "Error al añadir nuevo cliente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_INFO', "Informações do cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_INFO', "Customer info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_INFO', "Información del Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_NAME', "Nome Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_NAME', "Customer name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_NAME', "Nombre de Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_NAME_DELIVERY_REPORT', "Nome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_NAME_DELIVERY_REPORT', "Name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_NAME_DELIVERY_REPORT', "Nombre");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_NOT_FOUND', "Cliente não encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_NOT_FOUND', "Customer not found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_NOT_FOUND', "Cliente no encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_WILL_BE_INSERTED', "Novo cliente detectado. Deseja adicioná-lo à base de clientes?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_WILL_BE_INSERTED', "New customer. Add it?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_WILL_BE_INSERTED', "Se ha encontrado un nuevo cliente. ¿Desea añadirlo?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'CUSTOMER_WILL_BE_UPDATED', "Encontradas alterações no cadastro do cliente. Deseja atualizá-lo?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'CUSTOMER_WILL_BE_UPDATED', "Customer modified update it?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'CUSTOMER_WILL_BE_UPDATED', "Se ha modificado un cliente. ¿Desea actualizarlo?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_DISHES_QUANTITY_ZERO_INSERTED', "Quantidade zero inserida, os produtos selecionados entrarão em ruptura. Deseja prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_DISHES_QUANTITY_ZERO_INSERTED', "Zero quantity inserted, selected product will enter in rupture. Do you want to proceed?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_DISHES_QUANTITY_ZERO_INSERTED', "Cantidad cero ingresada, los productos seleccionado entrarán en ruptura. ¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_DISHES_UPDATE_ERROR', "Erro na atualização de quantidade de pratos do dia.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_DISHES_UPDATE_ERROR', "Error updating daily dishes quantity.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_DISHES_UPDATE_ERROR', "Error actualizando la cantidad de platos del día.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_DISHES_UPDATE_SUCCESS', "Quantidade de pratos do dia atualizado com sucesso. \\ Em alguns instantes o cardápio será atualizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_DISHES_UPDATE_SUCCESS', "Daily dishes quantity successfully updated. \\ The catalog will be updated in a while");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_DISHES_UPDATE_SUCCESS', "Cantidad de platos del día actualizados con éxito. En un momento el menú será actualizado.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_GOALS', "Metas diárias");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_GOALS', "Daily Goal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_GOALS', "Metas Diarias");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_GOALS_BY_AMOUNT', "Meta por valor vendido (%)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_GOALS_BY_AMOUNT', "Daily goal by value (%)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_GOALS_BY_AMOUNT', "Meta diaria por valor (%)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_GOALS_STORE', "Meta diária de vendas da loja (%)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_GOALS_STORE', "Daily goal (%)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_GOALS_STORE', "Meta Diaria (%)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_GOALS_STORE_BY_AVERAGE_TICKET', "Meta por ticket médio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_GOALS_STORE_BY_AVERAGE_TICKET', "Daily goal by AT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_GOALS_STORE_BY_AVERAGE_TICKET', "Meta diaria por ticket");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAILY_GOALS_STORE_BY_CATEGORY', "Meta por quantidade vendida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAILY_GOALS_STORE_BY_CATEGORY', "Daily goal by qtt (%)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAILY_GOALS_STORE_BY_CATEGORY', "Meta diaria por cantidad (%)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DASHBOARD', "Painel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DASHBOARD', "Dashboard");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DASHBOARD', "Panel");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DATE', "Hora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DATE', "Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DATE', "Hora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DATE_DELIVERY_REPORT', "Data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DATE_DELIVERY_REPORT', "Date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DATE_DELIVERY_REPORT', "Fecha");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DATE_TIME', "Data/Hora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DATE_TIME', "Date/Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DATE_TIME', "Fecha/Hora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAY_IS_ALREADY_CLOSED', "Dia já está fechado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAY_IS_ALREADY_CLOSED', "The day is already closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAY_IS_ALREADY_CLOSED', "El día ya está cerrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAY_IS_ALREADY_OPENED', "Dia já está aberto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAY_IS_ALREADY_OPENED', "The day is already opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAY_IS_ALREADY_OPENED', "El día ya está abierto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAY_IS_CLOSED', "O dia está fechado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAY_IS_CLOSED', "The day is closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAY_IS_CLOSED', "El día está cerrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAY_IS_NOT_CLOSED', "Dia não está fechado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAY_IS_NOT_CLOSED', "The day is not closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAY_IS_NOT_CLOSED', "El día no está cerrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DAY_QUANTITIES', "Quantidades do Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DAY_QUANTITIES', "Day Quantities");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DAY_QUANTITIES', "Cantidades del Dia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEBIT_CARD', "Cartão de Débito");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEBIT_CARD', "Debit Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEBIT_CARD', "Tarjeta de Débito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEBIT_CARD_NOT_PROCESSED', "Não foi possível processar o cartão de débito.\Resposta do servidor: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEBIT_CARD_NOT_PROCESSED', "The debit card could not be processed.\Server response: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEBIT_CARD_NOT_PROCESSED', "La tarjeta de débito no pudo ser procesada.\Respuesta del servidor: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEBIT_CARD_VOUCHER_CARD', "Débito ou Vale Refeição");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEBIT_CARD_VOUCHER_CARD', "Debit Card or Voucher");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEBIT_CARD_VOUCHER_CARD', "Débito o Tarjeta Alimentación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETE', "Apagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETE', "Eliminar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETED_SAVED_ORDERS', "Pedidos salvos apagados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETED_SAVED_ORDERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETED_SAVED_ORDERS', "Pedidos guardados eliminados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETE_LAST', "Apagar Último");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETE_LAST', "Delete Last");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETE_LAST', "Borrar Último");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETE_LINE', "Deletar Linha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETE_LINE', "Delete Line");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETE_LINE', "Eliminar Línea");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETE_SALE_CHALLENGE_CONFIRMATION', "Tem certeza que deseja deletar o desafio de venda?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETE_SALE_CHALLENGE_CONFIRMATION', "PENDING_TRADUCTION_SALE_CHALLENGE_DELETE_SALE_CHALLENGE_CONFIRMATION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETE_SALE_CHALLENGE_CONFIRMATION', "¿Está seguro que desea eliminar el desafío de venta?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELETE_USER', "Tem certeza que deseja excluir o usuário?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELETE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELETE_USER', "¿Está seguro que desea eliminar el usuario?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY', "Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERYMEN', "Entregadores");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERYMEN', "Delivery people");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERYMEN', "Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ADDITIONAL_FEE_DELIVERY_REPORT', "Taxa Adicional");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ADDITIONAL_FEE_DELIVERY_REPORT', "Additional Fee");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ADDITIONAL_FEE_DELIVERY_REPORT', "Tarifa adicional");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ADDRESS_FORM_ERRORS', "Existem campos obrigatórios que devem ser preenchidos antes de prosseguir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ADDRESS_FORM_ERRORS', "There are mandatory fields that must be completed before proceeding");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ADDRESS_FORM_ERRORS', "Hay campos obligatorios que deben completarse antes de continuar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ARRANGEMENT', "Acerto Entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ARRANGEMENT', "PENDING_TRADUCTION_DELIVERY_ARRANGEMENT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ARRANGEMENT', "Arreglo con Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_CONFIRM_ERROR_NOT_PAID', "Valor do pagamento menor que o valor total do pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_CONFIRM_ERROR_NOT_PAID', "Payment due");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_CONFIRM_ERROR_NOT_PAID', "Valor de pago menor que el valor total del pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_CONTROL', "Controle de Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_CONTROL', "Delivery Control");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_CONTROL', "Control de Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_DELIVERY', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_DELIVERY', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_DELIVERY', "Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_DRIVE_THRU', "Retirada Drive");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_DRIVE_THRU', "Take out Drive Thru");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_DRIVE_THRU', "Drive Thru");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_EAT_IN', "Retirada Balcão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_EAT_IN', "Take out Counter");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_EAT_IN', "Contador de Para Llevar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_EAT_OUT', "Retirada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_EAT_OUT', "Take out");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_EAT_OUT', "Para Llevar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR', "Erro:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR', "Error:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR', "Error");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_COMPOSITIONTREEEXCEPTION', "Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_COMPOSITIONTREEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_COMPOSITIONTREEEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_COMPOSITIONTREEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_COMPOSITIONTREEEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_FISCALEXCEPTION', "Fiscalização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_FISCALEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_FISCALEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_FISCALEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_FISCALEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_INVALIDJSONEXCEPTION', "Validação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_INVALIDJSONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDJSONEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_INVALIDJSONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDJSONEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_INVALIDORDEREXCEPTION', "Produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_INVALIDORDEREXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDORDEREXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_INVALIDORDEREXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDORDEREXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_INVALIDQUANTITYEXCEPTION', "Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_INVALIDQUANTITYEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDQUANTITYEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_INVALIDQUANTITYEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDQUANTITYEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_INVALIDSONEXCEPTION', "Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_INVALIDSONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDSONEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_INVALIDSONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDSONEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_INVALIDVALUEEXCEPTION', "Preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_INVALIDVALUEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDVALUEEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_INVALIDVALUEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_INVALIDVALUEEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_ORDEREXCEPTION', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_ORDEREXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_ORDEREXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_ORDEREXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_ORDEREXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_ORDERVALIDATIONEXCEPTION', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_ORDERVALIDATIONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_ORDERVALIDATIONEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_ORDERVALIDATIONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_ORDERVALIDATIONEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_PAYMENTEXCEPTION', "Pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_PAYMENTEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_PAYMENTEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_PAYMENTEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_PAYMENTEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_PRODUCTNOTFOUND', "Produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_PRODUCTNOTFOUND', "Product");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_PRODUCTNOTFOUND', "Producto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_PRODUCTUNAVAILABLEEXCEPTION', "Ruptura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_PRODUCTUNAVAILABLEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_PRODUCTUNAVAILABLEEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_PRODUCTUNAVAILABLEEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_PRODUCTUNAVAILABLEEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_UNKNOWN', "Desconhecido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_UNKNOWN', "PENDING_TRADUCTION_DELIVERY_ERROR_UNKNOWN");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_UNKNOWN', "PENDING_TRADUCTION_DELIVERY_ERROR_UNKNOWN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_ERROR_VALIDATIONEXCEPTION', "Validação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_ERROR_VALIDATIONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_VALIDATIONEXCEPTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_ERROR_VALIDATIONEXCEPTION', "PENDING_TRADUCTION_DELIVERY_ERROR_VALIDATIONEXCEPTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_FEE', "Taxa Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_FEE', "Delivery Fee");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_FEE', "Entrega Gratis");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_FEE_DELIVERY_REPORT', "Taxa de Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_FEE_DELIVERY_REPORT', "Delivery Fee");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_FEE_DELIVERY_REPORT', "Impuesto de Entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_FEE_DISCOUNT_DELIVERY_REPORT', "Desconto da Taxa de Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_FEE_DISCOUNT_DELIVERY_REPORT', "Delivery fee discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_FEE_DISCOUNT_DELIVERY_REPORT', "Descuento en la tarifa de envío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_FEE_VALUE', "Taxa de entrega:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_FEE_VALUE', "Delivery fee:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_FEE_VALUE', "Impuesto de Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_INFO', "Informações de Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_INFO', "Delivery Info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_INFO', "Información de Entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_INTEGRATION_ERROR', "ERRO NA INTEGRAÇÃO CONSULTE PARCEIRO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_INTEGRATION_ERROR', "INTEGRATION ERROR CONSULT PARTNER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_INTEGRATION_ERROR', "ERROR DE INTEGARACIÓN. CONSULTE CON SOCIO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_MAN', "Entregador:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_MAN', "Delivery person:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_MAN', "Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_MAN_NAME', "Nome do entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_MAN_NAME', "Delivery person name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_MAN_NAME', "Nombre de Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_MAN_NAME_ARRANGEMENT', "Nome do entregador: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_MAN_NAME_ARRANGEMENT', "Delivery person name: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_MAN_NAME_ARRANGEMENT', "Nombre de Repartidor: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_MAN_TIME', "Horário de saída para entrega:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_MAN_TIME', "Leave time:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_MAN_TIME', "Horario de salida para entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_PED', "Pedido:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_PED', "Order:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_PED', "Pedido:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_RECALL_WARNING', "O pedido [{0}] do [{1}] possui {2} item(s)    no valor total de {3}   o lançamento não bate com o pedido original     gostaria de prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_RECALL_WARNING', "The order [{0}] from [{1}] has {2} item(s)    at a total cost of {3}  the local order does not equal the original order   do you want to proceed?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_RECALL_WARNING', "El pedido [{0}] de [{1}] posee {2} artículo(s)    al costo total de {3}   la orden local no coincide con la orden del pedido original     ¿desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_STORE', "Loja Delivery:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_STORE', "Delivery Store:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_STORE', "Tienda Delivery:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_TAKE_OUT', "Retirada Balcão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_TAKE_OUT', "Take out Counter");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_TAKE_OUT', "Conador de Para Llevar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_TYPE', "Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_TYPE', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_TYPE', "Entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVERY_UNKNOWN', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVERY_UNKNOWN', "Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVERY_UNKNOWN', "Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DELIVER_AT_THE_COUNTER', "Entregar no balcão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DELIVER_AT_THE_COUNTER', "Deliver at the counter");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DELIVER_AT_THE_COUNTER', "Entregar en el mostrador");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DESCRIPTION', "Descrição");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DESCRIPTION', "Description");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DESCRIPTION', "Descripción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DESCRIPTION_DELIVERY_REPORT', "Descrição");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DESCRIPTION_DELIVERY_REPORT', "Description");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DESCRIPTION_DELIVERY_REPORT', "Descripción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DESELECT_TABLE', "Deselecionar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DESELECT_TABLE', "Deselect Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DESELECT_TABLE', "Deseleccionar Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DESSERTS', "Sobremesas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DESSERTS', "Desserts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DESSERTS', "Postres");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DESTINY_TABLE_MODE', "Escolha o modo de serviço da mesa destino:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DESTINY_TABLE_MODE', "Choose destiny table service mode:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DESTINY_TABLE_MODE', "Escoja el modo de servicio de la mesa de destino:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEVELOPER_DELIVERY_REPORT', "Desenvolvido por: E-Deploy");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEVELOPER_DELIVERY_REPORT', "Developed by: E-DEPLOY");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEVELOPER_DELIVERY_REPORT', "Desarrollado por: E-DEPLOY");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT', "www.e-deploy.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT', "www.e-deploy.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT', "www.e-deploy.com.br");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT_3S', "www.3scheckout.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT_3S', "www.3scheckout.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEVELOPER_WEB_SITE_DELIVERY_REPORT_3S', "www.3scheckout.com.br");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEVICE_MANAGER', "Gerenciador de Dispositivos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEVICE_MANAGER', "Device Manager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEVICE_MANAGER', "Gerenciador de dispositivos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DEVICE_NOT_FOUND', "Dispositivo não encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DEVICE_NOT_FOUND', "Device not found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DEVICE_NOT_FOUND', "Dispositivo no encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DIFFERENT_TABLE_MODE_IN_SOURCE_AND_DESTINATION', "O modo de serviço da mesa de origem não pode ser diferente do modo de serviço da mesa destino");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DIFFERENT_TABLE_MODE_IN_SOURCE_AND_DESTINATION', "Source table service mode cannot be different from destination table service mode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DIFFERENT_TABLE_MODE_IN_SOURCE_AND_DESTINATION', "El modo de servicio de la mesa de origen no puede ser diferente del modo de servicio de la mesa de destino");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DIGITAL_WALLET', "Carteira Digital");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DIGITAL_WALLET', "Digital Wallet");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DIGITAL_WALLET', "Billetera Digital");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCONNECTED', "Desligado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCONNECTED', "Off");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCONNECTED', "Apagado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT', "Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT', "Discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT', "Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNTS', "Descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNTS', "Discounts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNTS', "Descuentos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNTS_CLEANED', "Descontos limpos com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNTS_CLEANED', "Discounts cleared");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNTS_CLEANED', "Descuentos borrados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_COUPON', "Cupom de desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_COUPON', "Discount coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_COUPON', "cupón de descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_JUSTIFICATION', "Desconto:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_JUSTIFICATION', "Discount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_JUSTIFICATION', "Descuento:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_NOT_FOUND', "Não existem descontos configurados!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_NOT_FOUND', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_NOT_FOUND', "¡No existen descuentos configurados!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_PERCENTAGE_TITLE', "Digite a porcentagem de desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_PERCENTAGE_TITLE', "Inform the discount rate");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_PERCENTAGE_TITLE', "Ingrese el porcentaje de descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_REPORT', "Relatório de descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_REPORT', "Discount Reports");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_REPORT', "Reporte de Descuentos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISCOUNT_VALUE_TITLE', "Digite o valor do desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISCOUNT_VALUE_TITLE', "Inform discount amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISCOUNT_VALUE_TITLE', "Informar descuento de monto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DISTRIBUTE_ITEMS_IN_SEATS', "Gostaria de distribuir os itens nos assentos?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DISTRIBUTE_ITEMS_IN_SEATS', "Would you like to distribute the items on the seats?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DISTRIBUTE_ITEMS_IN_SEATS', "¿Te gustaría distribuir los artículos en los asientos?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DIVISION_FOR_SEATS', "Divisão Assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DIVISION_FOR_SEATS', "PENDING_TRADUCTION_DIVISION_FOR_SEATS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DIVISION_FOR_SEATS', "División Asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DIVISION_FROM_TABLE', "Divisão Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DIVISION_FROM_TABLE', "PENDING_TRADUCTION_DIVISION_FROM_TABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DIVISION_FROM_TABLE', "División Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DIVISION_SUGESTION_FOR', "Divisão para");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DIVISION_SUGESTION_FOR', "Division for");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DIVISION_SUGESTION_FOR', "División para");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DLV_CHANGE', "Troco necessário:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DLV_CHANGE', "Change:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DLV_CHANGE', "Cambio:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DLV_CONFIRMED', "DELIVERY\CONCLUÍDOS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DLV_CONFIRMED', "DELIVERY/CLOSED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DLV_CONFIRMED', "DELIVERY/CERRADOS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DLV_CUSTOMER_NAME', "Nome do Cliente:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DLV_CUSTOMER_NAME', "Customer Name:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DLV_CUSTOMER_NAME', "Nombre de Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DLV_ERROR', "DELIVERY\ERROS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DLV_ERROR', "DELIVERY/ERRORS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DLV_ERROR', "DELIVERY/ERRORES");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DLV_OR_PICKUP', "Modo de entrega:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DLV_OR_PICKUP', "Delivery method:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DLV_OR_PICKUP', "Método de Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOCUMENT_DELIVERY_REPORT', "CPF na Nota Fiscal:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOCUMENT_DELIVERY_REPORT', "Document:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOCUMENT_DELIVERY_REPORT', "Documento:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOC_REQUIRED', "Para vendas com valor igual ou superior de R$ {0} é obrigatório informar o CPF/CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOC_REQUIRED', "$PENDING_TRADUCTION_DOC_REQUIREDE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOC_REQUIRED', "$PENDING_TRADUCTION_DOC_REQUIRED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DONATION_CNPJ', "CNPJ. {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DONATION_CNPJ', "CNPJ. {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DONATION_CNPJ', "CNPJ. {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DONATION_INSTITUTION', "Doação - {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DONATION_INSTITUTION', "Donation - {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DONATION_INSTITUTION', "Donación - {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DONATION_VALUE', "VALOR DA DOAÇÃO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DONATION_VALUE', "DONATION VALUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DONATION_VALUE', "VALOR DE DONACIÓN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DONE', "Feito");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DONE', "Done");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DONE', "Hecho");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DONT_MAKE', "Não Fazer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DONT_MAKE', "Don''t Make");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DONT_MAKE', "No hacer");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOOR_OPENED_ERROR', "A porta da impressora está aberta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOOR_OPENED_ERROR', "Printer door is open");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOOR_OPENED_ERROR', "La puerta de la impresora está abierta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOWNLOADING_CORE_FROM_BACKOFFICE', "Baixando CORE do BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOWNLOADING_CORE_FROM_BACKOFFICE', "$PENDING_TRADUCTION_DOWNLOADING_CORE_FROM_BACKOFFICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOWNLOADING_CORE_FROM_BACKOFFICE', "Descargando CORE de BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOWNLOADING_SRC_FROM_BACKOFFICE', "Baixando SRC do BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOWNLOADING_SRC_FROM_BACKOFFICE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOWNLOADING_SRC_FROM_BACKOFFICE', "Descargando SRC de BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DOWNLOAD_IMAGES_AND_CATEGORY', "Baixando imagens de produtos e categorias");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DOWNLOAD_IMAGES_AND_CATEGORY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DOWNLOAD_IMAGES_AND_CATEGORY', "Bajando imágenes de productos y categorías");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DO_TOTAL', "Totalizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DO_TOTAL', "Total Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DO_TOTAL', "Total de Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DRIVE', "DRIVE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DRIVE', "DRIVE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DRIVE', "DRIVE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DRIVE_LOWER', "Drive");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DRIVE_LOWER', "Drive");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DRIVE_LOWER', "Driver");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'DUPLICATE', "Duplicado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'DUPLICATE', "Duplicate");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'DUPLICATE', "Duplicado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EAT_IN', "Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EAT_IN', "Eat In");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EAT_IN', "Comer aquí");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EDIT_COMMENT', "Editar Instrução");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EDIT_COMMENT', "Edit comment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EDIT_COMMENT', "Editar comentario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EDIT_ORDER', "Alterar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EDIT_ORDER', "Change");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EDIT_ORDER', "Cambiar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EIGHTH_READING', "Oitava");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EIGHTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EIGHTH_READING', "Octava");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ELECTRONIC_PAYMENT', "Pagamento Eletrônico");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ELECTRONIC_PAYMENT', "Electronic Payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ELECTRONIC_PAYMENT', "Pago Electrónico");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ELECTRONIC_PAYMENTS', "Pagamentos eletrônicos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ELECTRONIC_PAYMENTS', "$PENDING_TRADUCTION_ELECTRONIC_PAYMENTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ELECTRONIC_PAYMENTS', "Pagos electrónicos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ELEVENTH_READING', "Décima Primeira");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ELEVENTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ELEVENTH_READING', "Decimoprimera");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EMPTY_CART_MESSAGE', "Toque em uma de nossas categorias de produtos para começar o seu pedido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EMPTY_CART_MESSAGE', "Touch on one of the categories to start your order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EMPTY_CART_MESSAGE', "Presione en una de las categorías para comenzar a ordenar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EMPTY_OPENED_TABS', "Não existem comandas abertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EMPTY_OPENED_TABS', "$PENDING_TRADUCTION_EMPTY_OPENED_TABS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EMPTY_OPENED_TABS', "No existen comandas abiertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'END_DATE', "Data Fim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'END_DATE', "PENDING_TRADUCTION_END_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'END_DATE', "Fecha Fin");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'END_PAYMENT_TEXT_1', "Seu pedido já está em preparo.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'END_PAYMENT_TEXT_1', "Your order is already in preparation.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'END_PAYMENT_TEXT_1', "Su orden está en preparación.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'END_PAYMENT_TEXT_2', "Retire sua nota");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'END_PAYMENT_TEXT_2', "Remove your receipt");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'END_PAYMENT_TEXT_2', "Retire su recibo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'END_PAYMENT_TEXT_3', "e aguarde que chamaremos seu nome.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'END_PAYMENT_TEXT_3', "and wait for us to call your name.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'END_PAYMENT_TEXT_3', "y espere a que nosotros llamemos su nombre.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_AMOUNT', "Digite o valor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_AMOUNT', "Inform a value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_AMOUNT', "Informar un valor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_AMOUNT_TO_TRANSFER_TYPE', "Digite o valor para fazer a {0}:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_AMOUNT_TO_TRANSFER_TYPE', "Inform amount to transfer to {0}:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_AMOUNT_TO_TRANSFER_TYPE', "Informar monto a transferir a {0}:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_CHALLENGE_NAME', "Digite o nome do desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_CHALLENGE_NAME', "PENDING_TRADUCTION_ENTER_CHALLENGE_NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_CHALLENGE_NAME', "Ingrese el nombre del desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_CLIENT_ADDRESS', "Digite o endereço do cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_CLIENT_ADDRESS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_CLIENT_ADDRESS', "Ingrese la dirección del cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_COUPON_CODE', "Digite o código do cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_COUPON_CODE', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_COUPON_CODE', "Ingrese el código del cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_CPF_CNPJ', "Digite o CPF/CNPJ:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_CPF_CNPJ', "Enter Document");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_CPF_CNPJ', "Ingrese Documento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_DATE_OR_PRESS_OK', "Digite a data ou pressione 'Ok' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_DATE_OR_PRESS_OK', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_DATE_OR_PRESS_OK', "Ingrese la fecha o presione 'Ok' para fecha actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_END_DATE_OR_OK_FOR_CURRENT_DATE', "Digite a data fim  ou pressione 'Ok' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_END_DATE_OR_OK_FOR_CURRENT_DATE', "PENDING_TRADUCTION_ENTER_END_DATE_OR_OK_FOR_CURRENT_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_END_DATE_OR_OK_FOR_CURRENT_DATE', "Ingrese la fecha final o presione 'Ok' para fecha actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_ID_FOR_NEW_USER', "Digite o Id para novo usuário (máx. 8 dígitos)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_ID_FOR_NEW_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_ID_FOR_NEW_USER', "Ingrese el ID para el nuevo usuario (máx. 8 dígitos)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_LOYALTY_ID', "Insira o número de fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_LOYALTY_ID', "Enter loyalty number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_LOYALTY_ID', "Introduce el número de fidelización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_NAME_NEW_USER', "Digite o nome do novo usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_NAME_NEW_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_NAME_NEW_USER', "Ingrese el nombre del nuevo usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_NEW_PASSWORD', "Digite a nova senha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_NEW_PASSWORD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_NEW_PASSWORD', "Ingrese nueva contraseña");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_PAGER_NUMBER', "Digite o número do Pager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_PAGER_NUMBER', "Enter pager number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_PAGER_NUMBER', "Ingrese el número de localizador");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_PASSWORD', "Digite a senha do usuário {0}:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_PASSWORD', "Enter the password for {0}:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_PASSWORD', "Ingrese la contraseña para {0}:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_PASSWORD_TO_REGISTER', "Digite a senha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_PASSWORD_TO_REGISTER', "Enter the password");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_PASSWORD_TO_REGISTER', "Ingrese la contraseña");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_PHONE', "Insiria o número do telefone");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_PHONE', "Enter phone number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_PHONE', "Ingresa número telefónico");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_RUPTURE', "Entrar em ruptura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_RUPTURE', "Put in rupture");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_RUPTURE', "Poner en ruptura");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_START_DATE_OR_OK_FOR_CURRENT_DATE', "Digite a data inicial  ou pressione 'Ok' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_START_DATE_OR_OK_FOR_CURRENT_DATE', "PENDING_TRADUCTION_ENTER_START_DATE_OR_OK_FOR_CURRENT_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_START_DATE_OR_OK_FOR_CURRENT_DATE', "Ingrese la fecha inicial o presione 'Ok' para fecha actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_TAB_NUMBER', "Entre com os últimos 3 dígitos da ficha de consumo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_TAB_NUMBER', "Enter the last 3 digits of the consumption card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_TAB_NUMBER', "Escriba los 3 últimos dígitos de la tarjeta de consumo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_AMOUNT', "Digite a quantidade:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_AMOUNT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_AMOUNT', "Ingrese la cantidad:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_AMOUNT_AVAILABLE', "Digite a quantidade disponível para venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_AMOUNT_AVAILABLE', "Enter quantity available for sale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_AMOUNT_AVAILABLE', "Ingrese la cantidad disponible para la venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_DAY', "Digite a data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_DAY', "Inform date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_DAY', "Ingrese la Fecha");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_INITIAL_FLOAT_AMOUNT', "Entre com o fundo de troco:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_INITIAL_FLOAT_AMOUNT', "Inform change value:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_INITIAL_FLOAT_AMOUNT', "Informar el valor de cambio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_ORDER_ID', "Digite o order ID");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_ORDER_ID', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_ORDER_ID', "Ingrese el order ID");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ENTER_THE_SECTOR', "Entrarão no {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ENTER_THE_SECTOR', "Enter the {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ENTER_THE_SECTOR', "Ingresarán a {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EQUAL_DIVISION_FOR_ALL_SEATS', "Divisão igual para os assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EQUAL_DIVISION_FOR_ALL_SEATS', "PENDING_TRADUCTION_EQUAL_DIVISION_FOR_ALL_SEATS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EQUAL_DIVISION_FOR_ALL_SEATS', "División igual para los asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR', "Erro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR', "Error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR', "Error");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_ABANDON_TABLE', "Ocorreu um erro no processo de abandono e a mesa continuará aberta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_ABANDON_TABLE', "An error occurred in the abandonment process and the table will remain open");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_ABANDON_TABLE', "Ocurrió un error en el proceso de abandono y la mesa permanecerá abierta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_EFT_TRANSACTION', "Não foi possível cancelar o pagamento eletrônico.\Deseja cancelar o pagamento no PDV?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_EFT_TRANSACTION', "Unable to cancel the electronic payment.\Would you like to cancel the payment at the POS?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_EFT_TRANSACTION', "No se puede cancelar el pago electrónico.\n¿Quieres cancelar el pago en el TDV?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_EFT_TRANSACTION_RETRY', "Não foi possível cancelar o pagamento eletrônico.\Deseja tentar novamente, ou continuar sem o estorno?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_EFT_TRANSACTION_RETRY', "Unable to cancel electronic payment.\Do you want to try again, or continue without reversal?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_EFT_TRANSACTION_RETRY', "No se puede cancelar el pago electrónico.\¿Quieres volver a intentarlo o continuar sin revertirlo?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_EFT_TRANSACTION_STOP_PROGRESS', "Erro cancelando pagamento eletrônico");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_EFT_TRANSACTION_STOP_PROGRESS', "Error canceling electronic payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_EFT_TRANSACTION_STOP_PROGRESS', "Error al cancelar el pago electrónico");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_ELECTRONIC_PAYMENTS_ORDER', "Erro ao cancelar o pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_ELECTRONIC_PAYMENTS_ORDER', "Error canceling payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_ELECTRONIC_PAYMENTS_ORDER', "Error al cancelar pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_FISCAL_ORDER', "Erro ao cancelar pedido fiscal:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_FISCAL_ORDER', "Error canceling fiscal order:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_FISCAL_ORDER', "Error al cancelar orden fiscal:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_ORDER', "Erro ao cancelar pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_ORDER', "Error voiding order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_ORDER', "Error al cancelar orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_ORDER_ANOTHER_EFT_PROCESSOR', "Esse pedido só pode ser cancelado em um PDV {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_ORDER_ANOTHER_EFT_PROCESSOR', "This order can only be canceled at a POS {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_ORDER_ANOTHER_EFT_PROCESSOR', "Este pedido solo se puede cancelar en un TDV {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CANCEL_ORDER_INVALID_POS_CONFIGURATION', "Erro ao cancelar pedido neste PDV. Tente novamente em outro terminal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CANCEL_ORDER_INVALID_POS_CONFIGURATION', "Error canceling order at this POS. Try again in another terminal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CANCEL_ORDER_INVALID_POS_CONFIGURATION', "Error al cancelar pedido en este TPV. Inténtalo de nuevo en otra terminal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHANGE_PROFILE', "Ocorreu um erro ao alterar o perfil do usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHANGE_PROFILE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHANGE_PROFILE', "¡Ocurrió un error al alterar el perfil del usuario!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHANGING_OPERATOR', "Não existem outros garçons disponíveis");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHANGING_OPERATOR', "There is no available waiters");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHANGING_OPERATOR', "No hay meseros disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHANGING_PASSWORD', "Ocorreu um erro ao alterar a senha!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHANGING_PASSWORD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHANGING_PASSWORD', "¡Ocurrió un error al cambiar a contraseña!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHANGING_REMOTE_ORDER_STORE_STATUS', "Erro alterando status da loja remota:\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHANGING_REMOTE_ORDER_STORE_STATUS', "Error changing remote state:\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHANGING_REMOTE_ORDER_STORE_STATUS', "Error al cambiar estado remoto:\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHANGING_SEATS', "Erro mudando quantidade de assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHANGING_SEATS', "Error changing seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHANGING_SEATS', "Error al cambiar asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CHECKING_READE_FINGERPRINT', "Erro verificando leitor!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CHECKING_READE_FINGERPRINT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CHECKING_READE_FINGERPRINT', "¡Error verificando lector!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CLEANING_KDS', "Erro ao limpar KDSs");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CLEANING_KDS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CLEANING_KDS', "Error al limpiar KDSs");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CLOSING_DAY', "Erro ao processar fechamento de dia:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CLOSING_DAY', "Error closing the day:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CLOSING_DAY', "Error al cerrar día:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CLOSING_DAYS', "Erro ao fechar o dia para os seguintes PDV:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CLOSING_DAYS', "Error closing the day in the following POS:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CLOSING_DAYS', "Error al cerrar día en el siguiente TPV:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CLOSING_DELIVERY_STORE', "Erro fechando loja do Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CLOSING_DELIVERY_STORE', "Error closing Delivery Store");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CLOSING_DELIVERY_STORE', "Error al cerrar Tienda Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CONFIRMING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "Erro interno confirmando pagamentos eletrônicos. Será necessário fazer a confirmação manualmente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CONFIRMING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "PENDING_TRADUCTION_ERROR_CONFIRMING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CONFIRMING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "Error interno confirmando pagos electrónicos. Será necesario hacer la confirmación manualmente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_COUPON_ALREADY_IN_USE', "O cupom já está em uso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_COUPON_ALREADY_IN_USE', "Coupon is already in use");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_COUPON_ALREADY_IN_USE', "El cupón ya está en uso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_COUPON_APPLIED', "Não é possível aplicar esse cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_COUPON_APPLIED', "Unable to apply this coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_COUPON_APPLIED', "No es posible aplicar este cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_COUPON_LOCKED_BY_ANOTHER_REQUEST', "Cupom bloqueado por outra requisição");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_COUPON_LOCKED_BY_ANOTHER_REQUEST', "Coupon locked by another request");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_COUPON_LOCKED_BY_ANOTHER_REQUEST', "Cupón bloqueado por otra solicitud");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_COUPON_NOT_FOUND', "Cupom não encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_COUPON_NOT_FOUND', "Coupon not found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_COUPON_NOT_FOUND', "Cupón no encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_CREATING_SALE', "Erro ao criar nova venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_CREATING_SALE', "PENDING_TRADUCTION_ERROR_CREATING_SALE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_CREATING_SALE', "Error al crear nueva venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_DELETED_SAVED_ORDERS', "Erro ao tentar apagar pedidos salvos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_DELETED_SAVED_ORDERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_DELETED_SAVED_ORDERS', "Error al intentar eliminar los pedidos guardados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_DELETE_USER', "Ocorreu um erro ao excluir usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_DELETE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_DELETE_USER', "¡Ocurrrió un error al eliminar el usuario!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_DISABLE_USER', "Ocorreu um erro ao desativar usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_DISABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_DISABLE_USER', "¡Ocurrió un error al desactivar usuario!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_DOWNLOAD', "Falha no download");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_DOWNLOAD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_DOWNLOAD', "Error en la descarga");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_ENABLE_USER', "Ocorreu um erro ao ativar usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_ENABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_ENABLE_USER', "¡Ocurrió un error al activar usuario!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_FINALIZE_SALE', "Ocorreu um erro finalizando a venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_FINALIZE_SALE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_FINALIZE_SALE', "Ocurrió un error finalizando la venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_GETTING_DATA', "Erro ao obter os dados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_GETTING_DATA', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_GETTING_DATA', "Error al obtener los datos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_GETTING_SCALE_WEIGHT', "Erro buscando o peso da balança");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_GETTING_SCALE_WEIGHT', "Error reading the scale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_GETTING_SCALE_WEIGHT', "Error al leer la balanza");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_GET_OPENED_TABS', "Ocorreu um erro ao recuperar as comandas abertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_GET_OPENED_TABS', "$PENDING_TRADUCTION_ERROR_GET_OPENED_TABS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_GET_OPENED_TABS', "Ocurrió un error obteniendo las comandas abiertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_INSERT_NEW_USER', "Ocorreu um erro ao cadastrar usuário!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_INSERT_NEW_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_INSERT_NEW_USER', "¡Ocurrió un error al registrar usuario!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY', "A entrega não pôde ser confirmada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY', "Failed to send delivery confirmation to the server");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY', "Error al enviar confirmación de pedido al servidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_ON_REMOTE_ORDER_DELIVERY_CONFIRMATION', "Erro confirmando pagamento:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_ON_REMOTE_ORDER_DELIVERY_CONFIRMATION', "Error confirming payment:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_ON_REMOTE_ORDER_DELIVERY_CONFIRMATION', "Error al confirmar pago:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_OPENING_DAY', "Erro ao processar abertura de dia:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_OPENING_DAY', "Error opening the day:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_OPENING_DAY', "Error al abrir el día:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_OPENING_DAYS', "Erro ao abrir o dia para os seguintes PDV:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_OPENING_DAYS', "Error opening the day in the following POS:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_OPENING_DAYS', "Error al abrir el día en los siguientes TPV:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_OPENING_DELIVERY_STORE', "Erro abrindo loja do Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_OPENING_DELIVERY_STORE', "Error opening Delivery Store");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_OPENING_DELIVERY_STORE', "Error al abrir Tienda Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_OPEN_DRAWER', "Erro ao abrir a gaveta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_OPEN_DRAWER', "Error opening drawer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_OPEN_DRAWER', "Error al abrir gaveta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_ORDER_TOTALED_NOT_SYNCHRONIZED', "Erro ao executar sua solicitação, seu pedido não está no status correto. Clique em OK para atualizarmos sua tela");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_ORDER_TOTALED_NOT_SYNCHRONIZED', "Error executing your request, your order is not in the correct status. Click OK to update your screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_ORDER_TOTALED_NOT_SYNCHRONIZED', "Error al ejecutar su solicitud, su pedido no está en el estado correcto. Haga clic en Aceptar para actualizar su pantalla");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PRINT_NFE', "A VENDA FOI CONCLUIDA COM SUCESSO\\\\Ocorreu um erro ao imprimir a Nota Fiscal:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PRINT_NFE', "THE SALE WAS COMPLETED WITH SUCCESS\\\\An error occurred while printing NFe:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PRINT_NFE', "LA VENTA SE HA COMPLETADO EXITOSAMENTE\\\\Ha ocurrido un error al imprimir Nota Fiscal:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PRINT_REPORT', "Erro ao imprimir relatório");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PRINT_REPORT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PRINT_REPORT', "Error al imprimir informe");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_COUPON', "Erro processando CUPOM Fiscal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_COUPON', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_COUPON', "Error procesando TICKET de compra");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_PAYMENT', "Erro ao processar pagamento: Total à pagar: {0}. TenderId: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_PAYMENT', "Error processing payment: Amount due: {0}. TenderId: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_PAYMENT', "Error al procesar Pago: Monto a pagar: {0}. TenderId: {1}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_PAYMENT_SELECTION', "Erro ao processar pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_PAYMENT_SELECTION', "Error processing payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_PAYMENT_SELECTION', "Error al procesar Pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_SALE', "Erro processando venda. Por favo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_SALE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_SALE', "Error procesando la venta. Por favo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_SKIM', "Não foi possível efetuar sangria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_SKIM', "Error processing skim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_SKIM', "Error al procesar efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_TAX_DATA', "Erro processando dados fiscais. Tentando novamente...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_TAX_DATA', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_TAX_DATA', "Error procesando datos fiscales. Intentando nuevamente...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_PROCESSING_TEF', "Falha ao processar TEF: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_PROCESSING_TEF', "Error processing TEF: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_PROCESSING_TEF', "Error al procesar TEF: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_READING_BARCODE', "Erro ao ler código de barras");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_READING_BARCODE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_READING_BARCODE', "Error al leer el código de barras");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_READING_FINGERPRINT', "Erro realizando a leitura da digital. Tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_READING_FINGERPRINT', "Error reading fingerprint. Try again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_READING_FINGERPRINT', "Error al registrar huella. Intente nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_RECALLING_ORDER', "Erro ao recuperar pedido: OrderId: #{0}. Exception: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_RECALLING_ORDER', "Error recovering order: OrderId: #{0}. Exception: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_RECALLING_ORDER', "Error al recuperar orden: OrderId: #{0}. Excepción: {1}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_REFUNDING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "Erro interno revertendo pagamentos eletrônicos. Será necessário fazer a reversão manualmente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_REFUNDING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "PENDING_TRADUCTION_ERROR_REFUNDING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_REFUNDING_ELECTRONIC_PAYMENTS_MANUAL_PROCESS_NEEDED', "Error interno revirtiendo pagos electrónicos. Será necesario hacer la reversión manualmente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_REGISTERING_FINGERPRINT', "Erro cadastrando impressao digital. Tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_REGISTERING_FINGERPRINT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_REGISTERING_FINGERPRINT', "Error registrando huella digital. Intente nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_REGISTER_PAYMENT_SEFAZ', "Erro ao registrar dados do pagamento no Integrador Sefaz");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_REGISTER_PAYMENT_SEFAZ', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_REGISTER_PAYMENT_SEFAZ', "Error al registrar datos del pago en el Integrador Sefaz");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_REPRINT_NFE', "Ocorreu um erro ao reimprimir a NFe:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_REPRINT_NFE', "An error occurred while printing NFe:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_REPRINT_NFE', "Ha ocurrido un error al reimprmir Nota Fiscal:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_REPRINT_PICKLIST', "Ocorreu um erro ao reimprimir a Picklist:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_REPRINT_PICKLIST', "An error occurred when reprinting the Picklist:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_REPRINT_PICKLIST', "Error al reimprimir la Picklist:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_RESTARTING_POS', "Erro reiniciando PDV{}.\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_RESTARTING_POS', "Error restarting POS{}.\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_RESTARTING_POS', "Error reiniciando TDV{}. \\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_RETRIEVING_USER_DATA', "Erro obtendo informações do usuário: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_RETRIEVING_USER_DATA', "Error getting user info: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_RETRIEVING_USER_DATA', "Error al obtener información de usuario: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SAVING_CLOSE_OPERATOR_REPORT', "Erro salvando informações do relatório de fechamento de Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SAVING_CLOSE_OPERATOR_REPORT', "Error saving operator closure report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SAVING_CLOSE_OPERATOR_REPORT', "Error al guardar reporte de cierre de Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SAVING_DATA', "Erro ao salvar os dados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SAVING_DATA', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SAVING_DATA', "Error al guardar los datos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SAVING_OPERATORS_BY_SECTOR', "Erro na atualização de operadores por setores.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SAVING_OPERATORS_BY_SECTOR', "Error updating sector operators.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SAVING_OPERATORS_BY_SECTOR', "Error en la actualización de operadores por sectores.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SAVING_ORDER', "Erro ao salvar pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SAVING_ORDER', "Error saving order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SAVING_ORDER', "Error al guardar orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SEARCHING_ZIP_CODE', "CEP não encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SEARCHING_ZIP_CODE', "Zipcode not found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SEARCHING_ZIP_CODE', "Código Postal no encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SENDING_REMOTE_ORDER_TO_PRODUCTION', "Erro enviando pedido para produção:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SENDING_REMOTE_ORDER_TO_PRODUCTION', "Error sending order to production:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SENDING_REMOTE_ORDER_TO_PRODUCTION', "Error al enviar orden para producir:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SEND_NFE', "Ocorreu um erro ao emitir Nota Fiscal:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SEND_NFE', "There was an error casting NFe:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SEND_NFE', "Se produjo un error al transmitir Nota Fiscal:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_SETTING_DELIVERYMAN', "Erro informando o nome do entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_SETTING_DELIVERYMAN', "Error informing delivery person name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_SETTING_DELIVERYMAN', "Error al informar nombre del Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_STARTING_PAYMENT_INTEGRATOR_SEFAZ', "Erro ao iniciar pagamento no Integrador Sefaz");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_STARTING_PAYMENT_INTEGRATOR_SEFAZ', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_STARTING_PAYMENT_INTEGRATOR_SEFAZ', "Error al iniciar el pago en el Integrador Sefaz");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_START_SALE', "Erro ao iniciar a venda - {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_START_SALE', "Error starting the sale - {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_START_SALE', "Error al iniciar la venta - {}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_STORING_PAYMENT_DATA', "Erro armazenando dados do pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_STORING_PAYMENT_DATA', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_STORING_PAYMENT_DATA', "Error almacenando los datos del pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_TOTALIZING_ORDER', "Erro ao totalizar pedido: OrderId: #{0}. Exception: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_TOTALIZING_ORDER', "Error totalizing order. OrderId: #{0}. Exception: {1}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_TOTALIZING_ORDER', "Error al totalizar orden. OrderId: #{0}. Excepción: {1}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_UPDATING_DEVICES', "Erro ao atualizar os dispositivos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_UPDATING_DEVICES', "Error updating devices");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_UPDATING_DEVICES', "Error al actualizar dispositivos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_VOIDING_REMOTE_ORDER', "Erro cancelando pedido:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_VOIDING_REMOTE_ORDER', "Error canceling order:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_VOIDING_REMOTE_ORDER', "Error al cancelar orden:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ERROR_VOIDING_STORED_ORDER', "Erro ao tentar apagar pedido salvo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ERROR_VOIDING_STORED_ORDER', "Error deleting saved order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ERROR_VOIDING_STORED_ORDER', "Error al eliminar orden guardada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ESTIMATED_DELIVERY_TIME', "Entrega prevista");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ESTIMATED_DELIVERY_TIME', "Estimated Delivery Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ESTIMATED_DELIVERY_TIME', "Tiempo de Entrega Estimado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EVENT_NOT_REGISTERED', "Evento nao cadastrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EVENT_NOT_REGISTERED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EVENT_NOT_REGISTERED', "Evento no registrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXCESS_AMOUNT_TURNED_INTO_TIP', "Valor excedente de {0} será transformado em gorjeta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXCESS_AMOUNT_TURNED_INTO_TIP', "PENDING_TRADUCTION_EXCESS_AMOUNT_TURNED_INTO_TIP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXCESS_AMOUNT_TURNED_INTO_TIP', "El valor excedente de {0} será transformado en propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXIT', "Sair");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXIT', "Exit");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXIT', "Salir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXIT_RUPTURE', "Sair de ruptura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXIT_RUPTURE', "Leave rupture");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXIT_RUPTURE', "Salir de ruptura");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXPIRED_STORE_CERTIFICATE', "Certificado da Loja Expirado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXPIRED_STORE_CERTIFICATE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXPIRED_STORE_CERTIFICATE', "Certificado de Tienda Expirado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXPO', "Expo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXPO', "Expo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXPO', "Expo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXPO_BEBIDAS', "Bebidas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXPO_BEBIDAS', "Drinks");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXPO_BEBIDAS', "Bebidas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXTERNAL_PAYMENT', "Maquininha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXTERNAL_PAYMENT', "External Payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXTERNAL_PAYMENT', "Pago Externo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXTERNAL_POS', "LEVAR MAQUININHA PARA COBRANÇA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXTERNAL_POS', "Take POS to charge customer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXTERNAL_POS', "Llevar máquina de punto de venta de  para cobrar al cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'EXTRA', "(Dividido)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'EXTRA', "(Split)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'EXTRA', "(Dividir)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FATAL_ERROR', "A operação não pôde ser realizada.\Por favor tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FATAL_ERROR', "Operation can be performed.\Try again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FATAL_ERROR', "No se pudo realizar operación.\Intentar nuevamente?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FATAL_ERROR_CALL_SUPPORT', "A operação não pôde ser realizada.\Por favor tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FATAL_ERROR_CALL_SUPPORT', "Operation can be performed.\Try again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FATAL_ERROR_CALL_SUPPORT', "No se pudo realizar operación.\Intentar nuevamente?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIFTH', "Quinto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIFTH', "Fifth");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIFTH', "Quinto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIFTH_READING', "Quinta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIFTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIFTH_READING', "Quinta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINALIZED', "Finalizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINALIZED', "PENDING_TRADUCTION_FINALIZED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINALIZED', "Finalizado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINAL_CARD_NUMBER', "Cartão Final");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINAL_CARD_NUMBER', "End Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINAL_CARD_NUMBER', "tarjeta final");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINGERPRINT', "Impressao Digital");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINGERPRINT', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINGERPRINT', "Huella digital");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINGERPRINT_ALREADY_FOR_USER', "Impressão digital já cadastrada para outro usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINGERPRINT_ALREADY_FOR_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINGERPRINT_ALREADY_FOR_USER', "Huella digital ya registrada para otro usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINGERPRINT_CHANGED_SUCCESSFULLY', "Impressão alterada com sucesso.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINGERPRINT_CHANGED_SUCCESSFULLY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINGERPRINT_CHANGED_SUCCESSFULLY', "Huella digital alterada con éxito.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINGER_READ_SUCCESSFULLY', "Leitura efetuada com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINGER_READ_SUCCESSFULLY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINGER_READ_SUCCESSFULLY', "Lectura realizada con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINISH', "Finalizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINISH', "Finish");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINISH', "Finalizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINISHED', "PRONTO!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINISHED', "READY!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINISHED', "¡LISTO!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FINISHED_ORDER', "Finalizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FINISHED_ORDER', "Finished");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FINISHED_ORDER', "Terminado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIRE_ITEM', "Mandar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIRE_ITEM', "Fire Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIRE_ITEM', "Mandar Artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIRST', "Primeiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIRST', "First");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIRST', "Primero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIRST_READING', "Primeira");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIRST_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIRST_READING', "Primera");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FISCAL_COMPONENT_FAILURE', "Falha no Componente Fiscal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FISCAL_COMPONENT_FAILURE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FISCAL_COMPONENT_FAILURE', "Falla en el Componente Fiscal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FISCAL_TABLE', "Fiscalizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FISCAL_TABLE', "Fiscal Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FISCAL_TABLE', "Tabla Fiscal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FIX_SEFAZ_PROTOCOL', "Corrigir Protocolo SEFAZ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FIX_SEFAZ_PROTOCOL', "Fix SEFAZ Protocol");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FIX_SEFAZ_PROTOCOL', "Corregir Protocolo SEFAZ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FLAG_CANCEL_PAID_ORDERS_ACTIVE', "Não é possível cancelar o pedido, a configuração para não cancelar pedidos pagos está ativada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FLAG_CANCEL_PAID_ORDERS_ACTIVE', "Cannot cancel order, setting not to cancel paid orders is enabled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FLAG_CANCEL_PAID_ORDERS_ACTIVE', "No se puede cancelar el pedido, la configuración para no cancelar pedidos pagados está habilitada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FORCE_OPEN_DAY', "Forçar Abrir o Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FORCE_OPEN_DAY', "Force Open Day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FORCE_OPEN_DAY', "Forzar Abrir el Día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FORMATTED_ADDRESS', "Endereço:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FORMATTED_ADDRESS', "Address:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FORMATTED_ADDRESS', "Dirección:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FOURTH', "Quarto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FOURTH', "Fourth");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FOURTH', "Cuarto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FOURTH_READING', "Quarta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FOURTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FOURTH_READING', "Cuarta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FRACTION_QTY', "Dividir por:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FRACTION_QTY', "Split to:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FRACTION_QTY', "Dividir en:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'FROM_TOTEM', "a partir de");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'FROM_TOTEM', "from");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'FROM_TOTEM', "desde");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GENERAL', "Geral");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GENERAL', "General");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GENERAL', "General");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_CHROME_VERSION', "Obtendo a versão do Chrome do servidor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_CHROME_VERSION', "Getting Chrome version from server");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_CHROME_VERSION', "Obtener la versión de Chrome del servidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_MENU_FROM_BACKOFFICE', "Buscando Menu no BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_MENU_FROM_BACKOFFICE', "GETTING_MENU_FROM_BACKOFFICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_MENU_FROM_BACKOFFICE', "Buscando Menú en el BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_NEW_CHROME', "Obtendo o novo Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_NEW_CHROME', "Getting new Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_NEW_CHROME', "Obtención de Chrome nuevo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_NEW_LOADERS', "Buscando Loaders no BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_NEW_LOADERS', "GETTING_NEW_LOADERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_NEW_LOADERS', "Buscando Loaders en el BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_TOTALED_TABLE', "Não é possível consultar informações de mesa totalizada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_TOTALED_TABLE', "Error consulting totalized table info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_TOTALED_TABLE', "Error al consultar información mesa totalizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GETTING_USERS_FROM_BACKOFFICE', "Obtendo usuários do BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GETTING_USERS_FROM_BACKOFFICE', "GETTING_USERS_FROM_BACKOFFICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GETTING_USERS_FROM_BACKOFFICE', "Obteniendo usuarios de BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GET_CONFIGURED_ALERTS_FAILED', "Erro ao obter as configurações de alertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GET_CONFIGURED_ALERTS_FAILED', "Error getting alerts settings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GET_CONFIGURED_ALERTS_FAILED', "Error al obtener la configuración de alertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GET_DEVICES_DATA_FAILED', "Erro ao obter informações dos dispositivos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GET_DEVICES_DATA_FAILED', "Error getting device information");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GET_DEVICES_DATA_FAILED', "Error al obtener información de los dispositivos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GET_WEIGHT_AGAIN', "PESAR NOVAMENTE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GET_WEIGHT_AGAIN', "WEIGHT AGAIN");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GET_WEIGHT_AGAIN', "PESAR NUEVAMENTE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GIFT_CARD', "Gift Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GIFT_CARD', "Gift Card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GIFT_CARD', "Tarjeta de regalo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GOAL', "Meta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GOAL', "Goal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GOAL', "Meta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GOALS_OF_THE_DAY', "Metas do Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GOALS_OF_THE_DAY', "Goal of the Day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GOALS_OF_THE_DAY', "Metas del Día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GO_TO_LINKED_TABLE', "IR ATÉ A MESA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GO_TO_LINKED_TABLE', "GO TO TABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GO_TO_LINKED_TABLE', "IR A MESA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'GO_TO_THE_CASHIER', "Dirija-se ao caixa para \n efetuar o pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'GO_TO_THE_CASHIER', "Go to the cashier to \n make the payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'GO_TO_THE_CASHIER', "Ir al cajero para \n realizar el pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_DELIVERY_ORDERS_IN_PROGRESS', "Existem pedidos do Delivery em progresso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_DELIVERY_ORDERS_IN_PROGRESS', "There is delivery orders in progress");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_DELIVERY_ORDERS_IN_PROGRESS', "Existen pedidos Delivery en proceso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_NO_ITEMS_IN_ORDER', "Não existem itens na venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_NO_ITEMS_IN_ORDER', "Order has no items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_NO_ITEMS_IN_ORDER', "La orden no tiene artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_NO_ITEMS_TO_SHOW', "Nenhum item para exibir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_NO_ITEMS_TO_SHOW', "No item to show");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_NO_ITEMS_TO_SHOW', "Ningún artículo para mostrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_NO_OPENED_TABS', "Não tem Comandas abertas para realizar a operação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_NO_OPENED_TABS', "PENDING_TRADUCTION_HAS_NO_OPENED_TABS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_NO_OPENED_TABS', "No posee Comandas abiertas para realizar la operación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_NO_POS_TO_CLOSE', "Não há nenhum PDV para fechar o dia.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_NO_POS_TO_CLOSE', "There is no POS to close the day.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_NO_POS_TO_CLOSE', "No hay ningún TPV para cerrar el día.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_NO_POS_TO_OPEN', "Não há nenhum PDV para abrir o dia.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_NO_POS_TO_OPEN', "There is no POS to open the day.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_NO_POS_TO_OPEN', "No hay ningún TPV para abrir día.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_OPENED_TABLES', "Existem mesas abertas para este usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_OPENED_TABLES', "There are tables opened by this operator.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_OPENED_TABLES', "Existen mesas abiertas por este usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_OPENED_TABLES_CONTINUE', "Existem mesas abertas para este usuário. Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_OPENED_TABLES_CONTINUE', "There are tables opened by this operator. Continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_OPENED_TABLES_CONTINUE', "Existen mesas abiertas por este Usuario. Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_OTHERS_POS_OPENED', "Existe uma atualização pendente que só pode ser aplicada quando todos os caixas estiverem fechados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_OTHERS_POS_OPENED', "$PENDING_TRADUCTION_HAS_OTHERS_POS_OPENED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_OTHERS_POS_OPENED', "Existe una actualización pendiente que sólo puede ser aplicada cuando todas las cajas estén cerradas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_PENDING_UPDATES', "Existem atualizações prontas para serem aplicadas.\\Deseja aplicar agora?\\Por favor mantenha a maquina ligada para atualizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_PENDING_UPDATES', "PENDING_TRADUCTION_HAS_PENDING_UPDATES");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_PENDING_UPDATES', "Existen actualizaciones listas para ser instaladas.\\¿Desea aplicarlas ahora?\\Por favor mantenga el equipo conectado para actualizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HAS_PENDING_UPDATES_TO_APPLY_NOW', "Existem atualizações prontas para serem atualizadas.\\Por favor mantenha a maquina ligada para atualizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HAS_PENDING_UPDATES_TO_APPLY_NOW', "PENDING_TRADUCTION_HAS_PENDING_UPDATES_TO_APPLY_NOW");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HAS_PENDING_UPDATES_TO_APPLY_NOW', "Existen actualizaciones listas para ser instaladas.\\Por favor mantenga el equipo conectado para actualizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HIGH_PRIORITY', "Prioritário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HIGH_PRIORITY', "Prioritary");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HIGH_PRIORITY', "Prioritario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HOLD_ITEM', "Segurar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HOLD_ITEM', "Hold item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HOLD_ITEM', "Retener Artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'HOUR', "Hora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'HOUR', "Hour");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'HOUR', "Hora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'IFOOD_LOCALIZER_TEXT', "Use o código do Localizador abaixo para fazer o rastreamento do seu pedido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'IFOOD_LOCALIZER_TEXT', "Use the following code to track your order.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'IFOOD_LOCALIZER_TEXT', "Use el siguiente código para rastrear su pedido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INCLUDE_CHANGE_AS_TIP', "O valor restante será incluído na Gorjeta?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INCLUDE_CHANGE_AS_TIP', "PENDING_TRADUCTION_INCLUDE_CHANGE_AS_TIP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INCLUDE_CHANGE_AS_TIP', "¿El monto restante será incluído en la Propina?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORMATION', "Informação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORMATION', "Information");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORMATION', "Información");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORMATION_DELIVERY_REPORT', "Informações do Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORMATION_DELIVERY_REPORT', "Customer Info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORMATION_DELIVERY_REPORT', "Información de Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_ANOTHER', "Informar Outro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_ANOTHER', "Inform Other");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_ANOTHER', "Informar Otro");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_CPF', "Informar CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_CPF', "Inform CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_CPF', "Informar CPF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_DELIVERYMAN_NAME', "Informe o nome do entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_DELIVERYMAN_NAME', "Delivery person name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_DELIVERYMAN_NAME', "Nombre de Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_LOYALTY_ID', "Informar Fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_LOYALTY_ID', "Inform Loyalty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_LOYALTY_ID', "Informar Fidelidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_ORDER_READY', "Pedido Pronto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_ORDER_READY', "Order Ready");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_ORDER_READY', "Orden Lista");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INFORM_ORDER_READY_CONFIRMATION', "Tem certeza que deseja informar este pedido como pronto?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INFORM_ORDER_READY_CONFIRMATION', "Are you sure you want to mark this order as ready?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INFORM_ORDER_READY_CONFIRMATION', "¿Está seguro de que desea informar que este pedido está listo?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INGREDIENTS', "Ingredientes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INGREDIENTS', "Ingredients");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INGREDIENTS', "Ingredientes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INITIAL_CARD_NUMBER', "Cartão início");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INITIAL_CARD_NUMBER', "Initial card");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INITIAL_CARD_NUMBER', "Tarjeta de inicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_CARD_CANCEL_PAYMENT', "Insira o cartão {0} para cancelamento. {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_CARD_CANCEL_PAYMENT', "$PENDING_TRADUCTION_INSERT_CARD_CANCEL_PAYMENT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_CARD_CANCEL_PAYMENT', "Inserte la tarjeta {0} para cancelamiento. {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_CUSTOMER_DOC', "Informe o CPF do cliente para iniciar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_CUSTOMER_DOC', "Enter the customer's CPF to start");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_CUSTOMER_DOC', "Introduce el CPF del cliente para empezar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_CUSTOMER_NUMBER', "Informe o número de fidelidade do cliente para iniciar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_CUSTOMER_NUMBER', "Enter your customer loyalty number to get started");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_CUSTOMER_NUMBER', "Introduce tu número de fidelidad de cliente para empezar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_CUSTOMER_PHONE', "Informe o telefone do cliente para iniciar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_CUSTOMER_PHONE', "Enter the customer's phone number to start");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_CUSTOMER_PHONE', "Introduce el número de teléfono del cliente para empezar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_DELIVERY_MAN_NAME', "Necessário inserir o nome do entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_DELIVERY_MAN_NAME', "Delivery person name needed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_DELIVERY_MAN_NAME', "Se necesita nombre de Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INSERT_ORDER_ID', "Insira o ID do pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INSERT_ORDER_ID', "Enter Order ID");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INSERT_ORDER_ID', "Ingrese el ID del pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INTERNAL_ERROR_CONFIRMING_ELECTRONIC_PAYMENTS', "Erro interno confirmando pagamentos eletrônicos. Clique OK para tentar novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INTERNAL_ERROR_CONFIRMING_ELECTRONIC_PAYMENTS', "PENDING_TRADUCTION_INTERNAL_ERROR_CONFIRMING_ELECTRONIC_PAYMENTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INTERNAL_ERROR_CONFIRMING_ELECTRONIC_PAYMENTS', "Error interno confirmando pagos electrónicos. Presione OK para intentar nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INTERNAL_ERROR_REFUNDING_ELECTRONIC_PAYMENTS', "Erro interno revertendo pagamentos eletrônicos. Clique OK para tentar novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INTERNAL_ERROR_REFUNDING_ELECTRONIC_PAYMENTS', "PENDING_TRADUCTION_INTERNAL_ERROR_REFUNDING_ELECTRONIC_PAYMENTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INTERNAL_ERROR_REFUNDING_ELECTRONIC_PAYMENTS', "Error interno revirtiendo pagos electrónicos. Presion OK para intentar nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INTERVAL_BY_ORDER_TIME', "Relatório de Lançamentos por Intervalo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INTERVAL_BY_ORDER_TIME', "$PENDING_TRADUCTION_INTERVAL_BY_ORDER_TIME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INTERVAL_BY_ORDER_TIME', "$PENDING_TRADUCTION_INTERVAL_BY_ORDER_TIME");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INTERVAL_SALES', "Relatório de Vendas por Intervalo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INTERVAL_SALES', "Sales Report by Period");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INTERVAL_SALES', "Reporte de Ventas por Periodo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_BOH_CONFIGURATION', "Existem configurações inválidas no BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_BOH_CONFIGURATION', "There are invalid settings in BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_BOH_CONFIGURATION', "Hay configuraciones no válidas en BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_COSTUMER_INFO', "Informação inválida. Por favor digite novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_COSTUMER_INFO', "Invalid value. Please type it again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_COSTUMER_INFO', "PENDING_TRADUCTION_INVALID_COSTUMER_INFO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_COUPON_CODE', "O código do cupom é inválido [{0}].");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_COUPON_CODE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_COUPON_CODE', "El código del cupón es inválido [{0}].");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CPF_CNPJ', "CPF/CNPJ inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CPF_CNPJ', "Invalid Document");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CPF_CNPJ', "Documento Inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CPF_ERROR', "O CPF digitado não é válido. Por favor digite novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CPF_ERROR', "The typed CPF is not valid. Please type it again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CPF_ERROR', "El CPF escrito no es válid");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CRM_COUPON', "Cupom Inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CRM_COUPON', "Cupom Inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CRM_COUPON', "Cupon Inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CURRENT_ORDER_STATUS', "Status do pedido inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CURRENT_ORDER_STATUS', "Invalid order status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CURRENT_ORDER_STATUS', "Estado de pedido no válido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CUSTOMER_NAME', "Nome do cliente inválido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CUSTOMER_NAME', "PENDING_TRADUCTION_INVALID_CUSTOMER_NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CUSTOMER_NAME', "Nombre de cliente inválido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_CUSTOMER_PHONE', "Telefone do cliente inválido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_CUSTOMER_PHONE', "PENDING_TRADUCTION_INVALID_CUSTOMER_PHONE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_CUSTOMER_PHONE', "Número de teléfono de cliente inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DATE', "Data inválida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DATE', "Invalid date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DATE', "Fecha inválida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DATE_FORMAT', "A data digitada não é valida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DATE_FORMAT', "PENDING_TRADUCTION_INVALID_DATE_FORMAT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DATE_FORMAT', "La fecha ingresada no es válida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DATE_FUTURE', "Data inválida. Data não pode ser maior do que a data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DATE_FUTURE', "Invalid date. Date cannot be greater than current date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DATE_FUTURE', "Fecha inválida. La fecha no puede ser mayor que la fecha actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DATE_MAX_PERIOD', "Data inválida. Período selecionado é maior do que o período máximo solicitado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DATE_MAX_PERIOD', "Invalid date. Selected period is greater than requested maximum period");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DATE_MAX_PERIOD', "Fecha inválida. El período seleccionado es más largo que el período máximo solicitado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DATE_RANGE', "Data inválida. Data final não pode ser maior do que a data inicial");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DATE_RANGE', "Invalid date. Final date cannot be greater than start date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DATE_RANGE', "Fecha inválida. La fecha de finalización no puede ser mayor que la fecha de inicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_DESTINATION_TABLE_MODE', "O modo de serviço de mesa destino é inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_DESTINATION_TABLE_MODE', "Invalid destination table service mode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_DESTINATION_TABLE_MODE', "El modo de servicio de la mesa de destino es inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_FINGERPRINT_READING', "Leitura da impressão digital inválida. Tente novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_FINGERPRINT_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_FINGERPRINT_READING', "Lectura de huella digital inválida. Intente nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_FRACTION_QTY', "Quantidade inválida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_FRACTION_QTY', "Invalid quantity");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_FRACTION_QTY', "Cantidad inválidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_ID', "Id inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_ID', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_ID', "ID inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_NUMBER_OF_SEATS', "O número de pessoas deve ser maior que 0");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_NUMBER_OF_SEATS', "The number of people must be greater than 0");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_NUMBER_OF_SEATS', "El número de personas debe ser mayor a {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_PAGER_NUMBER', "É necessário informar um número válido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_PAGER_NUMBER', "You must enter a valid number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_PAGER_NUMBER', "Debes ingresar un número válido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_SKIM_NUMBER', "O número do envelope precisa ter entre {0} e {1} dígitos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_SKIM_NUMBER', "Skim number has to have from {0} to {1} digits");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_SKIM_NUMBER', "El efectivo debe ser de {0} a {1} dígitos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_TAB_NAME', "Nome de Comanda inválido. O nome deve conter no máximo {0} caracteres alfanuméricos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_TAB_NAME', "Invalid Tab Name. The name must contain a maximum of {0} alphanumeric characters");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_TAB_NAME', "Nombre de Comanda inválido. El nombre debe contener un máximo de {0} caracteres alfanuméricos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_USER', "Usuário inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_USER', "Invalid user");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_USER', "Usuario Inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_USERNAME_OR_PASSWORD', "Usuário ou senha inválidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_USERNAME_OR_PASSWORD', "Invalid user or password");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_USERNAME_OR_PASSWORD', "Usuario o contraseña inválidos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_USER_ENTERED', "Garçom não encontrado: {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_USER_ENTERED', "Operator not found: {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_USER_ENTERED', "No se encontró Mesero: {}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_USER_OR_PASSWORD', "Usuário ou senha inválidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_USER_OR_PASSWORD', "Invalid user or password");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_USER_OR_PASSWORD', "Usuario o contraseña inválidos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'INVALID_VOUCHER', "Id de pagamento do voucher inválido: {0}}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'INVALID_VOUCHER', "Invalid voucher payment ID: {0}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'INVALID_VOUCHER', "ID de pago de voucher inválido: {0}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'IN_PROGRESS', "Aberta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'IN_PROGRESS', "In progress");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'IN_PROGRESS', "En Progreso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'IN_RUPTURE', "Em Ruptura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'IN_RUPTURE', "In Rupture");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'IN_RUPTURE', "En Ruptura");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ISSUED_AT', "Emitido em");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ISSUED_AT', "Issued at");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ISSUED_AT', "Emitido el");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ITEM', "Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ITEM', "Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ITEM', "Artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ITEMS_DELIVERY_REPORT', "Itens do Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ITEMS_DELIVERY_REPORT', "Order Items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ITEMS_DELIVERY_REPORT', "Artículos ordenados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ITEMS_RUPTURE', "Ruptura de Itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ITEMS_RUPTURE', "Items rupture");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ITEMS_RUPTURE', "Ruptura de Artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ITEM_TOTAL_VALUE', "Valor Total do Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ITEM_TOTAL_VALUE', "Item Total Price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ITEM_TOTAL_VALUE', "Precio total de Artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ITENS_BREAKDOWN', "RUPTURA DE ITENS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ITENS_BREAKDOWN', "PRODUCT BREAKDOWN");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ITENS_BREAKDOWN', "DESGLOCE DE ARTÍCULOS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JOIN', "Juntar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JOIN', "Join");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JOIN', "Unir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION', "Justificativa:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION', "Justification:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION', "Justificación:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_CANCELLATION_TITLE', "JUSTIFICATIVA DE CANCELAMENTO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_CANCELLATION_TITLE', "CANCELLATION GROUNDS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_CANCELLATION_TITLE', "MOTIVOS DE CANCELACIÓN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_DISCOUNT', "Informe a justificativa do desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_DISCOUNT', "PENDING_TRADUCTION_JUSTIFICATION_DISCOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_DISCOUNT', "Ingrese la justificación del descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_DISCOUNT_REPORT', "Relatório Desconto com Justificativas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_DISCOUNT_REPORT', "Discount Report with Justifications");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_DISCOUNT_REPORT', "Informe de descuento con justificaciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_DISCOUNT_TITLE', "JUSTIFICATIVA DE DESCONTO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_DISCOUNT_TITLE', "CANCELLATION GROUNDS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_DISCOUNT_TITLE', "MOTIVOS DE CANCELACIÓN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_TITLE_ITEM', "ITEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_TITLE_ITEM', "ITEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_TITLE_ITEM', "ARTÍCULO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'JUSTIFICATION_TITLE_ORDER', "PEDIDO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'JUSTIFICATION_TITLE_ORDER', "ORDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'JUSTIFICATION_TITLE_ORDER', "PEDIDO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_CLEANUP', "Limpar KDSs");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_CLEANUP', "Clear KDSs");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_CLEANUP', "Limpiar KDSs");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_CLEANUP_MESSAGE', "Os pedidos serão apagados de todos os KDSs e não será possível recuperá-los.\Isso pode causar reimpressão de algumas Picklists.\\Deseja Continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_CLEANUP_MESSAGE', "The orders will be deleted from all KDSs and will not be possible to recover them.\This can cause some Picklists reprint.\\Continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_CLEANUP_MESSAGE', "Los pedidos seran eliminadas de todos los KDSs y no será posible recuperarlas.\Esto puede causar la reimpresión de algunas Picklists.\\¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_DONE', "Prontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_DONE', "Ready");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_DONE', "Listo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_HEADER_DISPLAY_TIME_AVERAGE', "TEMPO:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_HEADER_DISPLAY_TIME_AVERAGE', "TIME:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_HEADER_DISPLAY_TIME_AVERAGE', "TIEMPO:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_HEADER_DISPLAY_TIME_GOAL', "META:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_HEADER_DISPLAY_TIME_GOAL', "GOAL:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_HEADER_DISPLAY_TIME_GOAL', "META:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_HELD', "EM ESPERA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_HELD', "ON HOLD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_HELD', "EN ESPERA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_HOLD', "Aguardando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_HOLD', "Waiting");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_HOLD', "Esperando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_IN_PROGRESS', "Preparando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_IN_PROGRESS', "Preparing");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_IN_PROGRESS', "Preparando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PAGE', "PÁGINA:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PAGE', "PAGE:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PAGE', "PÁGINA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREPARED', "SERVIDOS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREPARED', "SERVED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREPARED', "SERVIDO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_COOK', "Preparo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_COOK', "Cook");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_COOK', "Preparación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_ITEM', "Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_ITEM', "Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_ITEM', "Artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_MAIN', "Prato Principal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_MAIN', "Main Dish");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_MAIN', "Platillo Principal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_NUM', "#");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_NUM', "#");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_NUM', "#");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_ORDER', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_ORDER', "Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_ORDER', "Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_QTY', "Qtd");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_QTY', "Qtt");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_QTY', "Cant");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_SALE_TYPE', "Tipo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_SALE_TYPE', "Type");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_SALE_TYPE', "Tipo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_STATUS', "Info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_STATUS', "Info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_STATUS', "Info");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_TABLE', "Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_TABLE', "Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_TABLE', "Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_TIME', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_TIME', "Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_TIME', "Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_PREP_ZONE', "Setor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_PREP_ZONE', "Sector");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_PREP_ZONE', "Sector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_RECALL', "(R)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_RECALL', "(R)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_RECALL', "(R)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_STATUS', "KDS Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_STATUS', "KDS Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_STATUS', "Estado del KDS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KDS_STATUS_ORDER_COUNTER', "PEDIDOS:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KDS_STATUS_ORDER_COUNTER', "ORDERS:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KDS_STATUS_ORDER_COUNTER', "ÓRDENES:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KEEP', "Manter");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KEEP', "KEEP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KEEP', "Mantener");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KIOSK_CANCEL_QUESTION', "O seu pedido será cancelado deseja prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KIOSK_CANCEL_QUESTION', "You will lose all chosen items continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KIOSK_CANCEL_QUESTION', "Su pedido será cancelad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KIOSK_COUPON', "CUPOM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KIOSK_COUPON', "PENDING_TRADUCTION_NDISCOUNT_SERVICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KIOSK_COUPON', "CUPÓN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KIOSK_EAT_IN', "COMER AQUI");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KIOSK_EAT_IN', "I'LL EAT IN");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KIOSK_EAT_IN', "COMER AQUÍ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KIOSK_NOT_SALE', "Não é permitido vender esse produto no Kiosk");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KIOSK_NOT_SALE', "You may not sell this product on Kiosk");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KIOSK_NOT_SALE', "Este artículo no se puede vender en Kiosco");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'KIOSK_TAKE_OUT', "VIAGEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'KIOSK_TAKE_OUT', "I'LL TAKE OUT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'KIOSK_TAKE_OUT', "PARA LLEVAR");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'L10N_CURRENCY_SYMBOL_POSITION', "before");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'L10N_CURRENCY_SYMBOL_POSITION', "before");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'L10N_CURRENCY_SYMBOL_POSITION', "antes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'L10_PYTHON_DATETIME_FORMAT', "%d/%m/%Y %H:%M:%S");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'L10_PYTHON_DATETIME_FORMAT', "%m/%d/%Y %I:%M:%S %p");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'L10_PYTHON_DATETIME_FORMAT', "%d/%m/%Y %H:%M:%S");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'L10_PYTHON_DATE_FORMAT', "%d/%m/%Y");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'L10_PYTHON_DATE_FORMAT', "%m/%d/%Y");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'L10_PYTHON_DATE_FORMAT', "%d/%m/%Y");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_ADDITION_AMOUNT', "Acréscimos: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_ADDITION_AMOUNT', "Added: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_ADDITION_AMOUNT', "Añadido: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_BUSINESS_DATE', "Data Fiscal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_BUSINESS_DATE', "Fiscal Date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_BUSINESS_DATE', "Fecha Fiscal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_CHANGE_AMOUNT', "Troco    : {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_CHANGE_AMOUNT', "Change  : {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_CHANGE_AMOUNT', "Cambio    : {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_CURRENT_DATE', "Data Atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_CURRENT_DATE', "Current Date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_CURRENT_DATE', "Fecha Actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_DUE_AMOUNT', "A Pagar : {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_DUE_AMOUNT', "Amount due: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_DUE_AMOUNT', "A pagar: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_POS', "PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_POS', "POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_POS', "TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_STORE', "Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_STORE', "Store");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_STORE', "Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_TIP_AMOUNT', "Gorjeta : {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_TIP_AMOUNT', "Tip     : {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_TIP_AMOUNT', "Propina : {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_TOTAL_AMOUNT', "Total:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_TOTAL_AMOUNT', "Total:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_TOTAL_AMOUNT', "Total:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LABEL_USER', "Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LABEL_USER', "Operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LABEL_USER', "Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LAST_SERVICE', "Último serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LAST_SERVICE', "Last service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LAST_SERVICE', "Último Servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LAST_UPDATE', "Última Atualização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LAST_UPDATE', "Last update");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LAST_UPDATE', "Última actualización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LAYOUT', "Layout (Mapa)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LAYOUT', "Floor Plan");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LAYOUT', "Plano");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LDISPLAY_POS_ACTIVE', "Seja Bem-Vindo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LDISPLAY_POS_ACTIVE', "Welcome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LDISPLAY_POS_ACTIVE', "Bienvenido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LEAVE_THE_SECTOR', "Sairão de {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LEAVE_THE_SECTOR', "Leave the {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LEAVE_THE_SECTOR', "Saldrán de {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LESS', "a menos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LESS', "less");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LESS', "menos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LIGHT', "(Pouco)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LIGHT', "(Little)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LIGHT', "(Poco)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LINKED', "Juntada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LINKED', "Linked");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LINKED', "Juntada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LINKED_TABLES', "Mesas Ligadas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LINKED_TABLES', "Linked tables");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LINKED_TABLES', "Mesas Ligadas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LINKED_TABLE_ALERT', "A mesa selecionada está vinculada com a mesa {0}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LINKED_TABLE_ALERT', "The selected table is linked with the table {0}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LINKED_TABLE_ALERT', "La mesa seleccionada está juntada con la mesa {0}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LIST_DISCOUNTS', "Listar Descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LIST_DISCOUNTS', "List Discounts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LIST_DISCOUNTS', "Listar Descuentos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOADER', "Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOADER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOADER', "Loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOADERS_BACKUP', "Criando backup dos Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOADERS_BACKUP', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOADERS_BACKUP', "Creando backup de los Loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOADERS_NOT_FOUND', "Não foram encontrados loaders para essa loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOADERS_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOADERS_NOT_FOUND', "No fueron encontrados loaders para esta tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOCALIZER', "Localizador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOCALIZER', "Record Number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOCALIZER', "Rastreador");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGGEDIN', "Logado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGGEDIN', "Loggedin");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGGEDIN', "Registrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGGEDOUT', "Deslogado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGGEDOUT', "Logged out");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGGEDOUT', "Desconectado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGIN', "Login");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGIN', "Login");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGIN', "Iniciar Sesión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_AWAITING_CANCELLATION', "Aguardando cancelamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_AWAITING_CANCELLATION', "Awaiting cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_AWAITING_CANCELLATION', "Esperando cancelación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_BY_PARTNER', "Entrega por parceiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_BY_PARTNER', "Deliver by partner");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_BY_PARTNER', "Entrega por Socio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_CANCEL', "Cancelar entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_CANCEL', "Cancel delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_CANCEL', "Cancelar Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_CANCELED', "Foi solicitado o cancelamento da entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_CANCELED', "Delivery cancellation requested");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_CANCELED', "Cancelación de pedido solicitada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_CANCELLED_ERROR', "Erro cancelando a logística");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_CANCELLED_ERROR', "Error canceling logistics");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_CANCELLED_ERROR', "Error al cancelar logística");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_CANCEL_CONFIRMATION', "Deseja cancelar a entrega?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_CANCEL_CONFIRMATION', "Are you sure you want to cancel the delivery?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_CANCEL_CONFIRMATION', "¿Está seguro de cancelar su pedido?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_CODE', "Informações Entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_CODE', "Delivery Partner Code");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_CODE', "Código de Socio de Entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_NOT_FOUND', "Nenhum entregador encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_NOT_FOUND', "No courier found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_NOT_FOUND', "No se encontró repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_PARTNER_BEEDELIVERY', "Bee Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_PARTNER_BEEDELIVERY', "Bee Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_PARTNER_BEEDELIVERY', "Bee Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_PARTNER_BRINGG', "Bringg");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_PARTNER_BRINGG', "Bringg");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_PARTNER_BRINGG', "Bringg");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_PARTNER_DEFAULT', "Entregador Próprio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_PARTNER_DEFAULT', "Local Delivery Person");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_PARTNER_DEFAULT', "Repartidor Local");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_PARTNER_DELIVERYBEE', "Delivery Bee");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_PARTNER_DELIVERYBEE', "Delivery Bee");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_PARTNER_DELIVERYBEE', "Delivery Bee");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_PARTNER_MOTTU', "Mottu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_PARTNER_MOTTU', "Mottu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_PARTNER_MOTTU', "Mottu");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_REQUEST', "Solicitar entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_REQUEST', "Request deliveryman");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_REQUEST', "Solicitar repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SEARCHING', "Confirmando entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SEARCHING', "Searching deliveryman");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SEARCHING', "Buscando Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SEARCHING_ERROR', "Erro ao solicitar entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SEARCHING_ERROR', "Could not request logistic");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SEARCHING_ERROR', "Error al solicitar repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SEND', "Sair para entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SEND', "Send order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SEND', "Enviar Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SEND_CONFIRMATION', "Deseja enviar o pedido para entrega?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SEND_CONFIRMATION', "Are you sure you want to send the order out for delivery?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SEND_CONFIRMATION', "¿Está seguro de enviar el pedido para entrega?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SENT', "Em rota de entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SENT', "Out for delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SENT', "Fuera para entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_SENT_ERROR', "Erro ao enviar pedido para entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_SENT_ERROR', "Could not send order to logistic");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_SENT_ERROR', "Error al enviar pedido para la entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGISTIC_WAITING', "Aguardando entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGISTIC_WAITING', "Awaiting deliveryman");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGISTIC_WAITING', "Esperando repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGOFF_REPORT', "Relatório de Logoff");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGOFF_REPORT', "Logoff Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGOFF_REPORT', "Reporte de Logoff");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOGOUT', "Sair");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOGOUT', "Logout");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOGOUT', "Salir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY', "Fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY', "Loyalty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY', "Fidelidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_FINAL_MESSAGE_CUSTOMER_HEADER', "Mensagem final ao cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_FINAL_MESSAGE_CUSTOMER_HEADER', "Final message to the customer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_FINAL_MESSAGE_CUSTOMER_HEADER', "Mensaje final al cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_INFORMATION_NOT_FOUND', "Usuário não cadastrado no sistema de fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_INFORMATION_NOT_FOUND', "User not registered in the loyalty system");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_INFORMATION_NOT_FOUND', "Usuario no registrado en el sistema de fidelización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_PARTNER_ERROR', "Erro no parceiro de fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_PARTNER_ERROR', "Loyalty partner error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_PARTNER_ERROR', "Error de socio de lealtad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_SERVICE_NOT_AVAILABLE', "Serviço de Fidelidade indisponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_SERVICE_NOT_AVAILABLE', "Loyalty Service not available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_SERVICE_NOT_AVAILABLE', "Servicio de fidelidad no disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_USER_DESCRIPTION_COUPON_WITH_SEAT', "{0} (Assento {1})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_USER_DESCRIPTION_COUPON_WITH_SEAT', "{0} (Seat {1})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_USER_DESCRIPTION_COUPON_WITH_SEAT', "{0} (Asiento {1})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER', "Seja bem-vindo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER', "Welcome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER', "Bienvenido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER_HEADER', "Mensagem de boas-vindas ao cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER_HEADER', "Customer welcome message");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'LOYALTY_WELCOME_MESSAGE_CUSTOMER_HEADER', "mensaje de bienvenida al cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAINTENANCE_COMPONENT_NOT_FOUND', "Componente de manutenção não encontrado para o {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAINTENANCE_COMPONENT_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAINTENANCE_COMPONENT_NOT_FOUND', "Componente de manutención no encontrado para {}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAIN_DISHES', "Pratos Principais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAIN_DISHES', "Main Dishes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAIN_DISHES', "Platillos Principales");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAKE_USER_SUPERVISOR', "Tornar o usuário supervisor?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAKE_USER_SUPERVISOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAKE_USER_SUPERVISOR', "¿Hacer al usuario supervisor?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGER', "Gerente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGER', "Manager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGER', "Gerente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGER_AUTHORIZATION_CANCELLATION', "Cancelamento com autorização do gerente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGER_AUTHORIZATION_CANCELLATION', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGER_AUTHORIZATION_CANCELLATION', "Cancelamiento con autorización del gerente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGER_MENU', "Menu Gerente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGER_MENU', "Manager Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGER_MENU', "Menú de Gerente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGE_DAILY_DISHES', "Gerenciar Quantidade de Pratos do Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGE_DAILY_DISHES', "Manage daily dishes quantity");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGE_DAILY_DISHES', "Administrar Cantidad de Platos del Día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGE_SERVICE_PERIOD', "Gerenciar Períodos de Serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGE_SERVICE_PERIOD', "Manage Period of Service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGE_SERVICE_PERIOD', "Administrar Período de Servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGE_SERVICE_SECTOR', "Gerenciar Setor de Atendimento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGE_SERVICE_SECTOR', "Manage Service Sector");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGE_SERVICE_SECTOR', "Administrar Sector de Atendimiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANAGE_STOCKOUT_ITEMS', "Gerenciar Itens Esgotados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANAGE_STOCKOUT_ITEMS', "Manage Stockout Items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANAGE_STOCKOUT_ITEMS', "Administrar Artículos Agotados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANDATORY_LOGISTIC_FOR_PRODUCTION', "Necessário solicitar entregador antes de Produzir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANDATORY_LOGISTIC_FOR_PRODUCTION', "Call for delivery person before produce");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANDATORY_LOGISTIC_FOR_PRODUCTION', "Llamar al Repartidor antes de la preparación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANUAL', "Manual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANUAL', "Manual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANUAL', "Manual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MANUAL_CANCEL', "Cancelamento Manual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MANUAL_CANCEL', "Manual Cancel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MANUAL_CANCEL', "Cancelar manualmente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAXIMUM_AMOUNT_TO_FISCALIZE', "Por restrições fiscais, apenas vendas com valores inferiores a {0} podem ser totalizadas. Separe a venda em grupo de itens em que a soma seja inferior a {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAXIMUM_AMOUNT_TO_FISCALIZE', "Due to tax restrictions, only sales with values lower than {0} can be totaled. Separate the sale into a group of items where the sum is less than {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAXIMUM_AMOUNT_TO_FISCALIZE', "Debido a las restricciones fiscales, solo se pueden totalizar las ventas con valores inferiores a {0}. Separe la venta en un grupo de artículos donde la suma sea menor que {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_NUMBER_OF_DIGITS', "Número máximo de digitos excedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_NUMBER_OF_DIGITS', "Maximum number of digits exceeded");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_NUMBER_OF_DIGITS', "Número máximo de dígitos excedidos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_NUMBER_OF_SEATS', "Número acima que o máximo de assentos permitidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_NUMBER_OF_SEATS', "Number bigger than the maximum allowed seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_NUMBER_OF_SEATS', "Número mayor al permitido de asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_NUMBER_OF_SEATS_QUESTION', "Número acima que o máximo de assentos permitidos. Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_NUMBER_OF_SEATS_QUESTION', "$PENDING_TRADUCTION_MAX_NUMBER_OF_SEATS_QUESTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_NUMBER_OF_SEATS_QUESTION', "$PENDING_TRADUCTION_MAX_NUMBER_OF_SEATS_QUESTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_QUANTITY', "Máx. ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_QUANTITY', "Max. ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_QUANTITY', "Máx. ({0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_QUANTITY_ITEM_EXCEEDED', "A quantidade máxima permitida para item {} ({}) é {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_QUANTITY_ITEM_EXCEEDED', "PENDING_TRADUCTION_MAX_QUANTITY_ITEMS_EXCEEDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_QUANTITY_ITEM_EXCEEDED', "La cantidad máxima permitida para el artículo {} ({}) es {}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_QUANTITY_OPTION_EXCEEDED', "A quantidade máxima para item {} ({}) na option {} ({}) foi excedida. Quantidade máxima da option é {}. Quantidade atingida para a option é {}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_QUANTITY_OPTION_EXCEEDED', "The maximum quantity for item {} ({}) in option {} ({}) has been exceeded. Maximum amount of the option is {}. Amount reached for the option is {}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_QUANTITY_OPTION_EXCEEDED', "La cantidad máxima para el artículo {} ({}) en la opción {} ({}) fue excedida. La cantidad máxima de la opción es {}. La cantidad alcanzada por la opción es {}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MAX_QUANTITY_OPTION_EXCEEDED_SELLING_ITEM', "A quantidade máxima para item {} ({}) na option {} ({}) foi excedida na venda do item {} ({})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MAX_QUANTITY_OPTION_EXCEEDED_SELLING_ITEM', "The maximum quantity for item {} ({}) in option {} ({}) has been exceeded when selling item {} ({})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MAX_QUANTITY_OPTION_EXCEEDED_SELLING_ITEM', "Se ha excedido la cantidad máxima para el artículo {} ({}) en la opción {} ({}) al vender el artículo {} ({})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MEDIA_UPDATE', "Atualizar Mídia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MEDIA_UPDATE', "Update Media");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MEDIA_UPDATE', "Actualizar Media");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MENU', "Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MENU', "Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MENU', "Menú");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MENU_UPDATE', "Atualizar Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MENU_UPDATE', "Update Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MENU_UPDATE', "Actualizar Menú");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MERGE_ITEM', "Combinar Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MERGE_ITEM', "Merge Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MERGE_ITEM', "Combinar artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MINIMAL_ORDER_FOR_COUPON', "O valor minino do pedido para o cupom [{0}] é [{1}].");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MINIMAL_ORDER_FOR_COUPON', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MINIMAL_ORDER_FOR_COUPON', "El valor mínimo del pedido para el cupón [{0}] es [{1}].");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MINIMUM_5_CHARACTERS_REQUIRED', "É necessário digitar pelo menos 5 caracteres");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MINIMUM_5_CHARACTERS_REQUIRED', "You must enter at least 5 characters");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MINIMUM_5_CHARACTERS_REQUIRED', "Debes ingresar al menos 5 caracteres");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIN_QTY_NOT_REACHED', "A quantidade mínima do produto {0} não foi satisfeita. Quantidade mínima {1} quantidade selecionada {2}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIN_QTY_NOT_REACHED', "Invalid minimum quantity of product {0}. Minimum quantity {1} selected quantity {2}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIN_QTY_NOT_REACHED', "Cantidad mínima de producto inválida {0}. Cantidad mínima {1} cantidad seleccionada {2}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIN_QUANTITY', "Mín. ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIN_QUANTITY', "Min. ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIN_QUANTITY', "Mín. ({0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIN_QUANTITY_ITEM_NOT_REACHED', "A quantidade mínima do item {0} ({1}) → {2} ({3}) não foi alcançada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIN_QUANTITY_ITEM_NOT_REACHED', "$PENDING_TRADUCTION_MIN_QUANTITY_ITEM_NOT_REACHED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIN_QUANTITY_ITEM_NOT_REACHED', "La cantidad mínima del artículo {0} ({1}) → {2} ({3}) no fue alcanzada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIN_QUANTITY_OF_ITEMS_NOT_REACHED', "Quantidade mínima de itens não alcançada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIN_QUANTITY_OF_ITEMS_NOT_REACHED', "$PENDING_TRADUCTION_MIN_QUANTITY_OF_ITEMS_NOT_REACHED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIN_QUANTITY_OF_ITEMS_NOT_REACHED', "$PENDING_TRADUCTION_MIN_QUANTITY_OF_ITEMS_NOT_REACHED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIRROR', "Espelho");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIRROR', "Mirror");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIRROR', "Espejo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MIRROR_SCREEN', "Espelhar Tela");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MIRROR_SCREEN', "Mirror Screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MIRROR_SCREEN', "Pantalla de Espejo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MOBILE_PRINTER_STATUS', "Impressora Mobile");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MOBILE_PRINTER_STATUS', "Mobile Printer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MOBILE_PRINTER_STATUS', "Impresora móvil");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MOBILE_PRINTER_STATUS_FAIL', "Falha na Impressora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MOBILE_PRINTER_STATUS_FAIL', "Printer Failure");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MOBILE_PRINTER_STATUS_FAIL', "Fallo de la impresora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MODIFIERS', "Modificadores");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MODIFIERS', "Modifiers");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MODIFIERS', "Modificadores");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MODIFY', "Modificar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MODIFY', "Modify");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MODIFY', "Modificar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MONEY_CHANGE', "Troco: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MONEY_CHANGE', "Change: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MONEY_CHANGE', "Cambio: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'MORE', "a mais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'MORE', "more");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'MORE', "más");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NAME', "Nome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NAME', "Name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NAME', "Nombre");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NAME_CHALLENGE', "Nome do Desafio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NAME_CHALLENGE', "PENDING_TRADUCTION_NAME_CHALLENGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NAME_CHALLENGE', "Nombre del Desafío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NAME_LABEL', "Nome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NAME_LABEL', "Name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NAME_LABEL', "Nombre");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_APPLY_BENEFIT_PLEASE_WAIT', "Aplicando descontos. Por favor aguarde");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_APPLY_BENEFIT_PLEASE_WAIT', "PENDING_TRADUCTION_NDISCOUNT_APPLY_BENEFIT_PLEASE_WAIT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_APPLY_BENEFIT_PLEASE_WAIT', "PENDING_TRADUCTION_NDISCOUNT_APPLY_BENEFIT_PLEASE_WAIT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ASK_VALUE_ALREADY_APPLIED', "Desconto manual já aplicado ao pedido. \\Para aplicar um novo desconto manual é necessário remover o anterior");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ASK_VALUE_ALREADY_APPLIED', "PENDING_TRADUCTION_NDISCOUNT_ASK_VALUE_ALREADY_APPLIED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ASK_VALUE_ALREADY_APPLIED', "Descuento manual ya aplicado al pedido. \\Para aplicar un nuevo descuento manual es necesario remover el anterior");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_BENEFIT_NOT_FOUND', "Cupom inválido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_BENEFIT_NOT_FOUND', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_BENEFIT_NOT_FOUND', "¡Cupón inválido!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_CANNOT_ADD_ITEMS_IN_TOTALED_ORDER', "Não é possível aplicar cupom que adiciona itens no pedido totalizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_CANNOT_ADD_ITEMS_IN_TOTALED_ORDER', "PENDING_TRADUCTION_NDISCOUNT_CANNOT_ADD_ITEMS_IN_TOTALED_ORDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_CANNOT_ADD_ITEMS_IN_TOTALED_ORDER', "No es posible aplicar cupón que adicione artículos en el pedido totalizado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_CANNOT_ADD_ITEMS_ON_TOTALED_ORDER', "Esse cupom só pode ser aplicado na tela de pedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_CANNOT_ADD_ITEMS_ON_TOTALED_ORDER', "This coupon can only be applied on the order screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_CANNOT_ADD_ITEMS_ON_TOTALED_ORDER', "Este cupón solo se puede aplicar en la pantalla de pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_CIRCULAR_DEPENDENCY_TRYING_TO_APPLY_DISCOUNT', "Não é possível aplicar o desconto por motivo de dependência circular com outros descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_CIRCULAR_DEPENDENCY_TRYING_TO_APPLY_DISCOUNT', "Cannot apply discount by reason of circular dependency with other discounts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_CIRCULAR_DEPENDENCY_TRYING_TO_APPLY_DISCOUNT', "No se puede aplicar el descuento debido a dependencia circular con otros descuentos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_END_HOUR', "O horário para a aplicação desse desconto já terminou");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_END_HOUR', "PENDING_TRADUCTION_NDISCOUNT_END_HOUR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_END_HOUR', "Ya terminó el horario para la aplicación del descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_END_PERIOD', "O periodo de validade desse desconto já terminou");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_END_PERIOD', "PENDING_TRADUCTION_NDISCOUNT_END_PERIOD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_END_PERIOD', "El periodo de validez del descuento ya finalizó");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_BENEFIT_MISSING_SEAT', "Assento requerido para desconto não existe no pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_BENEFIT_MISSING_SEAT', "Seat required for discount does not exist on order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_BENEFIT_MISSING_SEAT', "El asiento requerido para el descuento no existe en el pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_BENEFIT_MISSING_VALUE', "O cupom requer valor de desconto (precisa ser aplicado pela opção Aplicar Desconto)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_BENEFIT_MISSING_VALUE', "The coupon requires discount value (needs to be apply by Apply Discount option)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_BENEFIT_MISSING_VALUE', "El cupón requiere valor de descuento (necesita se aplicado por la optión Aplicar Descuento)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_BRAND', "Marca não válida para a aplicação do desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_BRAND', "PENDING_TRADUCTION_NDISCOUNT_ERROR_BRAND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_BRAND', "Marca inválida para la aplicación del descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_MAX_DISCOUNT_SALE_AMOUNT', "A porcentagem máxima do desconto deve ser menor que {0}%");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_MAX_DISCOUNT_SALE_AMOUNT', "PENDING_TRADUCTION_NDISCOUNT_ERROR_MAX_DISCOUNT_SALE_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_MAX_DISCOUNT_SALE_AMOUNT', "El porcentaje máximo de descuento debe ser menor que {0}%");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_MAX_SALE_AMOUNT', "O valor do pedido é maior que o valor máximo permitido para aplicar o desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_MAX_SALE_AMOUNT', "PENDING_TRADUCTION_NDISCOUNT_ERROR_MAX_SALE_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_MAX_SALE_AMOUNT', "El valor del pedido es mayor que el valor máximo permitido para aplicar el descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_MIN_REQUIRED_SALE_AMOUNT', "Valor máximo de desconto para essa venda é de R$ {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_MIN_REQUIRED_SALE_AMOUNT', "PENDING_TRADUCTION_NDISCOUNT_ERROR_MIN_REQUIRED_SALE_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_MIN_REQUIRED_SALE_AMOUNT', "El valor máximo de descuento para esta venta es de R$ {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_MIN_REQUIRED_SEATS', "Necessária uma mesa com no mínimo {0} assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_MIN_REQUIRED_SEATS', "PENDING_TRADUCTION_NDISCOUNT_ERROR_MIN_REQUIRED_SEATS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_MIN_REQUIRED_SEATS', "Es necesaria una mesa con un mínimo de {0} asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_MIN_SALE_AMOUNT', "O valor do pedido é menor que o valor mínimo permitido para aplicar o desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_MIN_SALE_AMOUNT', "PENDING_TRADUCTION_NDISCOUNT_ERROR_MIN_SALE_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_MIN_SALE_AMOUNT', "El valor del pedido es menor que el valor mínimo permitido para aplicar el descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_POD_TYPE', "O desconto não pode ser aplicado nesse tipo de POD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_POD_TYPE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_POD_TYPE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_POD_TYPE', "El descuento no puede ser aplicado en este tipo de POD");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_REQUIRED_GROUPS_ON_SALE', "Items de grupos obrigatórios para aplicação do desconto não encontrados!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_REQUIRED_GROUPS_ON_SALE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_REQUIRED_GROUPS_ON_SALE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_REQUIRED_GROUPS_ON_SALE', "¡Artículos de grupos obligatorios para aplicar el descuento no encontrados!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_REQUIRED_ITEMS_ON_SALE', "Items obrigatórios para aplicação do desconto não encontrados!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_REQUIRED_ITEMS_ON_SALE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_REQUIRED_ITEMS_ON_SALE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_REQUIRED_ITEMS_ON_SALE', "¡Artículos obligatorios para aplicar el descuento no encontrados!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SALE_TYPE', "Cupom inválido para o tipo de venda!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SALE_TYPE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_SALE_TYPE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SALE_TYPE', "¡Cupón inválido para el tipo de venta!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SECTOR', "Cupom inválido para o setor da mesa/comanda!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SECTOR', "PENDING_TRADUCTION_NDISCOUNT_ERROR_SECTOR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SECTOR', "¡Cupón inválido para el sector de la mesa/comanda!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SECTOR_NOT_TABLE', "Cupom válidos apenas para mesas/comandas!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SECTOR_NOT_TABLE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_SECTOR_NOT_TABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SECTOR_NOT_TABLE', "¡Cupón válido solo para mesas/comandas!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SERVICE_TYPE_QUICK', "Cupom inválido para pedidos no balcão!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SERVICE_TYPE_QUICK', "PENDING_TRADUCTION_NDISCOUNT_ERROR_SERVICE_TYPE_QUICK");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SERVICE_TYPE_QUICK', "¡Cupón inválido para pedidos en balcón!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SERVICE_TYPE_TABLE', "Cupom inválido para pedidos de mesa!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SERVICE_TYPE_TABLE', "PENDIG_TRADUCTION_NDISCOUNT_ERROR_SERVICE_TYPE_TABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SERVICE_TYPE_TABLE', "¡Cupón inválido para pedidos de mesa!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_SINGLE_USE', "Não é possível aplicar o mesmo cupom novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_SINGLE_USE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_SINGLE_USE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_SINGLE_USE', "No se puede aplicar el mismo cupón nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_UNIQUE_FOR_GROUP_HAS_EXISTS', "Já existe um cupom do mesmo grupo aplicado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_UNIQUE_FOR_GROUP_HAS_EXISTS', "There is already a coupon from the same group applied");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_UNIQUE_FOR_GROUP_HAS_EXISTS', "Ya hay un cupón del mismo grupo aplicado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE', "Esse cupom só pode ser aplicado caso não exista nenhum outro tipo de cupom na venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE', "PENDING_TRADUCTION_NDISCOUNT_ERROR_UNIQUE_FOR_SALE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE', "Este cupón solo puede ser aplicado en caso no exista ningún otro tipo de cupón en la venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE_HAS_EXISTS', "Já existe um cupom de uso único na venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE_HAS_EXISTS', "There is already a single-use coupon on sale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_UNIQUE_FOR_SALE_HAS_EXISTS', "Ya hay un cupón de un solo uso a la venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_UNKNOWN', "Ocorreu um erro ao aplicar o cupom!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_UNKNOWN', "PENDING_TRADUCTION_NDISCOUNT_ERROR_UNKNOWN");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_UNKNOWN', "¡Ocurrió un error al aplicar el cupón!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ERROR_USER_MIN_LEVEL', "O operador não tem autorização para aplicar o desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ERROR_USER_MIN_LEVEL', "The operator is not authorized to apply the discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ERROR_USER_MIN_LEVEL', "El mesero no tiene autorización para aplicar el descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_INVALID_AMOUNT', "O valor de desconto do cupom é inválido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_INVALID_AMOUNT', "PENDING_TRADUCTION_NDISCOUNT_INVALID_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_INVALID_AMOUNT', "¡El valor de descuento del cupón es inválido!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_INVALID_ITEM_ADDITION_IN_TOTALED_ORDER', "Não é possível adicionar itens no pedido totalizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_INVALID_ITEM_ADDITION_IN_TOTALED_ORDER', "PENDING_TRADUCTION_NDISCOUNT_INVALID_ITEM_ADDITION_IN_TOTALED_ORDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_INVALID_ITEM_ADDITION_IN_TOTALED_ORDER', "No es posible agregar artículos en la orden totalizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_INVALID_PERCENTAGE', "A porcentagem de desconto do cupom é inválido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_INVALID_PERCENTAGE', "PENDING_TRADUCTION_NDISCOUNT_INVALID_PERCENTAGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_INVALID_PERCENTAGE', "¡El porcentaje de descuento del cupón es inválido!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_INVALID_SERVICE_PERIOD_ORDER', "Esse cupom não é aplicavel no Periodo de Serviço atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_INVALID_SERVICE_PERIOD_ORDER', "PENDING_TRADUCTION_NDISCOUNT_INVALID_SERVICE_PERIOD_ORDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_INVALID_SERVICE_PERIOD_ORDER', "Este cupón no se puede aplicar en el Periodo de Servicio actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_INVALID_TENDER_ID', "TenderId do cupom inválido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_INVALID_TENDER_ID', "PENDING_TRADUCTION_NDISCOUNT_INVALID_TENDER_ID");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_INVALID_TENDER_ID', "¡TenderId del cupón inválido!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_LIMITED_AMOUNT_DISCOUNT', "Valor informado maior que o máximo permitido. Desconto aplicado de R$ {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_LIMITED_AMOUNT_DISCOUNT', "Reported value greater than the maximum allowed. Applied discount of R$ {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_LIMITED_AMOUNT_DISCOUNT', "Valor informado mayor que el máximo permitido. Descuento aplicado de R$ {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_LIMITED_DISCOUNT', "Desconto limitado a R$ {0}. Informe um valor igual ou menor que R$ {0} para continuar.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_LIMITED_DISCOUNT', "Discount limited to BRL {0}. Enter an amount equal to or less than R$ {0} to continue.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_LIMITED_DISCOUNT', "Descuento limitado a BRL {0}. Introduzca un valor igual o inferior a R$ {0} para continuar.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_LIMITED_PERCENTAGE_DISCOUNT', "Desconto de {0}% limitado a R$ {1}. O desconto aplicado será de R$ {2}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_LIMITED_PERCENTAGE_DISCOUNT', "{0}% discount limited to BRL {1}. The discount applied will be BRL {2}.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_LIMITED_PERCENTAGE_DISCOUNT', "{0}% de descuento limitado a BRL {1}. El descuento aplicado será BRL{2}.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_MAX_DISCOUNT_APPLIED', "O valor máximo permitido para descontos já foi aplicado.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_MAX_DISCOUNT_APPLIED', "The maximum amount allowed for discounts has already been applied.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_MAX_DISCOUNT_APPLIED', "Ya se ha aplicado el importe máximo permitido para los descuentos.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_MAX_DISCOUNT_VALUE', "O valor máximo para utilização desta promoção é de R$ {0}. Informe um valor igual ou menor que R$ {1} para continuar.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_MAX_DISCOUNT_VALUE', "The maximum amount to use this promotion is BRL {0}. Enter an amount equal to or less than R$ {1} to continue.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_MAX_DISCOUNT_VALUE', "El monto máximo para utilizar esta promoción es BRL {0}. Introduzca un valor igual o inferior a R$ {1} para continuar.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_NOT_EFFECTIVE_DISCOUNT', "Esse cupom não tem descontos efetivos no valor total do pedido ou já foi aplicado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_NOT_EFFECTIVE_DISCOUNT', "PENDING_TRADUCTION_NDISCOUNT_NOT_EFFECTIVE_DISCOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_NOT_EFFECTIVE_DISCOUNT', "El cupón no tiene descuentos efectivos sobre el valor total del pedido o ya fue aplicado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_ORDER_NOT_FOUND', "Para aplicar esse cupon é necessário iniciar uma venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_ORDER_NOT_FOUND', "PENDIG_TRADUCTION_NDISCOUNT_ORDER_NOT_FOUND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_ORDER_NOT_FOUND', "Para aplicar cupones se necesita iniciar una venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_PRODUCT_NOT_FOUND', "O produto relacionado a este desconto não existe");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_PRODUCT_NOT_FOUND', "The product related to this discount does not exist");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_PRODUCT_NOT_FOUND', "El producto relacionado con este descuento no existe");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_SERVICE', "Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_SERVICE', "PENDING_TRADUCTION_NDISCOUNT_SERVICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_SERVICE', "Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_START_HOUR', "Horário para a aplicação do desconto não iniciado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_START_HOUR', "PENDING_TRADUCTION_NDISCOUNT_START_HOUR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_START_HOUR', "Aún no inicia el horario para la aplicación del descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_START_PERIOD', "Ainda não inícia o periodo de validade desse desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_START_PERIOD', "PENDING_TRADUCTION_NDISCOUNT_START_PERIOD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_START_PERIOD', "Aún no inicia el periodo de validez del descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_VALID_DAYS', "O desconto não pode ser aplicado nesse dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_VALID_DAYS', "PENDING_TRADUCTION_NDISCOUNT_VALID_DAYS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_VALID_DAYS', "El descuento no se puede aplicar en este día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_VERIFYING_BENEFIT_PLEASE_WAIT', "Verificando descont");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_VERIFYING_BENEFIT_PLEASE_WAIT', "PENDING_TRADUCTION_NDISCOUNT_VERIFYING_BENEFIT_PLEASE_WAIT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_VERIFYING_BENEFIT_PLEASE_WAIT', "Verificando descuent");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NDISCOUNT_WITHOUT_PRODUCTS_FOR_SERVICE_PERIOD_DISCOUNT', "O pedido não possui itens com o Período de Serviço do cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NDISCOUNT_WITHOUT_PRODUCTS_FOR_SERVICE_PERIOD_DISCOUNT', "PENDING_TRADUCTION_NDISCOUNT_WITHOUT_PRODUCTS_FOR_SERVICE_PERIOD_DISCOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NDISCOUNT_WITHOUT_PRODUCTS_FOR_SERVICE_PERIOD_DISCOUNT', "El pedido no tiene artículos con el Periodo de Servicio del cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NECESSARY_AUTHORIZATION', "Necessário autorização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NECESSARY_AUTHORIZATION', "Authorization needed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NECESSARY_AUTHORIZATION', "Autorización necesaria");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NECESSARY_BLEED', "Necessario fazer a Sangria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NECESSARY_BLEED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NECESSARY_BLEED', "$PENDING_TRADUCTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NECESSARY_UPDATE_VERSION_MOBILE', "Necessário atualizar o PDV Mobile para versão {0}. Em caso de dúvidas consultar o suporte.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NECESSARY_UPDATE_VERSION_MOBILE', "Need to update Mobile POS to version {0}. If in doubt, consult support.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NECESSARY_UPDATE_VERSION_MOBILE', "Es necesario actualizar el TPV móvil a la versión {0}. En caso de duda, consulte con soporte.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_START_SALE', "Necessário iniciar a venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_START_SALE', "Start order first");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_START_SALE', "Iniciar orden primero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_TO_CLOSE_ALL_TABLES_AND_TABS', "Necessário fechar todas as mesas e comandas.\\Deseja visualizar as mesas abertas?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_TO_CLOSE_ALL_TABLES_AND_TABS', "It is necessary to close all tables and pagers.\\Show opened tables?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_TO_CLOSE_ALL_TABLES_AND_TABS', "Es necesario cerrar todas las mesas y comandas.\\Mostrar mesas abiertas?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_TO_CLOSE_DAY_IN_ALL_POS', "Necessário fechar o dia de negócio em todos os PDV's.\\PDV's com dia aberto: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_TO_CLOSE_DAY_IN_ALL_POS', "Close business day in all POS's.\\POS's opened: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_TO_CLOSE_DAY_IN_ALL_POS', "Elija el día de negocio en todos los TPV.\\TPV abiertos: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_TO_FINISH_ORDER', "Necessário finalizar o pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_TO_FINISH_ORDER', "It is necessary to close the order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_TO_FINISH_ORDER', "Es necesario cerrar la cuenta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_TO_HAVE_ITEM', "Necessário selecionar uma linha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_TO_HAVE_ITEM', "Select a line first");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_TO_HAVE_ITEM', "Selecciones una línea primero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEED_TO_OPEN_DAY_FIRST', "Necessário abrir o dia de négocio primeiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEED_TO_OPEN_DAY_FIRST', "The business day must be opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEED_TO_OPEN_DAY_FIRST', "El día de negocio debe estár abierto primero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEIGHBORHOOD', "Bairro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEIGHBORHOOD', "Neighborhood");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEIGHBORHOOD', "Sección");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEIGHBORHOOD_DELIVERY_REPORT', "Bairro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEIGHBORHOOD_DELIVERY_REPORT', "Address 3");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEIGHBORHOOD_DELIVERY_REPORT', "Sector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST', "Teste de Rede");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST', "Network Test");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST', "Prueba de red");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST_CLOSE', "Fechar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST_CLOSE', "Close");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST_CLOSE', "Cerrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST_CONTINUOUS_TEST', "Teste Contínuo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST_CONTINUOUS_TEST', "Continuous Test");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST_CONTINUOUS_TEST', "Prueba continua");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST_RESPONSE_TIME', "Tempo de Resposta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST_RESPONSE_TIME', "Response Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST_RESPONSE_TIME', "Tiempo de respuesta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST_SERVER_REPORTED_RESPONSE_TIME', "Tempo de Resposta Reportado pelo Servidor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST_SERVER_REPORTED_RESPONSE_TIME', "Server Reported Response Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST_SERVER_REPORTED_RESPONSE_TIME', "Tiempo de respuesta informado por el servidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NETWORK_TEST_SINGLE_TEST', "Teste Único");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NETWORK_TEST_SINGLE_TEST', "Single Test");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NETWORK_TEST_SINGLE_TEST', "Prueba única");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEW_CHANGE', "Novo troco");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEW_CHANGE', "New change");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEW_CHANGE', "Nuevo Cambio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEW_PASSWORD', "Nova senha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEW_PASSWORD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEW_PASSWORD', "Nueva contraseña");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEW_PRICE_OR_ZERO', "Insira o preço ou 0 para preço original");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEW_PRICE_OR_ZERO', "Insert price or 0 for default price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEW_PRICE_OR_ZERO', "Ingresar precio o 0 por precio predetermindo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEW_PRICE_TO_APPLY', "Digite o novo preço deste item.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEW_PRICE_TO_APPLY', "Inform the new item price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEW_PRICE_TO_APPLY', "Informar el precio de nuevo artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEW_TAB', "Nova Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEW_TAB', "New Pager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEW_TAB', "Nueva Comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NEXT', "PRÓXIMO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NEXT', "Next");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NEXT', "Siguiente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NINTH_READING', "Nona");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NINTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NINTH_READING', "Novena");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO', "Não");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO', "NO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO', "No");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NONE', "Nenhum");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NONE', "None");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NONE', "Ninguno");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_CONNECTED_ERROR', "Sem conexão com a impressora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_CONNECTED_ERROR', "No connection to the printer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_CONNECTED_ERROR', "Sin conexión a la impresora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_DEFINED', "Indefinido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_DEFINED', "Undefined");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_DEFINED', "Indefinido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_FOUND', "Não encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_FOUND', "No encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_INFORM', "Não informar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_INFORM', "Do not inform");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_INFORM', "No informar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_INFORMED', "Não informado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_INFORMED', "Not Informed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_INFORMED', "No informado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_INSIDE_UPDATE_WINDOW', "Fora da janela de atualização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_INSIDE_UPDATE_WINDOW', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_INSIDE_UPDATE_WINDOW', "Fuera de la ventana de actualización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_ORDER_PENDING_FOUND', "Nenhum pedido pendente encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_ORDER_PENDING_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_ORDER_PENDING_FOUND', "Ningún pedido pendiente encontrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_POSSIBLE_TO_CANCEL_DELIVERY', "Venda passou do horário permitido de cancelamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_POSSIBLE_TO_CANCEL_DELIVERY', "Time exceded to cancel the order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_POSSIBLE_TO_CANCEL_DELIVERY', "Tiempo excedido para cancelar pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_VALID_TABLE_OPERATION_MODE', "Modo de operação de Mesa não válido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_VALID_TABLE_OPERATION_MODE', "Invalid table operation mode.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_VALID_TABLE_OPERATION_MODE', "Modo de operación de Mesa inválido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NOT_VERIFIED_USER_FINGERPRINT', "Usuário não identificado a partir da digital. Tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NOT_VERIFIED_USER_FINGERPRINT', "Fingerprint unkwon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NOT_VERIFIED_USER_FINGERPRINT', "Huella no identificada. Intente nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_CANCELLATION_ERRORS', "Nenhum erro de cancelamento encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_CANCELLATION_ERRORS', "Cancellation errors not found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_CANCELLATION_ERRORS', "Ningún error de cancelamiento hallado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_COURSE_AVAILABLE', "Nenhum curso disponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_COURSE_AVAILABLE', "PENDING_TRADUCTION_NO_COURSE_AVAILABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_COURSE_AVAILABLE', "Ningún curso disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_DEVICES_TO_UPDATE', "Não há dispositivos para atualizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_DEVICES_TO_UPDATE', "There are no devices to update");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_DEVICES_TO_UPDATE', "No hay dispositivos para actualizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_GOALS_AVAILABLE', "Nenhuma meta cadastrada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_GOALS_AVAILABLE', "No goals available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_GOALS_AVAILABLE', "Sin metas disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_LINE_SELECTED', "Nenhuma linha selecionada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_LINE_SELECTED', "No line selected");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_LINE_SELECTED', "Ninguna línea seleccionada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_NAVIGATION_AVAILABLE', "Navegação Indisponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_NAVIGATION_AVAILABLE', "Unavailable navigation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_NAVIGATION_AVAILABLE', "Navegación No Disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_OPTIONS_TO_SHOW', "Nenhuma Mesa/Comanda a ser exibida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_OPTIONS_TO_SHOW', "No Table/Pager to be displayed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_OPTIONS_TO_SHOW', "No se encuentra mesa/Se mostrará comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_ORDERS_FOUND', "Nenhum pedido encontrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_ORDERS_FOUND', "No order found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_ORDERS_FOUND', "No se encontró ninguna orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_ORDER_FOUND_TO_CANCEL', "Nenhum pedido encontrado para cancelamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_ORDER_FOUND_TO_CANCEL', "No order found");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_ORDER_FOUND_TO_CANCEL', "No se encontró ninguna orden para cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_ORDER_TO_SLICE', "Sem pedidos para separar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_ORDER_TO_SLICE', "No orders to split");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_ORDER_TO_SLICE', "No existen órdenes para separar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_PAPER_ERROR', "Impressora sem papel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_PAPER_ERROR', "paperless printer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_PAPER_ERROR', "impresora sin papel");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_PAYMENT_DETECTED', "Nenhum pagamento detectado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_PAYMENT_DETECTED', "No payments detected");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_PAYMENT_DETECTED', "Ningún pago detectado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_REPORT_DATA', "Sem dados de relatório");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_REPORT_DATA', "No report data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_REPORT_DATA', "Sin datos de informe");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_RUPTURE_ITEMS', "Não há items cadastrados nessa funcionalidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_RUPTURE_ITEMS', "No items registered in this feature");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_RUPTURE_ITEMS', "No hay artículos registrados en esta funcionalidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_SCANNER_CONFIGURED', "Scanner não configurado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_SCANNER_CONFIGURED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_SCANNER_CONFIGURED', "Scanner no configurado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_SEAT', "Sem Assento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_SEAT', "Items without seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_SEAT', "Artículos sin asiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_TABLE_FOUND', "Mesa não encontrada!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_TABLE_FOUND', "Table not found!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_TABLE_FOUND', "Mesa no encontrada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NO_WITHOUT_ICON', "Não");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NO_WITHOUT_ICON', "No");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NO_WITHOUT_ICON', "No");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NSU_HOST', "NsuHost");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NSU_HOST', "NsuHost");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NSU_HOST', "NsuHost");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'NUMBER', "Número");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'NUMBER', "Number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'NUMBER', "Número");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OBSERVATION', "Observação:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OBSERVATION', "Observation:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OBSERVATION', "Observación:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OBSERVATION_DELIVERY', "Pedidos com observações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OBSERVATION_DELIVERY', "Order Commentaries");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OBSERVATION_DELIVERY', "Comentarios de Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OBSERVATION_DELIVERY_REPORT', "Observações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OBSERVATION_DELIVERY_REPORT', "Commentaries");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OBSERVATION_DELIVERY_REPORT', "Comentarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OFFLINE', "Offline");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OFFLINE', "Offline");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OFFLINE', "Fuera de línea");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OK', "Ok");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OK', "Ok");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OK', "Ok");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ONLINE', "Online");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ONLINE', "Online");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ONLINE', "En línea");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ONLY', "apenas por");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ONLY', "only for");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ONLY', "solo por");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ONLY_NUMBER', "Apenas números");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ONLY_NUMBER', "Only numbers");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ONLY_NUMBER', "Solo números");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ONLY_NUMBERS', "(somente números)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ONLY_NUMBERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ONLY_NUMBERS', "(solamente números)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ONSIDE', "(Ao Lado)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ONSIDE', "(Side)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ONSIDE', "(Aparte)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPENED', "Aberto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPENED', "Opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPENED', "Abierto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPENED_TABLES', "Relatório de Mesas Abertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPENED_TABLES', "Opened Tables Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPENED_TABLES', "Reporte de Mesas Abiertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPEN_CLOSE_TOTEM', "Abrir/Fechar Totem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPEN_CLOSE_TOTEM', "Open/Close Totem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPEN_CLOSE_TOTEM', "Abrir/Cerrar Tótem");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPEN_DAY', "Abrir o Dia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPEN_DAY', "Open Day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPEN_DAY', "Abrir el Día");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPEN_DRAWER', "Abrir Gaveta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPEN_DRAWER', "Open Drawer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPEN_DRAWER', "Abrir Gaveta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPEN_TAB', "Abrir Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPEN_TAB', "$PENDING_TRADUCTION_OPEN_TAB");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPEN_TAB', "Abrir Comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPEN_TABLE', "Abrir Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPEN_TABLE', "Open Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPEN_TABLE', "Mesa Abierta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATION_CANCELLED_BY_OPERATOR', "Operação cancelada pelo operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATION_CANCELLED_BY_OPERATOR', "Operation canceled by operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATION_CANCELLED_BY_OPERATOR', "Operación cancelada por el operador");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATION_FAILURE', "A operação falhou. Tente novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATION_FAILURE', "Operation failed. Try again");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATION_FAILURE', "Operación fallida. Intente nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR', "Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR', "Operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR', "Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATORS', "Operadores");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATORS', "Operators");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATORS', "Operadoers");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATORS_BY_SECTOR_UPDATE_SUCCESS', "Atualização de operadores por setores realizada com sucesso.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATORS_BY_SECTOR_UPDATE_SUCCESS', "Successful sector operators update.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATORS_BY_SECTOR_UPDATE_SUCCESS', "Actualización de operadores por sectores realizada con éxito.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_ALREADY_LOGGED', "Operador já está logado. PDV ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_ALREADY_LOGGED', "Operator already logged. POS ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_ALREADY_LOGGED', "El Mesero ya ha iniciado sesión. TPV ({0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_CLOSING', "Fechamento de Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_CLOSING', "Operator clousure");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_CLOSING', "Cierre de Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_DOES_NOT_BELONG_TO_TABLE_SECTOR', "O Operador não pertence no setor de atendimento da mesa/comanda.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_DOES_NOT_BELONG_TO_TABLE_SECTOR', "The Operator does not belong to the service table/tab sector.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_DOES_NOT_BELONG_TO_TABLE_SECTOR', "El Operador no pertenece al sector de atedimiento de la mesa/comanda.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_IS_NOT_OPENED', "Operador não está aberto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_IS_NOT_OPENED', "Operator not opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_IS_NOT_OPENED', "Mesero no abierto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_IS_PERFORMING_AN_OPEN_OR_CLOSE_ACTION', "Este usuário já está executando uma ação de abertura ou fechamento de operador em outro PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_IS_PERFORMING_AN_OPEN_OR_CLOSE_ACTION', "This user is already performing an operator opening or closing action on another POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_IS_PERFORMING_AN_OPEN_OR_CLOSE_ACTION', "Este usuario ya está realizando una acción de apertura o cierre de operador en otro TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_LABEL', "Operador:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_LABEL', "Operator:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_LABEL', "Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPERATOR_OPENING', "Abertura Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPERATOR_OPENING', "Operator Opening");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPERATOR_OPENING', "Apertura de Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OPTIONS', "Opções");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OPTIONS', "Options");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OPTIONS', "Opciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OR', "ou");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OR', "or");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OR', "o");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER', "Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER', "Orden");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDERS_COUNT', "Total de pedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDERS_COUNT', "PENDING_TRADUCTION_ORDERS_COUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDERS_COUNT', "PENDING_TRADUCTION_ORDERS_COUNT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_ALREADY_PRODUCED', "Pedido já produzido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_ALREADY_PRODUCED', "Produced order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_ALREADY_PRODUCED', "Orden Producida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_CALLED', "Pedidos já chamados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_CALLED', "Called Orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_CALLED', "Pedidos llamados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_CANCELLED_NO_REFUND', "O pedido será cancelado, mas os valores pagos não serão estornados. Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_CANCELLED_NO_REFUND', "The order will be cancelled, but the amounts paid will not be refunded. Do you wish to continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_CANCELLED_NO_REFUND', "El pedido será cancelado, pero no se devolverán los importes abonados. ¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_DELIVERY_PAYMENT', "Forma de pagamento: Na Entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_DELIVERY_PAYMENT', "PENDING_TRADUCTION_ORDER_DELIVERY_VALUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_DELIVERY_PAYMENT', "Forma de pago: En la Entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_DELIVERY_REPORT', "Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_DELIVERY_REPORT', "Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_DELIVERY_REPORT', "Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_DELIVERY_VALUE', "Valor da Entrega: R$ {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_DELIVERY_VALUE', "PENDING_TRADUCTION_ORDER_DELIVERY_VALUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_DELIVERY_VALUE', "Valor de la Entrega: $ {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_DETAILS', "Informações do Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_DETAILS', "Order Information");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_DETAILS', "Información del Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_DISCOUNT_DELIVERY_REPORT', "Desconto do pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_DISCOUNT_DELIVERY_REPORT', "Order discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_DISCOUNT_DELIVERY_REPORT', "Descuento de Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_HAS_NO_APPLIED_DISCOUNT', "Não há nenhum desconto aplicado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_HAS_NO_APPLIED_DISCOUNT', "PENDING_TRADUCTION_ORDER_HAS_NO_APPLIED_COUPONS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_HAS_NO_APPLIED_DISCOUNT', "No hay ningún descuento aplicado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_HAS_NO_VALUE', "Pedido não possui valor de compra!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_HAS_NO_VALUE', "Order has no value!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_HAS_NO_VALUE', "¡Orden sin valor!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_HAS_TENDERS', "Pedido possui pagamentos abertos. Apague os pagamentos e tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_HAS_TENDERS', "Order has open Tenders. Remove the tenders and try again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_HAS_TENDERS', "La orden tiene cuentas abiertas. Borre los pagos e intente nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_HAS_TENDERS_CANNOT_SAVE', "Pedido não pode ser salvo. Possui pagamentos abertos.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_HAS_TENDERS_CANNOT_SAVE', "Order cannot be saved. Has open Tenders.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_HAS_TENDERS_CANNOT_SAVE', "La orden no se pudo guardar. Tiene cuentas abiertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_NOT_FINALIZED_DUE_TO_MAX_SAT_PAYMENT_GROUPS_REACHED', "A venda não pode ser finalizada por ter mais de {0} formas de pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_NOT_FINALIZED_DUE_TO_MAX_SAT_PAYMENT_GROUPS_REACHED', "PENDING_TRADUCTION_ORDER_NOT_FINALIZED_DUE_TO_MAX_SAT_PAYMENT_GROUPS_REACHED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_NOT_FINALIZED_DUE_TO_MAX_SAT_PAYMENT_GROUPS_REACHED', "La venta no puede ser finalizada por tener más de {0} formas de pagos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_NOT_PRODUCED', "Pedido ainda não produzido. Aguardando produção manual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_NOT_PRODUCED', "Order not produced. Waiting for manual production.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_NOT_PRODUCED', "Pedido no producido. Esperando producción manual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_NUMBER', "Pedido: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_NUMBER', "Order Number: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_NUMBER', "Número de Orden: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_OBSERVATIONS', "Observações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_OBSERVATIONS', "Observations");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_OBSERVATIONS', "Observaciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_PICKLIST_SUCCESSFULLY_PRINTED', "Picklist do Pedido {0} reimpressa com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_PICKLIST_SUCCESSFULLY_PRINTED', "Order Picklist {0} successfully reprinted");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_PICKLIST_SUCCESSFULLY_PRINTED', "Picklist del Pedido {0} reimpresa con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_PREVIEW', "Detalhes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_PREVIEW', "Details");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_PREVIEW', "Detalles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_PRODUCED', "Pedido produzido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_PRODUCED', "Produced order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_PRODUCED', "Orden preparada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_READY', "Pedidos prontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_READY', "Orders Ready");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_READY', "Pedidos Listos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_RETRIEVED_ANOTHER_CASH', "Pedido número {0} já recuperado em outro caixa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_RETRIEVED_ANOTHER_CASH', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_RETRIEVED_ANOTHER_CASH', "Pedido número {0} ya recuperado en otra caja");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_SAVED', "Pedido salvo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_SAVED', "Order saved");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_SAVED', "Orden guardada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY', "Pedido salvo em dia de negócio anterior ao atual. Recupere-o em outro caixa ou apague-o");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY', "Pedido guardado en un día laboral anterior al actual. Recupérelo en otra caja o bórrelo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY_OPEN_NEW_DAY', "Pedido foi salvo em um dia de negócio posterior ao atual. Por favor abra um novo dia de negócio.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY_OPEN_NEW_DAY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_SAVED_ON_PREVIOUS_BUSINESS_DAY_OPEN_NEW_DAY', "Pedido fue guardado en un día laboral posterior al actual. Por favor abra un nuevo día laboral.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_STORE_REMOTE_ID', "Pedido\(Loja / Remoto)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_STORE_REMOTE_ID', "Order\(Store / Remote)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_STORE_REMOTE_ID', "Pedido\(Tienda / Remoto)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_SUMMARY', "RESUMO DO PEDIDO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_SUMMARY', "ORDER SUMMARY");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_SUMMARY', "RESUMEN DE ORDEN");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_VOIDED_BY_OPERATOR', "PEDIDO CANCELADO VIA OPERADOR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_VOIDED_BY_OPERATOR', "ORDER CANCELED BY OPERATOR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_VOIDED_BY_OPERATOR', "CANCELADO POR MESERO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ORDER_VOIDED_WITH_SUCCESS', "Pedido cancelado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ORDER_VOIDED_WITH_SUCCESS', "Order canceled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ORDER_VOIDED_WITH_SUCCESS', "Orden cancelada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OTHER', "Outro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OTHER', "Other");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OTHER', "Otro");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OTHERS', "Outros");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OTHERS', "Others");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OTHERS', "Otros");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OTHER_DISCOUNTS', "Outros Descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OTHER_DISCOUNTS', "Other Discounts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OTHER_DISCOUNTS', "Otros Descuentos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OVERWRITE_PRICE', "Mudar Preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OVERWRITE_PRICE', "Change Price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OVERWRITE_PRICE', "Cambiar Precio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'OVERWRITE_PRICE_WITHOUT_VALUE', "Não há valores para mudança de preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'OVERWRITE_PRICE_WITHOUT_VALUE', "There are no values for price change");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'OVERWRITE_PRICE_WITHOUT_VALUE', "No hay valores para cambiar precio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAGER_SYMBOL', "C-");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAGER_SYMBOL', "P-");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAGER_SYMBOL', "C-");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAID', "Pago");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAID', "Paid");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAID', "Pagado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PARTIAL', "Parcial");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PARTIAL', "Partial");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PARTIAL', "Parcial");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PARTICIPATE_ROUND_PROJECT_BODY', "Você gostaria de arredondar {0}{1} centavos para apoiar projetos de educação?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PARTICIPATE_ROUND_PROJECT_BODY', "Would you like to donate {0}{1} to social projects?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PARTICIPATE_ROUND_PROJECT_BODY', "Quisiera donar {0}{1} a proyectos sociales?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PARTNER', "Parceiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PARTNER', "Partner");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PARTNER', "Socio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PARTNER_CONTACT_NUMBER', "Contato Parceiro:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PARTNER_CONTACT_NUMBER', "Partner contact:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PARTNER_CONTACT_NUMBER', "Contacto de Socio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PART_CODE_MIN_QTY_NOT_REACHED', "A quantidade mínima de opções do item {} ({}) não foi alcançada. Quantidade mínima: {}. Quantidade selecionada: {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PART_CODE_MIN_QTY_NOT_REACHED', "PENDING_TRADUCTION_PART_CODE_MIN_QTY_NOT_REACHED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PART_CODE_MIN_QTY_NOT_REACHED', "PENDING_TRADUCTION_PART_CODE_MIN_QTY_NOT_REACHED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PART_CODE_NOT_A_PRODUCT', "O partCode {0} não é um produto opção ou combo.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PART_CODE_NOT_A_PRODUCT', "The partCode {0} is not a product option or combo.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PART_CODE_NOT_A_PRODUCT', "El partCode {0} no es una opción de producto o combo.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PART_CODE_NOT_A_VALID_SON', "O partCode {0} ({1}) não é um filho válido de {2} ({3})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PART_CODE_NOT_A_VALID_SON', "The partCode {0} ({1}) is not a valid child of {2} ({3})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PART_CODE_NOT_A_VALID_SON', "El partCode {0} ({1}) no es un elemento secundario valido de {2} ({3})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PASSWORDS_DONT_MATCH', "Senhas não conferem. Repita a operação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PASSWORDS_DONT_MATCH', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PASSWORDS_DONT_MATCH', "Las contraseñas no coinciden. Repita la operación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PASSWORD_CHANGED_SUCCESSFULLY', "Senha alterada com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PASSWORD_CHANGED_SUCCESSFULLY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PASSWORD_CHANGED_SUCCESSFULLY', "¡Contraseña cambiada con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAUSED', "Pausado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAUSED', "Paused");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAUSED', "En Pausa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAY', "Pagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAY', "PAY");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAY', "Pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENTS_NOT_DISTRIBUTED', "Existem pagamentos parciais no pedido que não foram distribuídos entre os assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENTS_NOT_DISTRIBUTED', "There are partial payments on the order that have not been allocated between seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENTS_NOT_DISTRIBUTED', "Hay pagos parciales del pedido que no se han repartido entre plazas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENT_CANCELED', "Pagamento Cancelado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENT_CANCELED', "PENDING_TRADUCTION_PAYMENT_CANCELED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENT_CANCELED', "Pago Cancelado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENT_CANCELED_CONFIRMATION', "Existem pagamentos na mesa que serão estornados. Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENT_CANCELED_CONFIRMATION', "There are payments on the table that will be reversed. Continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENT_CANCELED_CONFIRMATION', "PENDING_TRADUCTION_PAYMENT_CANCELED_CONFIRMATION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENT_HAS_NO_TEF', "Pagamento não possui comprovante ou comprovante não pode ser reimpresso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENT_HAS_NO_TEF', "Payment does not have receipt or receipt cannot be reprinted");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENT_HAS_NO_TEF', "El pago no tiene recibo o el recibo no se puede reimprimir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENT_NOT_IMPLEMENTED', "Forma de pagamento não implementada: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENT_NOT_IMPLEMENTED', "Invalid payment method: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENT_NOT_IMPLEMENTED', "Forma de pago inválida: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PAYMENT_REPORT', "Pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PAYMENT_REPORT', "Payments");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PAYMENT_REPORT', "Pagos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PENDING_UPDATE', "Pendente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PENDING_UPDATE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PENDING_UPDATE', "Pendiente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PERCENTAGE', "Porcentagem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PERCENTAGE', "Percentage");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PERCENTAGE', "Porcentaje");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PERIOD', "Periodo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PERIOD', "Period");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PERIOD', "Periodo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PERIOD_GREATER_THAN_THIRTY_DAYS', "Não é possivel consultar datas com mais de 30 dias");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PERIOD_GREATER_THAN_THIRTY_DAYS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PERIOD_GREATER_THAN_THIRTY_DAYS', "No es posible consultar fechas con más de 30 días");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PHONE_DELIVERY_REPORT', "Telefone");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PHONE_DELIVERY_REPORT', "Phone");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PHONE_DELIVERY_REPORT', "Teléfono");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PICKLIST', "Picklist");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PICKLIST', "Picklist");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PICKLIST', "Picklist");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PICKUP', "Pickup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PICKUP', "Pickup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PICKUP', "Pickup");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PICKUP_TYPE', "Retirada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PICKUP_TYPE', "Pickup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PICKUP_TYPE', "Retiro");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PINPAD', "Pinpad");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PINPAD', "Pinpad");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PINPAD', "Pinpad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PLACE_FINGER_AUTHORIZATION', "Posicione o dedo no leitor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PLACE_FINGER_AUTHORIZATION', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PLACE_FINGER_AUTHORIZATION', "Posicione el dedo en el lector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PLACE_FINGER_LOGIN', "Login de usuario");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PLACE_FINGER_LOGIN', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PLACE_FINGER_LOGIN', "Login de usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PLACE_RIGHT_INDEX_FINGER', "Posicione o indicador direito no leitor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PLACE_RIGHT_INDEX_FINGER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PLACE_RIGHT_INDEX_FINGER', "Posicione el índice derecho en el lector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PLEASE_WAIT', "Por favor aguarde...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PLEASE_WAIT', "Please wait ...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PLEASE_WAIT', "Por favor espere...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PLU', "PLU");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PLU', "PLU");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PLU', "PLU");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POSTAL_CODE_DELIVERY_REPORT', "CEP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POSTAL_CODE_DELIVERY_REPORT', "Zipcode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POSTAL_CODE_DELIVERY_REPORT', "Código Postal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_BLOCKED', "Este PDV está bloqueado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_BLOCKED', "This POS is blocked");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_BLOCKED', "Este TPV está bloqueado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_BLOCKED_NEED_CLOSE_DAY', "Este PDV está bloqueado\\Necessário fechar o dia de negócio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_BLOCKED_NEED_CLOSE_DAY', "This POS is blocked\\Need to close the business day");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_BLOCKED_NEED_CLOSE_DAY', "Este TPV está bloqueado\\Necesita cerrar el día hábil");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_CANNOT_OPEN_THIS_TABLE', "Este PDV não pode abrir Mesas Totalizadas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_CANNOT_OPEN_THIS_TABLE', "POS cannot open totalized tables");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_CANNOT_OPEN_THIS_TABLE', "Este TPV no pudo abrir mesas totalizadas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_NOT_FOUND', "PDV digitado não existe");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_NOT_FOUND', "POS entered does not exist");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_NOT_FOUND', "TPV ingresado no existe");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_NUMBER_INVALID', "Numero de PDV inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_NUMBER_INVALID', "Invalid POS number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_NUMBER_INVALID', "Número de TPV inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'POS_USERS_UPDATED', "Usuários atualizados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'POS_USERS_UPDATED', "POS_USERS_UPDATED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'POS_USERS_UPDATED', "Usuarios actualizados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PREPAID_ORDER', "PAGO ONLINE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PREPAID_ORDER', "PAYED ONLINE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PREPAID_ORDER', "PAGADO EN LÍNEA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PREP_MODE_DONE', "Prontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PREP_MODE_DONE', "Ready");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PREP_MODE_DONE', "Listo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PREP_MODE_HELD', "Aguardando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PREP_MODE_HELD', "Waiting");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PREP_MODE_HELD', "Esperando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PREP_MODE_NORMAL', "Em Preparo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PREP_MODE_NORMAL', "In Production");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PREP_MODE_NORMAL', "En Preparación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PREVIOUS', "Voltar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PREVIOUS', "Previous");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PREVIOUS', "Previo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRE_ACCOUNT_BY_SEAT_OR_BY_TABLE', "Pré-Conta por MESA ou por ASSENTOS?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRE_ACCOUNT_BY_SEAT_OR_BY_TABLE', "Bill by TABLE or by SEAT?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRE_ACCOUNT_BY_SEAT_OR_BY_TABLE', "¿Reporte de cuenta por MESA o por ASIENTO?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRE_ACCOUNT_BY_TABS_OR_BY_TABLE', "Pré-Conta por MESA ou por COMANDAS?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRE_ACCOUNT_BY_TABS_OR_BY_TABLE', "PENDING_TRADUCTION_PRE_ACCOUNT_BY_TABS_OR_BY_TABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRE_ACCOUNT_BY_TABS_OR_BY_TABLE', "¿Reporte de cuenta por MESA o por COMANDA?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRICE', "Preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRICE', "Price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRICE', "Precio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRICED', "Com preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRICED', "Priced");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRICED', "Con precio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRINTER', "Impressora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRINTER', "Printer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRINTER', "Impresora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRINTER_DONATION_ERROR', "Não foi possível imprimir o cupom: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRINTER_DONATION_ERROR', "Could not print coupon: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRINTER_DONATION_ERROR', "No se pudo imprimir el cupón: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRINTER_ERROR', "Erro na impressora: Verifique o papel e a conexão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRINTER_ERROR', "Printer error: Check paper and connection");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRINTER_ERROR', "Error de impresora: Comprobar papel y conexión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRINTER_STATUS', "Status da Impressora");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRINTER_STATUS', "Printer Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRINTER_STATUS', "Estado de la Impresora");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRINT_DELIVERY', "Imprimir Detalhes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRINT_DELIVERY', "Print Details");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRINT_DELIVERY', "Detalles de Impresión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROCESSING', "Processando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROCESSING', "Processing");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROCESSING', "Procesando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROCESSING_FISCAL_DATA', "Processando cupom fiscal por favor aguarde...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROCESSING_FISCAL_DATA', "Processing fiscal data please wait...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROCESSING_FISCAL_DATA', "Procesando datos fiscales por favor espere...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROCESSING_PAYMENT_DATA', "Processando pagamento por favor aguarde...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROCESSING_PAYMENT_DATA', "Processing payment please wait...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROCESSING_PAYMENT_DATA', "Procesando pago por favor espere...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROCESSING_PAYMENT_SEAT', "Processando pagamento para assento: {0}...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROCESSING_PAYMENT_SEAT', "Processing payment for seat: {0}...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROCESSING_PAYMENT_SEAT', "Procesando pago para asiento: {0}...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCE_DELIVERY_ORDER', "Produzir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCE_DELIVERY_ORDER', "Produce");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCE_DELIVERY_ORDER', "Producir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCTS', "Produtos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCTS', "PENDING_TRADUCTION_PRODUCTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCTS', "Productos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCTS_RESUME', "Resumo dos Produtos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCTS_RESUME', "Products Summary");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCTS_RESUME', "Resumen de Productos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_CODE', "Código do Produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_CODE', "Product Code");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_CODE', "Código de Producto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_DESCRIPTION', "Descrição do Produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_DESCRIPTION', "Product Description");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_DESCRIPTION', "Descripción de Producto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_DOESNT_EXISTS', "Produto {} não existe no MENU da Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_DOESNT_EXISTS', "Product {} does not exist in the Store MENU");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_DOESNT_EXISTS', "El producto {} no existe en el MENÚ de la Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_DOES_NOT_EXISTS', "O produto não existe");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_DOES_NOT_EXISTS', "The product does not exist");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_DOES_NOT_EXISTS', "El producto no existe");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_MIX_BUSINESS_PERIOD', "Relatório de Mix de Produtos\(Data de Negócio)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_MIX_BUSINESS_PERIOD', "Product Mix Report\(Business Date)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_MIX_BUSINESS_PERIOD', "Reporte de Mix de Productos\(Fecha de Negocio)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PRODUCT_NAME', "Nome do Produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PRODUCT_NAME', "Product Name");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PRODUCT_NAME', "Nombre del Producto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROHIBITED_CHARACTERS', "Nome inválido. Caracteres proibidos: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROHIBITED_CHARACTERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROHIBITED_CHARACTERS', "Nombre inválido. Caracteres prohibidos: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PROMOTIONAL_TENDERS', "Pagamentos Promocionais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PROMOTIONAL_TENDERS', "$PROMOTIONAL_TENDERS_PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PROMOTIONAL_TENDERS', "$PROMOTIONAL_TENDERS_PENDING_TRADUCTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'PUT_CODE_IN_SCANNER_BELOW', "Por favo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'PUT_CODE_IN_SCANNER_BELOW', "$PENDING_TRADUCTION_PUT_CODE_IN_SCANNER_BELOW");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'PUT_CODE_IN_SCANNER_BELOW', "Por favo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'QTY', "Qtd");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'QTY', "Qty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'QTY', "Cant");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'QTY_DELIVERY_REPORT', "Qtd");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'QTY_DELIVERY_REPORT', "Qty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'QTY_DELIVERY_REPORT', "Cant");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'QUANTITY', "Quantidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'QUANTITY', "Quantity");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'QUANTITY', "Cantidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'QUANTITY_OF_ITEMS_EXCEEDED', "Quantidade máxima de itens excedida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'QUANTITY_OF_ITEMS_EXCEEDED', "$PENDING_TRADUCTION_QUANTITY_OF_ITEMS_EXCEEDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'QUANTITY_OF_ITEMS_EXCEEDED', "Cantidad máxima de artículos excedida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'QUANTITY_SHORT', "Qty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'QUANTITY_SHORT', "PENDING_TRADUCTION_QUANTITY_SHORT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'QUANTITY_SHORT', "Cant.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REACHED', "Alcançado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REACHED', "Reached");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REACHED', "Alcanzado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'READER_FINGERPRINT_NOT_FOUND', "Leitor não encontrado!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'READER_FINGERPRINT_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'READER_FINGERPRINT_NOT_FOUND', "¡Lector no encontrado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'READING_MISTAKE', "Erro de leitur");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'READING_MISTAKE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'READING_MISTAKE', "Error de lectur");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'READ_QR_CODE', "Por favor leia o QR Code e aguarde o processamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'READ_QR_CODE', "Please read the QR Code and wait for the processing");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'READ_QR_CODE', "Por favor registre el código QR y espere el proceso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REAL', "Real");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REAL', "Real");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REAL', "Real");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RECALL_ORDER_TO_OLD', "Pedido muito antigo para ser recuperado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RECALL_ORDER_TO_OLD', "Order too old to be retrieved");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RECALL_ORDER_TO_OLD', "Pedido demasiado antiguo para ser recuperado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REFERENCE_POINT', "Referência");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REFERENCE_POINT', "Reference");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REFERENCE_POINT', "Referencia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REGISTERS_NOT_OPENED', "Dia não pode ser aberto. Motivo:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REGISTERS_NOT_OPENED', "The day could not be opened. The reason is:\\{0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REGISTERS_NOT_OPENED', "El día no se pudo abrir. El motivo es:\\{0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REGISTRATION_PERFORMED_SUCCESSFULLY', "Cadastro realizado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REGISTRATION_PERFORMED_SUCCESSFULLY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REGISTRATION_PERFORMED_SUCCESSFULLY', "Registro realizado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RELEASE', "Liberar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RELEASE', "Release");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RELEASE', "Liberar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMAINING', "Restante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMAINING', "Remaining");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMAINING', "Restante");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMAINING_AMOUNT', "Valor restante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMAINING_AMOUNT', "remaining amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMAINING_AMOUNT', "cantidad restante");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_ORDER_SEND_TO_PRODUCTION', "Pedido enviado para produção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_ORDER_SEND_TO_PRODUCTION', "Order sent to production");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_ORDER_SEND_TO_PRODUCTION', "Orden enviada para produción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_ORDER_STORE_STATUS_CHANGE', "A loja remota está {0}.\Deseja alterar o status?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_ORDER_STORE_STATUS_CHANGE', "The remote store is {0}.\Change state?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_ORDER_STORE_STATUS_CHANGE', "La tienda remota es {0}.\¿Desea cambiar el estado?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_ORDER_STORE_STATUS_CHANGED', "Status de loja remota alterado com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_ORDER_STORE_STATUS_CHANGED', "State changed!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_ORDER_STORE_STATUS_CHANGED', "¡Estado cambiado con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_ORDER_VOIDED_SUCCESS', "Pedido cancelado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_ORDER_VOIDED_SUCCESS', "Order canceled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_ORDER_VOIDED_SUCCESS', "Orden cancelada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_ORDER_WILL_BE_VOIDED', "Solicitação de cancelamento enviada para o parceiro, em breve seu pedido será cancelado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_ORDER_WILL_BE_VOIDED', "Cancellation request sent to the partner, soon your order will be cancelled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_ORDER_WILL_BE_VOIDED', "Solicitud de cancelación enviada al socio, pronto su pedido será cancelado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOTE_STATUS', "Status Remoto:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOTE_STATUS', "Remote Status:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOTE_STATUS', "Estado Remoto:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOVE', "Remover");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOVE', "Remove");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOVE', "Borrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOVE_CPF', "Remover CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOVE_CPF', "Remove CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOVE_CPF', "Borrar CPF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REMOVE_LOYALTY', "Remover Fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REMOVE_LOYALTY', "Remove Loyalty");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REMOVE_LOYALTY', "Borrar Fidelidad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REOPEN_TABLE', "Reabrir Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REOPEN_TABLE', "Reopen table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REOPEN_TABLE', "Reabrir Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REORGANIZE', "Reorganizar Assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REORGANIZE', "Reorganize Seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REORGANIZE', "Reorganizar asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPEAT_PASSWORD', "Repita a senha");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPEAT_PASSWORD', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPEAT_PASSWORD', "Repita la contraseña");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPORT_DELIVERY', "Relatório Vendas Integradas Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPORT_DELIVERY', "Integrated Delivery Sales");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPORT_DELIVERY', "Reporte de Ventas Delivery Integradas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPORT_ITEMS_CONFERENCE', "Conferência de Itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPORT_ITEMS_CONFERENCE', "PENDING_TRADUCTION_REPORT_ITEMS_CONFERENCE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPORT_ITEMS_CONFERENCE', "PENDING_TRADUCTION_REPORT_ITEMS_CONFERENCE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_FORBIDDEN', "Reimpresão não permitida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_FORBIDDEN', "Reprint is not Allowed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_FORBIDDEN', "Reimpresión no permitida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_LAST_EFT_RECEIPT', "Reimprimir última via TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_LAST_EFT_RECEIPT', "Reprint last via TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_LAST_EFT_RECEIPT', "Reimprimir última vía TEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_ORDERS', "Reimpressão de pedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_ORDERS', "Reprint orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_ORDERS', "Reimprimir Ordenes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_ORDER_SELECT', "Selecione um pedido para reimpressão:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_ORDER_SELECT', "Select order to reprint:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_ORDER_SELECT', "Seleccione orden a reimprimir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_RECEIPT', "Reimprimir comprovante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_RECEIPT', "Reprint receipt");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_RECEIPT', "Reimprimir recibo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_TEF_ERROR', "Erro ao reimprimir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_TEF_ERROR', "Error when reprinting");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_TEF_ERROR', "Error al reimprimir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REPRINT_TEXT_MESSAGE', "****REIMPRESSAO****");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REPRINT_TEXT_MESSAGE', "****REPRINT****");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REPRINT_TEXT_MESSAGE', "****REIMPRIMIR****");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'REQUEST_DOCUMENT_ON_PINPAD', "Solicitar ao cliente que digite o CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'REQUEST_DOCUMENT_ON_PINPAD', "$PENDING_TRADUCTION_REQUEST_DOCUMENT_ON_PINPAD");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'REQUEST_DOCUMENT_ON_PINPAD', "Solicitar al cliente que digite el documento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESIGN_XML', "Reassinar XML");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESIGN_XML', "Resign XML");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESIGN_XML', "Renunciar XML");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESTARTING_POS', "Reiniciando o PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESTARTING_POS', "Restarting POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESTARTING_POS', "Reiniciando el TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESTART_AFTER_UPDATE', "Deseja reiniciar todo o sistema após a a atualização para aplicá-la?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESTART_AFTER_UPDATE', "Restart after update?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESTART_AFTER_UPDATE', "¿Desea reiniciar después de la actualización?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESTAURANT_SALES_BUSINESS_PERIOD', "Relatório de Vendas\(Data de Negócio)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESTAURANT_SALES_BUSINESS_PERIOD', "Sales Report\(Business Date)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESTAURANT_SALES_BUSINESS_PERIOD', "Reporte de Ventas\(Fecha de Negocio)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESTAURANT_SALES_REAL_DATE', "Relatório de Vendas\(Data Fiscal)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESTAURANT_SALES_REAL_DATE', "Sales Report\(Fiscal Date)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESTAURANT_SALES_REAL_DATE', "Reporte de Ventas\(Fecha Fiscal)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RESTAURANT_SALES_XML', "Relatório de Vendas (XML)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RESTAURANT_SALES_XML', "Sales Report (XML)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RESTAURANT_SALES_XML', "Reporte de Ventas (XML)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RETRANSMIT_ORDERS', "Retransmitir Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RETRANSMIT_ORDERS', "Retransmit Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RETRANSMIT_ORDERS', "Retransmitir Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RETRANSMIT_ORDER_ERROR', "Erro ao tentar retransmitir pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RETRANSMIT_ORDER_ERROR', "Error when trying to send order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RETRANSMIT_ORDER_ERROR', "Error al intentar retransmitir el pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RETRANSMIT_ORDER_SUCCESS', "Pedido retransmitido com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RETRANSMIT_ORDER_SUCCESS', "Order retransmitted successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RETRANSMIT_ORDER_SUCCESS', "Pedido retransmitido exitosamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ROUND_PROJECT_TITLE', "Projeto Arredondar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ROUND_PROJECT_TITLE', "Round Project");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ROUND_PROJECT_TITLE', "Proyecto Redondo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RUPTURED_PRODUCT', "O produto {0} não está disponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RUPTURED_PRODUCT', "The product {0} is unavailable");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RUPTURED_PRODUCT', "El producto {0} no está disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RUPTURE_ERROR', "Erro realizar a ruptura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RUPTURE_ERROR', "Rupture error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RUPTURE_ERROR', "Error de Ruptura");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RUPTURE_RESET_UPDATE', "Todos os itens em ruptura ficarão disponíveis após a atualização.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RUPTURE_RESET_UPDATE', "All items in rupture will be available after the update.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RUPTURE_RESET_UPDATE', "PENDING_TRADUCTION_RUPTURE_RESET");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'RUPTURE_UPDATE_SUCCESS', "Produtos em Ruptura atualizados com sucesso.\\ Em alguns instantes o cardápio será atualizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'RUPTURE_UPDATE_SUCCESS', "$PENDING_TRADUCTION_RUPTURE_UPDATE_SUCCESS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'RUPTURE_UPDATE_SUCCESS', "Productos en Ruptura actualizados con éxito.\\ En un momento el menú será actualizado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALESBYBRAND', "Relatório de Vendas por Bandeiras");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALESBYBRAND', "Sales Report\Brands");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALESBYBRAND', "Reporte de Ventas por Marcas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALES_CHALLENGE', "Desafios de Vendas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALES_CHALLENGE', "PENDING_TRADUCTION_SALES_CHALLENGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALES_CHALLENGE', "Desafíos de Ventas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALES_SUMMARY', "Resumo de Vendas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALES_SUMMARY', "Sales Summary");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALES_SUMMARY', "Resumen de Ventas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_CREATION_SUCCESS', "Desafio de venda criado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_CREATION_SUCCESS', "PENDING_TRADUCTION_SALE_CHALLENGE_CREATION_SUCCESS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_CREATION_SUCCESS', "Desafio de venta creado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_EDITION_SUCCESS', "Desafio de venda editado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_EDITION_SUCCESS', "PENDING_TRADUCTION_SALE_CHALLENGE_EDITION_SUCCESS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_EDITION_SUCCESS', "Desafío de venta editado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_END_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "O desafio não pode ser criado pois a Data Fim é menor que a Data Atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_END_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "PENDING_TRADUCTION_SALE_CHALLENGE_END_DATE_CANNOT_BE_EARLIER_THAN_TODAY");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_END_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "El desafío no puede ser creado pues la Fecha Final es menor que la Fecha Actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_START_DATE_AFTER_THAN_END_DATE', "O desafio não pode ser criado pois a Data Início é maior que a Data Fim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_START_DATE_AFTER_THAN_END_DATE', "PENDING_TRADUCTION_SALE_CHALLENGE_START_DATE_AFTER_THAN_END_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_START_DATE_AFTER_THAN_END_DATE', "El desafío no puede ser creado pues la Fecha Inicial es mayor que la Fecha Final");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_START_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "O desafio não pode ser criado pois a Data Início é menor que a Data Atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_START_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "PENDING_TRADUCTION_SALE_CHALLENGE_START_DATE_CANNOT_BE_EARLIER_THAN_TODAY");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_START_DATE_CANNOT_BE_EARLIER_THAN_TODAY', "El desafío no puede ser creado pues la Fecha Inicial es anterior a la Fecha Actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_WITHOUT_END_DATE', "O desafio não pode ser criado pois não foi dado a Data Fim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_WITHOUT_END_DATE', "PENDING_TRADUCTION_SALE_CHALLENGE_WITHOUT_END_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_WITHOUT_END_DATE', "El desafío no puede ser creado pues no fue dado una Fecha Final");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_WITHOUT_MODIFICATIONS', "O desafio de venda não foi alterado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_WITHOUT_MODIFICATIONS', "PENDING_TRADUCTION_SALE_CHALLENGE_WITHOUT_MODIFICATIONS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_WITHOUT_MODIFICATIONS', "El desafío de venta no fue alterado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_WITHOUT_NAME', "O desafio não pode ser criado pois não foi dado um nome a ele");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_WITHOUT_NAME', "PENDING_TRADUCTION_SALE_CHALLENGE_WITHOUT_NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_WITHOUT_NAME', "El desafío no puede ser creado pues no se le ha asignado un nombre");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_WITHOUT_PRODUCTS', "O desafio não pode ser criado pois nenhum produto foi selecionado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_WITHOUT_PRODUCTS', "PENDING_TRADUCTION_SALE_CHALLENGE_WITHOUT_PRODUCTS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_WITHOUT_PRODUCTS', "El desafío no puede ser creado pues ningún producto fue seleccionado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CHALLENGE_WITHOUT_START_DATE', "O desafio não pode ser criado pois não foi dado a data de início");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CHALLENGE_WITHOUT_START_DATE', "PENDING_TRADUCTION_SALE_CHALLENGE_WITHOUT_START_DATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CHALLENGE_WITHOUT_START_DATE', "El desafío no puede ser creado pues no fue dado una fecha inicial");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CUSTOMER_CNPJ', "CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CUSTOMER_CNPJ', "CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CUSTOMER_CNPJ', "CNPJ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CUSTOMER_DOC', "CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CUSTOMER_DOC', "Document");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CUSTOMER_DOC', "Documento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SALE_CUSTOMER_NAME', "Cliente:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SALE_CUSTOMER_NAME', "Customer:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SALE_CUSTOMER_NAME', "Cliente:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SANDWICHES', "sanduíches");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SANDWICHES', "sandwiches");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SANDWICHES', "sandwiches");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAT_INFO', "Informações SAT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAT_INFO', "SAT Information");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAT_INFO', "Información SAT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAT_STATUS', "SAT Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAT_STATUS', "SAT Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAT_STATUS', "Estado del SAT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVE', "Salvar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVE', "Save");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVE', "Guardar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVED_ORDER_CANCELED', "Pedido Salvo Cancelado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVED_ORDER_CANCELED', "PENDING_TRADUCTION_SAVED_ORDER_CANCELED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVED_ORDER_CANCELED', "Perdido Guardado Cancelado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVE_AND_QUIT', "Salvar e Fechar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVE_AND_QUIT', "Save and Close");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVE_AND_QUIT', "Guardar y Cerrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVE_CUSTOMER', "Salvar Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVE_CUSTOMER', "Save Client");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVE_CUSTOMER', "Guardar cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVE_CUSTOMER_INSERT_ON_SALE', "Salvar Cliente e Incluir Na Venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVE_CUSTOMER_INSERT_ON_SALE', "Save Customer and Include In Sale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVE_CUSTOMER_INSERT_ON_SALE', "Guardar Cliente e Incluir en la Venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SAVE_ORDER', "Salvar Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SAVE_ORDER', "Save Order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SAVE_ORDER', "Guardar Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SCHEDULED', "Agendado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SCHEDULED', "PENDING_TRADUCTION_SCHEDULED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SCHEDULED', "Agendado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SCHEDULED_ORDER', "AGENDADO:\\");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SCHEDULED_ORDER', "SCHEDULED:\\");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SCHEDULED_ORDER', "AGENDADO:\\");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCH', "Acesso Rápido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCH', "Quick Access");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCH', "Acceso Rápido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCHING_CUSTOMER', "Buscando informações do cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCHING_CUSTOMER', "Searching for customer info");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCHING_CUSTOMER', "Buscando información del Cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCHING_LOGISTIC_PARTNER', "Confirmando entregador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCHING_LOGISTIC_PARTNER', "Confirming delivery person");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCHING_LOGISTIC_PARTNER', "Confirmando Repartidor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCHING_ZIP_CODE', "Buscando informações do CEP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCHING_ZIP_CODE', "Searching for zipcode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCHING_ZIP_CODE', "Bucando Código Postal");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCH_ITEMS', "Buscar itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCH_ITEMS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCH_ITEMS', "Buscar artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEARCH_OPENED_TABS', "Consultar comandas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEARCH_OPENED_TABS', "$PENDING_TRADUCTION_SEARCH_OPENED_TABS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEARCH_OPENED_TABS', "Consultar comandas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEAT', "Assento {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEAT', "Seat {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEAT', "Asiento {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEATED', "Sentado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEATED', "Seated");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEATED', "Sentado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEATS', "Assentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEATS', "Seats");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEATS', "Asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEATS_NUMBER', "Nº de assentos:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEATS_NUMBER', "Number of seats:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEATS_NUMBER', "Número de asientos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEAT_NUMBER', "Assento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEAT_NUMBER', "Seat");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEAT_NUMBER', "Asiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEAT_SELECTION_TITLE', "Selecione o assento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEAT_SELECTION_TITLE', "Select seat");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEAT_SELECTION_TITLE', "Selecciones asiento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEAT_SPLIT_CONFIRMATION', "Irá totalizar a mesa por assento?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEAT_SPLIT_CONFIRMATION', "Will you total the table by seat?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEAT_SPLIT_CONFIRMATION', "¿Totalizará la mesa por asiento?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SECOND', "Segundo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SECOND', "Second");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SECOND', "Segundo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SECOND_READING', "Segunda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SECOND_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SECOND_READING', "Segunda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SECTOR', "Setor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SECTOR', "Sector");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SECTOR', "Sector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEFAZ_CONNECTION', "Conexão SEFAZ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEFAZ_CONNECTION', "SEFAZ Connection");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEFAZ_CONNECTION', "Conexión SEFAZ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_ALL_PARTS', "Necessário selecionar todos as partes do item dividido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_ALL_PARTS', "Please select all item parts");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_ALL_PARTS', "Por favor seleccionar todas las partes del artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_AN_ORDER_TO_CANCEL', "Selecione um pedido para cancelar:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_AN_ORDER_TO_CANCEL', "Select a order to cancel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_AN_ORDER_TO_CANCEL', "Seleccione orden para cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_A_COURSE', "Selecione um curso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_A_COURSE', "Select a course");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_A_COURSE', "Selecione un curso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_A_DATE', "Selecione uma data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_A_DATE', "Select a date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_A_DATE', "Seleccione una fecha");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_A_POS_FUNCTION', "Selecione a função do PDV:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_A_POS_FUNCTION', "Select POS function:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_A_POS_FUNCTION', "Seleccionar funciones del TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_BLEED', "Selecione uma Sangria :");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_BLEED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_BLEED', "$PENDING_TRADUCTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_BRAND', "Selecione a marca:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_BRAND', "Select brand:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_BRAND', "PENDING_TRADUCTION_SELECT_BRAND");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_ITEM', "Selecione um item:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_ITEM', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_ITEM', "Seleccione un artículo:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_LINE_FIRST', "Selecione uma linha primeiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_LINE_FIRST', "Select a line first");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_LINE_FIRST', "Seleccione una línea primero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_LINE_TO_CLEAR', "Selecione a linha para apagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_LINE_TO_CLEAR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_LINE_TO_CLEAR', "Seleccione la línea a eliminar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_NEW_QUANTITY_OF_SEATS', "Quantos assentos?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_NEW_QUANTITY_OF_SEATS', "How many seats?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_NEW_QUANTITY_OF_SEATS', "¿Cuántos asientos?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_ONE_COUPON', "Selecione o Cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_ONE_COUPON', "Select coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_ONE_COUPON', "Seleccione cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_ONE_SECTOR', "Selecione um setor:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_ONE_SECTOR', "Select a sector:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_ONE_SECTOR', "Seleccione un sector:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_OPERATOR_TO_ACTIVATE', "Selecione um operador para ativar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_OPERATOR_TO_ACTIVATE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_OPERATOR_TO_ACTIVATE', "Selecccione un operador para activar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_OPTION', "Selecione uma opção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_OPTION', "Select an option");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_OPTION', "Seleccione una opción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_OPTION_TO_CLEAN', "Selecione uma opção para limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_OPTION_TO_CLEAN', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_OPTION_TO_CLEAN', "Seleccione una opción para limpiar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_PAYMENT_METHOD', "Selecione a forma de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_PAYMENT_METHOD', "Select payment method");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_PAYMENT_METHOD', "Seleccione forma de pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_PAYMENT_TYPE', "Selecione uma forma de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_PAYMENT_TYPE', "Select a payment method");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_PAYMENT_TYPE', "Seleccione método de pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_QUANTITY_OF_SEATS', "Quantas pessoas?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_QUANTITY_OF_SEATS', "How many people?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_QUANTITY_OF_SEATS', "¿Cuántas personas?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_REASON', "Selecione o motivo:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_REASON', "Select the reason:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_REASON', "Seleccione la razón:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_REPRINT_ORDER', "Selecione um pedido para reimpressão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_REPRINT_ORDER', "Select a reprint order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_REPRINT_ORDER', "Seleccione un pedido de reimpresión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_SEAT_FOR_LINE', "Selecione o assento para o item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_SEAT_FOR_LINE', "Select seat for item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_SEAT_FOR_LINE', "Seleccione asiento para el artículo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_SECTORS_SERVICE_PERIOD', "Selecione o setor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_SECTORS_SERVICE_PERIOD', "Select sector");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_SECTORS_SERVICE_PERIOD', "Seleccione Sector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_SERVICE_PERIOD', "Selecione o período de serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_SERVICE_PERIOD', "Select period of service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_SERVICE_PERIOD', "Seleccionar período de servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_SUPPLY', "Selecione um Suprimento :");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_SUPPLY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_SUPPLY', "$PENDING_TRADUCTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TAB', "Selecione a Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TAB', "Select tab");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TAB', "Seleccione comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TABLE', "Selecione a Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TABLE', "Select table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TABLE', "Seleccione mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TABLE_TO_JOIN', "Juntar qual mesa?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TABLE_TO_JOIN', "Join which table?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TABLE_TO_JOIN', "¿Juntar cuál mesa?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TABLE_TO_TRANSFER', "Para qual mesa?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TABLE_TO_TRANSFER', "To which table?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TABLE_TO_TRANSFER', "¿Para qué mesa?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TABLE_TO_UNLINK', "Qual mesa será separada?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TABLE_TO_UNLINK', "Which table will be splitted?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TABLE_TO_UNLINK', "¿Qué mesa será dividida?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TENDER', "Selecione um pagamento:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TENDER', "$PENDING_TRADUCTION_SELECT_TENDER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TENDER', "Seleccione un pago:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_THE_ACTION', "Selecione a ação desejada:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_THE_ACTION', "Select the action:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_THE_ACTION', "Seleccione la acción:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TIP_PERCENTAGE', "Selecione o valor da porcentagem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TIP_PERCENTAGE', "Select the tip percentage");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TIP_PERCENTAGE', "Seleccione el porcentaje de propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TIP_VALUE', "Selecione o valor da gorjeta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TIP_VALUE', "Select the tip value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TIP_VALUE', "Seleccione el valor de propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TRANSACTION', "Selecione a Transação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TRANSACTION', "Select Transaction");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TRANSACTION', "Seleccionar transacción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SELECT_TYPE', "Selecione o tipo:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SELECT_TYPE', "Select the type:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SELECT_TYPE', "Seleccione tipo:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEND', "Enviar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEND', "Send");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEND', "Enviar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEND_AND_QUIT', "Enviar e Fechar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEND_AND_QUIT', "Send and Close");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEND_AND_QUIT', "Enviar y Cerrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEND_DELIVERYMAN', "Sair para entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEND_DELIVERYMAN', "Out for delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEND_DELIVERYMAN', "Fuera para entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEND_MANUAL', "Lançar manual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEND_MANUAL', "Manual entry");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEND_MANUAL', "Ingreso Manual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICES_PERIOD_NOT_FOUND', "Não foram encontrados períodos de serviços cadastrados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICES_PERIOD_NOT_FOUND', "PENDING_TRADUCTION_SERVICES_PERIOD_NOT_FOUND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICES_PERIOD_NOT_FOUND', "No se encontraron periodos de servicio registrados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_ID', "Id do serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_ID', "Service Id");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_ID', "ID de Servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_PERIOD_OUT_OF_SCHEDULE_RECORDED', "O período de serviço {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_PERIOD_OUT_OF_SCHEDULE_RECORDED', "$PENDING_TRADUCTION_SERVICE_PERIOD_OUT_OF_SCHEDULE_RECORDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_PERIOD_OUT_OF_SCHEDULE_RECORDED', "El periodo de servicio {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_PERIOD_TABLE', "Período de Serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_PERIOD_TABLE', "Period of Service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_PERIOD_TABLE', "Período de servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ALL_SECTORS', "O período de serviço {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ALL_SECTORS', "$PENDING_TRADUCTION_SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ALL_SECTORS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ALL_SECTORS', "El periodo de servicio {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ENTIRE_STORE', "O período de serviço {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ENTIRE_STORE', "$PENDING_TRADUCTION_SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ENTIRE_STORE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_ENTIRE_STORE', "El periodo de servicio {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_SECTORS', "O período de serviço {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_SECTORS', "$PENDING_TRADUCTION_SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_SECTORS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_PERIOD_WILL_BE_ACTIVATE_FOR_SECTORS', "El periodo de servicio {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SERVICE_TAX_IS_NOT_MANDATORY', "Taxa de serviço não obrigatória");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SERVICE_TAX_IS_NOT_MANDATORY', "Service tax is not mandatory");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SERVICE_TAX_IS_NOT_MANDATORY', "El impuesto por servicio no es obligatorio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SESSION_ID_REPORT', "Relatório Surpresa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SESSION_ID_REPORT', "Surprise Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SESSION_ID_REPORT', "Reporte Sorpresa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SEVENTH_READING', "Sétima");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SEVENTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SEVENTH_READING', "Séptima");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_DAYNOTCLOSED', "O dia não está fechado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_DAYNOTCLOSED', "Day not closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_DAYNOTCLOSED', "Día no cerrado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_DAYNOTOPENED', "O dia não está aberto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_DAYNOTOPENED', "Day not opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_DAYNOTOPENED', "Día no abierto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_INVALIDPASSWD', "Senha inválida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_INVALIDPASSWD', "Invalid password");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_INVALIDPASSWD', "Contraseña inválida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_OPERLOGGEDIN', "Operador está logado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_OPERLOGGEDIN', "Operator logged in");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_OPERLOGGEDIN', "El Usuario inició sesión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_OPERNOTLOGGEDIN', "Operador não está logado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_OPERNOTLOGGEDIN', "Operator not logged in");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_OPERNOTLOGGEDIN', "El usuario no ha iniciado sesión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_OPEROPENED', "Existem Operadores abertos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_OPEROPENED', "There are logged in operators");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_OPEROPENED', "Existen Meseros abiertos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_ORDEROPENED', "Existem pedidos abertos. Tente novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_ORDEROPENED', "There are opened orders. Try Again");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_ORDEROPENED', "Existen cuentas abiertas. Inténtalo de nuevo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SE_UNKOWNUSR', "Usuário desconhecido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SE_UNKOWNUSR', "Unkown operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SE_UNKOWNUSR', "Usuario desconocido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SHORT_REFERENCE', "Pedido:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SHORT_REFERENCE', "Order:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SHORT_REFERENCE', "Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SHOW_SEPARATE_SALES', "Ver Vendas Separadas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SHOW_SEPARATE_SALES', "Show Separate Sales");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SHOW_SEPARATE_SALES', "Ver Ventas Separadas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SIGNATURE', "Assinatura");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SIGNATURE', "Signature");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SIGNATURE', "Firma");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SINCE', "(DESDE {0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SINCE', "(SINCE {0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SINCE', "(DESDE {0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SITE', "https://www.e-deploy.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SITE', "https://www.e-deploy.com.br");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SITE', "https://www.e-deploy.com.br");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SITEF_CONNECTION', "Conexão SITEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SITEF_CONNECTION', "SITEF Connection");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SITEF_CONNECTION', "Conexión SITEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SITEF_OK', "SITEF OK");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SITEF_OK', "SITEF OK");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SITEF_OK', "SITEF OK");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SITEF_SERVER_ERROR', "Falha ao encontrar servidor SITEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SITEF_SERVER_ERROR', "SITEF connection failed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SITEF_SERVER_ERROR', "Falla al encontrar servidor SITEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SITEF_UNAVAILABLE', "Servidor SITEF Inoperante");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SITEF_UNAVAILABLE', "$PENDING_TRADUCTION_SITEF_UNAVAILABLE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SITEF_UNAVAILABLE', "Servidor SITEF inoperativo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SIXTH_READING', "Sexta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SIXTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SIXTH_READING', "Sexta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SKIM', "Sangria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SKIM', "Skim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SKIM', "Efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SKIM_ALERT', "Alerta de Sangria");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SKIM_ALERT', "Skim Alert");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SKIM_ALERT', "Alerta de Efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SKIM_VALUE_MUCH_BIGGER', "Valor da sangria não pode ser maior que o valor presente na gaveta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SKIM_VALUE_MUCH_BIGGER', "Skim can not be bigger than drawer values");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SKIM_VALUE_MUCH_BIGGER', "El efectivo no puede ser mayor al valor en la gaveta de dinero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SLICE_SERVICE', "Separar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SLICE_SERVICE', "Slice service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SLICE_SERVICE', "Separar Servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SOFTWARE_VERSION', "Versão do Software");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SOFTWARE_VERSION', "Software Version");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SOFTWARE_VERSION', "Versión del Software");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SOME_PAYMENTS_WERE_NOT_REFUNDED', "Erro revertendo alguns pagamentos. Clique OK para tentar novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SOME_PAYMENTS_WERE_NOT_REFUNDED', "PENDING_TRADUCTION_SOME_PAYMENTS_WERE_NOT_REFUNDED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SOME_PAYMENTS_WERE_NOT_REFUNDED', "Error revirtiendo algunos pagos. Presione OK para intentar nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SOME_USER_LOGGED', "Existe usuário logado no PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SOME_USER_LOGGED', "There is a user logged into the POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SOME_USER_LOGGED', "Existe usuario conectado al TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SPECIAL_INSTRUCTION', "Comentário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SPECIAL_INSTRUCTION', "Comment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SPECIAL_INSTRUCTION', "Comentario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SPEEDOFSERVICE', "Tempo de serviço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SPEEDOFSERVICE', "Speed of service");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SPEEDOFSERVICE', "Velocidad de Servicio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SPLIT_ITEM', "Dividir Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SPLIT_ITEM', "Split Item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SPLIT_ITEM', "Dividir artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'START_DELIVERY', "Abrir o Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'START_DELIVERY', "Open Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'START_DELIVERY', "Abrir Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'START_HOUR', "Início:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'START_HOUR', "Start:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'START_HOUR', "Inicio:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'START_ORDER', "Iniciar Venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'START_ORDER', "Start order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'START_ORDER', "Comenzar Order");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STATE', "Estado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STATE', "State");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STATE', "Estado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STATUS', "Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STATUS', "Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STATUS', "Estado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STATUS_ABANDONED', "Abandonado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STATUS_ABANDONED', "Abandoned");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STATUS_ABANDONED', "Abandonado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STATUS_LABEL', "Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STATUS_LABEL', "Status");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STATUS_LABEL', "Estado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STOCKOUT', "Esgotado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STOCKOUT', "Stockout");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STOCKOUT', "Agotado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STOCKOUT_CLEANUP', "Limpeza de Esgotados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STOCKOUT_CLEANUP', "Stockout Cleanup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STOCKOUT_CLEANUP', "Limpieza de Agotados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STOP_DELIVERY', "Fechar o Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STOP_DELIVERY', "Close Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STOP_DELIVERY', "Cerrar Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORE', "Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORE', "Store");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORE', "Tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORED_ORDERS_CLEANUP', "Limpeza de Pedidos Salvos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORED_ORDERS_CLEANUP', "Stored Orders Cleanup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORED_ORDERS_CLEANUP', "Limpieza de Pedidos Almacenados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORED_ORDERS_CLEANUP_MESSAGE', "Os pedidos salvos serão apagados de todos os PDVs e não será possível recuperá-los.\\Deseja Continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORED_ORDERS_CLEANUP_MESSAGE', "Saved orders will be deleted from all POS and it will not be possible to retrieve them.\\Do you want to continue?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORED_ORDERS_CLEANUP_MESSAGE', "Los pedidos guardados se eliminarán de todos los TPVs y no será posible recuperarlos.\\¿Quieres continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORED_ORDER_IS_VOIDED', "Pedido salvo apagado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORED_ORDER_IS_VOIDED', "Order deleted");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORED_ORDER_IS_VOIDED', "Orden borrada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORE_CLOSED', "FECHADA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORE_CLOSED', "CLOSED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORE_CLOSED', "CERRADA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORE_IS_CLOSED', "Loja está fechada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORE_IS_CLOSED', "Store is closed");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORE_IS_CLOSED', "La Tienda está Cerrada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORE_NOT_FOUND', "Não existe uma Loja configurada no BackOffice");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORE_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORE_NOT_FOUND', "No existe una Tienda configurada en el BackOffice");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'STORE_OPENED', "ABERTA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'STORE_OPENED', "OPENED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'STORE_OPENED', "ABIERTA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUBMIT_YOUR_ORDER', "Envie seu pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUBMIT_YOUR_ORDER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUBMIT_YOUR_ORDER', "Envíe su pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUBTOTAL', "Subtotal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUBTOTAL', "Subtotal");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUBTOTAL', "Total Parcial");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESSFULLY_CLEANED_KDS', "KDSs limpos com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESSFULLY_CLEANED_KDS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESSFULLY_CLEANED_KDS', "KDSs limpios con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_CHANGE_PROFILE', "Perfil do usuário alterado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_CHANGE_PROFILE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_CHANGE_PROFILE', "Perfil de usuario alterado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_CONFIRM_REMOTE_ORDER_DELIVERY', "Entrega confirmada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_CONFIRM_REMOTE_ORDER_DELIVERY', "Confirmed delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_CONFIRM_REMOTE_ORDER_DELIVERY', "Pedido confirmado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_COUPON_APPLIED', "Cupom aplicado com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_COUPON_APPLIED', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_COUPON_APPLIED', "¡Cupón aplicado con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_DELETE_USER', "Usuário excluído com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_DELETE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_DELETE_USER', "¡Usuario eliminado con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_DISABLE_USER', "Usuário desativado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_DISABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_DISABLE_USER', "Usuario desactivado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_ENABLE_USER', "Usuário reativado com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_ENABLE_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_ENABLE_USER', "Usuario activado con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_INSERT_NEW_USER', "Usuário cadastrado com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_INSERT_NEW_USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_INSERT_NEW_USER', "¡Usuario registrado con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_NOT_TOTALED_COUPON_APPLIED', "Cupom aplicado com sucesso!\\Os descontos do cupom serão visualizados na tela de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_NOT_TOTALED_COUPON_APPLIED', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_NOT_TOTALED_COUPON_APPLIED', "¡Cupón aplicado con éxito!\\Los descuentos del cupón serán visulizados en la ventana de pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_PLACE_YOUR_FINGER_AGAIN', "Leitura efetuada com sucesso. Posicione o dedo novamente no leitor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_PLACE_YOUR_FINGER_AGAIN', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_PLACE_YOUR_FINGER_AGAIN', "Lectura realizada con éxito. Posicione el dedo nuevamente en el lector");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUCCESS_UPDATE', "Concluído");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUCCESS_UPDATE', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUCCESS_UPDATE', "Concluído");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUGGESTED_TIP', "Gorjeta Sugerida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUGGESTED_TIP', "Suggested Tip");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUGGESTED_TIP', "Propina Sugerida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUPERVISOR', "Supervisor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUPERVISOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUPERVISOR', "Supervisor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SUPPORT_MENU', "Menu Suporte");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SUPPORT_MENU', "Support Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SUPPORT_MENU', "Menú de Soporte");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SYNC_DELIVERY_MENU', "Sincronizar Menu Delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SYNC_DELIVERY_MENU', "Sync Delivery Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SYNC_DELIVERY_MENU', "Sincronizar Menú del Delivery");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SYNC_ERROR', "Erro de sincronização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SYNC_ERROR', "PENDING_TRADUCTION_SYNC_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SYNC_ERROR', "Error de sincronización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'SYSTEM', "Sistema");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'SYSTEM', "System");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'SYSTEM', "Sistema");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB', "Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB', "Pager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB', "Comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE', "Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE', "Table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE', "Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLES', "Mesas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLES', "Tables");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLES', "Mesas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_HAS_TENDERS', "Mesa possui pagamentos abertos. Apague os pagamentos e tente novamente.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_HAS_TENDERS', "Table has open Tenders. Remove the tenders and try again.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_HAS_TENDERS', "La mesa tiene cuentas abiertas. Borre los pagos e intente nuevamente.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_INFO', "Informação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_INFO', "Information");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_INFO', "Información");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_INFORMATION', "Mesa: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_INFORMATION', "Table: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_INFORMATION', "Mesa: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_INFORMATION2', "Mesa: {0} - {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_INFORMATION2', "Table: {0} - {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_INFORMATION2', "Mesa: {0} - {1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_INITIAL_LETTER', "M");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_INITIAL_LETTER', "T");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_INITIAL_LETTER', "M");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_LIST', "Layout (Mesa)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_LIST', "Layout (Table)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_LIST', "Plano (Mesa)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_NOT_AVAILABLE', "Mesa/Comanda não disponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_NOT_AVAILABLE', "Table/Pager not available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_NOT_AVAILABLE', "Mesa/Comanda no disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_NUMBER', "Número da mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_NUMBER', "Table Number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_NUMBER', "Número de Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_OPENED_BY_ANOTHER_OPERATOR', "Mesa já aberta pelo usuário: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_OPENED_BY_ANOTHER_OPERATOR', "Table already opened by operator: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_OPENED_BY_ANOTHER_OPERATOR', "PENDING_TRADUCTION");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_OPENED_IN_ANOTHER_POS', "Mesa/Comanda já aberta no PDV {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_OPENED_IN_ANOTHER_POS', "Table/Pager already opened on POS {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_OPENED_IN_ANOTHER_POS', "Mesa/Comanda ya abierta en el TPV {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_SECTOR_HAS_NO_REGISTERED_OPERATORS', " O setor da mesa não tem operadores disponíveis cadastrados.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_SECTOR_HAS_NO_REGISTERED_OPERATORS', "Table sector has no registered operators.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_SECTOR_HAS_NO_REGISTERED_OPERATORS', "El sector de la mesa no tiene operadores disponibles registrados.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_SERVICE_MODE', "Escolha o modo de serviço:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_SERVICE_MODE', "Choose service mode:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_SERVICE_MODE', "Elija el modo de servicio:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABLE_SYMBOL', "M-");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABLE_SYMBOL', "T-");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABLE_SYMBOL', "M-");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TABS', "Comandas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TABS', "Pagers");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TABS', "Comandas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_ADDITIONAL_INFO', "Informação adicional para entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_ADDITIONAL_INFO', "Additional info for delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_ADDITIONAL_INFO', "Información adicional para entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_ADDITIONAL_INFO_NONE', "Confirma envio sem informação adicional?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_ADDITIONAL_INFO_NONE', "Confirm delivery without additional info?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_ADDITIONAL_INFO_NONE', "¿Confirmar envío sin información adicional?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_ALREADY_EXISTS', "Comanda já utilizada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_ALREADY_EXISTS', "Pager already exists");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_ALREADY_EXISTS', "Comanda ya utilizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_IDENTIFIER', "Comanda {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_IDENTIFIER', "Pager {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_IDENTIFIER', "Comanda {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_INFORMATION', "Comanda: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_INFORMATION', "Pager: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_INFORMATION', "Comanda: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_INFORMATION2', "Comanda: {0} - {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_INFORMATION2', "Pager: {0} - {1}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_INFORMATION2', "Comanda {0} -{1}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_INITIAL_LETTER', "C");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_INITIAL_LETTER', "P");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_INITIAL_LETTER', "C");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_JOIN_NOT_STRAIGHT_ALLOWED', "Para unir uma comanda à uma mes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_JOIN_NOT_STRAIGHT_ALLOWED', "To join a tab to a tabl");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_JOIN_NOT_STRAIGHT_ALLOWED', "Para unir una comanda a una mes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAB_NUMBER', "Comanda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAB_NUMBER', "Pager");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAB_NUMBER', "Comanda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAKE_CONSUMPTION_SHEET', "Por favo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAKE_CONSUMPTION_SHEET', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAKE_CONSUMPTION_SHEET', "Por favo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TAKE_OUT', "Viagem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TAKE_OUT', "Take Out");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TAKE_OUT', "Para llevar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TEF_CANCELLATION_REPORT', "Relatório de erros para cancelamentos TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TEF_CANCELLATION_REPORT', "TEF cancellation error report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TEF_CANCELLATION_REPORT', "Reporte de errores para cancelamientos TEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TEF_CANCELLATION_REPORT_38_CHARS', "RELATORIO DE ERROS P CANCELAMENTOS TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TEF_CANCELLATION_REPORT_38_CHARS', "TEF CANCELLATION ERROR REPORT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TEF_CANCELLATION_REPORT_38_CHARS', "REPORTE D ERRORES P CANCELAMIENTOS TEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TEF_CANCELLATION_REPORT_DATE', "Data:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TEF_CANCELLATION_REPORT_DATE', "Date:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TEF_CANCELLATION_REPORT_DATE', "Fecha:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TEF_CANCELLATION_REPORT_TOTAL', "Total:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TEF_CANCELLATION_REPORT_TOTAL', "Total:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TEF_CANCELLATION_REPORT_TOTAL', "Total:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TEF_REPRINT', "Reimprimir TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TEF_REPRINT', "reprint TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TEF_REPRINT', "reimprimir TEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TENDERS_ARE_BIGGER_THEN_DUE_AMOUNT', "Pagamentos maiores que o valor do pedido. Cancele ou adicione mais itens.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TENDERS_ARE_BIGGER_THEN_DUE_AMOUNT', "Tenders are bigger then due amount. Cancel or add more items.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TENDERS_ARE_BIGGER_THEN_DUE_AMOUNT', "El pago es mayor que el monto a deber. Cancelar o añadir más artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TENDER_DIVISION', "Divisão de Pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TENDER_DIVISION', "Tender split");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TENDER_DIVISION', "Dividir pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TENDER_ID_NOT_FOUND', "TenderId não encontrado: Tipo de Pagamento: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TENDER_ID_NOT_FOUND', "TenderId {0} not found: Payment method: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TENDER_ID_NOT_FOUND', "TenderId {0} no encontrado: Tipos de Pago {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TENTH_READING', "Décima");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TENTH_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TENTH_READING', "Décima");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THANK_TEXT', "Obrigado tenha um bom apetite!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THANK_TEXT', "Thank you have a nice meal!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THANK_TEXT', "¡Gracia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THANK_YOU_FOR_YOUR_PREFERENCE', "Obrigado pela preferência");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THANK_YOU_FOR_YOUR_PREFERENCE', "Thank you for your preference");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THANK_YOU_FOR_YOUR_PREFERENCE', "Gracias por su preferencia");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_ARE_NO_CLOSED_SESSIONS', "Não há sessões encerradas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_ARE_NO_CLOSED_SESSIONS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_ARE_NO_CLOSED_SESSIONS', "No hay sesiones cerradas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_ARE_NO_DATA_TO_SHOW', "Não existem valores a serem exibidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_ARE_NO_DATA_TO_SHOW', "No data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_ARE_NO_DATA_TO_SHOW', "No existen datos para mostrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_ARE_NO_DRIVE_ORDERS', "Não há pedidos salvos no drive");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_ARE_NO_DRIVE_ORDERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_ARE_NO_DRIVE_ORDERS', "No hay pedidos guardados en el drive");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_ARE_NO_STORE_ORDERS', "Não há pedidos salvos na loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_ARE_NO_STORE_ORDERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_ARE_NO_STORE_ORDERS', "No hay pedidos guardados en la tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_ARE_OPEN_OPERATORS', "Existem operadores logados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_ARE_OPEN_OPERATORS', "There is operators logged in");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_ARE_OPEN_OPERATORS', "No hay Meseros registrados actualmente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_IS_AN_OPENED_ORDER_ON_POS', "Existe uma venda aberta no PDV {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_IS_AN_OPENED_ORDER_ON_POS', "There is a opened order on POS {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_IS_AN_OPENED_ORDER_ON_POS', "Existe una cuenta abierta en el TPV {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_IS_NO_OPENED_USER', "Não há nenhum usuário aberto.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_IS_NO_OPENED_USER', "There is no opened user.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_IS_NO_OPENED_USER', "No hay usuario abierto.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_IS_NO_PROOF_TO_BE_PRINTED', "Não há comprovante para ser impresso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_IS_NO_PROOF_TO_BE_PRINTED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_IS_NO_PROOF_TO_BE_PRINTED', "No hay comprobante a imprimir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_NO_ITEMS_IN_ORDER', "Pedido não possui itens!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_NO_ITEMS_IN_ORDER', "Order has no items!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_NO_ITEMS_IN_ORDER', "¡La orden no tiene artículos!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THERE_NO_SHIFT_OPERATOR', "Não existe nenhum turno para o operador selecionado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THERE_NO_SHIFT_OPERATOR', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THERE_NO_SHIFT_OPERATOR', "No existe ningún turno para el operador seleccionado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THIRD', "Terceiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THIRD', "Third");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THIRD', "Tercero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THIRD_READING', "Terceira");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THIRD_READING', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THIRD_READING', "Tercera");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'THIS_POS', "Neste Ponto de Venda");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'THIS_POS', "This POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'THIS_POS', "En esta Terminal de Venta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIMEOUT', "Detecção de Inatividade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIMEOUT', "Detection of Inactivity");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIMEOUT', "Detección de Inactividad");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIMEOUT_CANCELLATION', "Cancelamento de tempo limite");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIMEOUT_CANCELLATION', "Timeout cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIMEOUT_CANCELLATION', "Cancelación de tiempo de espera");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIME_OPENED', "Tempo Mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIME_OPENED', "Table Time");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIME_OPENED', "Tiempo de Mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIME_TO_HOLD_QUESTION', "Segurar por quantos minutos?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIME_TO_HOLD_QUESTION', "Hold for how many minutos?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIME_TO_HOLD_QUESTION', "Aguanta cuantos minutos?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP', "Gorjeta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP', "Tip");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP', "Propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_AMOUNT_MUST_BE_VALID', "A porcentagem ou valor deve ser um valor válido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_AMOUNT_MUST_BE_VALID', "Percentage or amount must be a valid value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_AMOUNT_MUST_BE_VALID', "El porcentaje o la cantidad debe ser un valor válido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_AMOUNT_NEED_TO_BE_LESS_THAN_DUE_AMOUNT', "Não é possível zerar a gorjeta em um pedido sem valor a pagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_AMOUNT_NEED_TO_BE_LESS_THAN_DUE_AMOUNT', "It is not possible to clear the tip on an order with no due amount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_AMOUNT_NEED_TO_BE_LESS_THAN_DUE_AMOUNT', "No es posible restablecer la propina en un pedido sin monto a pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_GIVEN', "Gorjeta Concedida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_GIVEN', "Granted Tip");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_GIVEN', "Propina Concedida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_PERCENTAGE_NOT_ALLOWED', "Percentual de gorjeta não permitido para pagamento em cartão de crédito ou débito");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_PERCENTAGE_NOT_ALLOWED', "Tip percentage not allowed to credit or debit.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_PERCENTAGE_NOT_ALLOWED', "Porcentaje de propina no permitido para crédito o débito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT', "Relatório de Gorjetas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT', "Tip Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT', "Reporte de Propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_CURRENT_DATE', "Data Atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_CURRENT_DATE', "Current Date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_CURRENT_DATE', "Fecha Actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_DATE', "Data");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_DATE', "Date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_DATE', "Fecha");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_OPERATOR', "Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_OPERATOR', "Operator");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_OPERATOR', "Mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_ORDERS', "Pedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_ORDERS', "Orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_ORDERS', "Órdenes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_TABLES', "Mesas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_TABLES', "Tables");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_TABLES', "Mesas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_TIP', "Gorjeta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_TIP', "Tip");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_TIP', "Propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_REPORT_TOTAL', "Total");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_REPORT_TOTAL', "Total");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_REPORT_TOTAL', "Total");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TIP_VALUE_CANNOT_BE_CHANGED_TENDER_AMOUNT', "Não é possível alterar a gorjeta.\\O valor dos pagamentos é maior do que o valor total após aplicar a gorjeta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TIP_VALUE_CANNOT_BE_CHANGED_TENDER_AMOUNT', "PENDING_TRADUCTION_TIP_VALUE_CANNOT_BE_CHANGED_TENDER_AMOUNT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TIP_VALUE_CANNOT_BE_CHANGED_TENDER_AMOUNT', "No es posible cambiar la propina. \\El valor total pagado es mayor que el valor total después de cambiar la propina");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TITLE_PLACE_FINGER_AUTHORIZATION', "Autorizacao requerida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TITLE_PLACE_FINGER_AUTHORIZATION', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TITLE_PLACE_FINGER_AUTHORIZATION', "Autorización requerida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TITLE_REPORT_DISCOUNT_COD', "Cod. Desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TITLE_REPORT_DISCOUNT_COD', "code Discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TITLE_REPORT_DISCOUNT_COD', "código Descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TMA', "TMA");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TMA', "AOT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TMA', "TMA");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TMA_ORDERS', "Pedidos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TMA_ORDERS', "Orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TMA_ORDERS', "PENDING_TRADUCTION_TMA_ORDERS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TMA_PICKUP_SCREEN', "Tela Pickup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TMA_PICKUP_SCREEN', "Pickup Screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TMA_PICKUP_SCREEN', "PENDING_TRADUCTION_TMA_ORDERS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TME', "TME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TME', "ADT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TME', "TME");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL', "Total");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL', "Total");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL', "Total");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTALED', "Totalizada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTALED', "Totaled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTALED', "Totalizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTALED_HOUR', "Totalizado:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTALED_HOUR', "Totalized:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTALED_HOUR', "Totalizado:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTALIZED', "Totalizada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTALIZED', "Totaled");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTALIZED', "Totalizada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_AMOUNT', "Subtotal / Pago");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_AMOUNT', "Subtotal / Paid");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_AMOUNT', "Subtotal / Pagado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_DELIVERY_REPORT', "Total do Pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_DELIVERY_REPORT', "Order Total");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_DELIVERY_REPORT', "Total de Pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_ITEMS_DELIVERY_REPORT', "Total dos Items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_ITEMS_DELIVERY_REPORT', "Total Items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_ITEMS_DELIVERY_REPORT', "Total de Artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_ITEMS_DISCOUNT_DELIVERY_REPORT', "Total de descontos nos itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_ITEMS_DISCOUNT_DELIVERY_REPORT', "Items total discount");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_ITEMS_DISCOUNT_DELIVERY_REPORT', "Descuento total de Artículos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_ORDERS', "Pedidos / Itens");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_ORDERS', "Orders / Items");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_ORDERS', "Órdenes / Itens");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_PAYABLE', "TOTAL A PAGAR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_PAYABLE', "AMOUNT DUE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_PAYABLE', "TOTAL A PAGAR");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_REMAINING_AMOUNT', "Resta pagar:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_REMAINING_AMOUNT', "Due Amount:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_REMAINING_AMOUNT', "Queda por pagar:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_REPORT', "Pré-Conta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_REPORT', "Total Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_REPORT', "Informe Total");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_SALES', "TOTAL DE VENDAS (R$)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_SALES', "TOTAL SALES ($)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_SALES', "TOTAL DE VENTAS ($)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTAL_TO_PAY', "Cobrar do Cliente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTAL_TO_PAY', "Customer amount due");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTAL_TO_PAY', "Monto a pagar del cliente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ADD', "Adicionais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ADD', "Additional");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ADD', "Adicional");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ADD_BUTTON', "Adicionar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ADD_BUTTON', "Add");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ADD_BUTTON', "Agregar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ADD_COUPON', "Aplicar cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ADD_COUPON', "Add coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ADD_COUPON', "Agregar cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ADD_REMOVE', "Remova/Adicione os ingredientes");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ADD_REMOVE', "Remove/Add Ingredients");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ADD_REMOVE', "Eliminar/Agregar ingredientes");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_APPLIED_COUPONS', "Cupons aplicados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_APPLIED_COUPONS', "Applied coupons");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_APPLIED_COUPONS', "Cupones aplicados");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_APPROVED', "Compra aprovada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_APPROVED', "Approved purchase");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_APPROVED', "Compra aprobada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_APPROVEDMSN', "Aguarde a impressão do comprovante.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_APPROVEDMSN', "Wait for the receipt to print.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_APPROVEDMSN', "Espere a que se imprima el recibo.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_AVAILABLE_COUPONS', "Cupons disponíveis");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_AVAILABLE_COUPONS', "Coupons available");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_AVAILABLE_COUPONS', "Cupones disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_BACK', "Voltar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_BACK', "Back");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_BACK', "Regresar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_BAG', "Sua sacola");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_BAG', "Your bag");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_BAG', "Su pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CANCEL', "Cancelar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CANCEL', "Cancel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CANCEL', "Cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CANCEL_ORDER', "Cancelar pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CANCEL_ORDER', "Cancel order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CANCEL_ORDER', "Cancelar pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CANCEL_ORDER_CONFIRM', "Cancelar o pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CANCEL_ORDER_CONFIRM', "Cancel order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CANCEL_ORDER_CONFIRM', "Cancelar el pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CANCEL_YOUR_ORDER', "cancelar o seu pedido?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CANCEL_YOUR_ORDER', "cancel your order?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CANCEL_YOUR_ORDER', "cancelar tu pedido?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CANNOT_ADD_ITEMS_ON_TOTALED', "Este cupom não pode ser aplicado na tela de pagamento. Para utilizar o cupom, clique em “Continuar Comprando” e selecione o cupom na tela de menu.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CANNOT_ADD_ITEMS_ON_TOTALED', "This coupon cannot be applied on the payment screen. To use the coupon, click on “Continue Shopping” and select the coupon from the menu screen.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CANNOT_ADD_ITEMS_ON_TOTALED', "Este cupón no se puede aplicar en la pantalla de pago. Para usar el cupón, haga clic en &quot;Continuar comprando&quot; y seleccione el cupón en la pantalla del menú.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHANGE_PRODUCT', "Deseja personalizar seu produto?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHANGE_PRODUCT', "Want to customize your product?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHANGE_PRODUCT', "¿Quieres personalizar tu producto?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE', "Escolha ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE', "Choose ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE', "Elige ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_COMBO', "Escolha uma opção:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_COMBO', "Choose one option:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_COMBO', "Elige una opción:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_EWALLET', "Escolha uma opção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_EWALLET', "Choose one option");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_EWALLET', "Elige una opción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_FROM', "Escolha de");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_FROM', "Choose");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_FROM', "Elige de");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_ONE_ITEM', "Escolha 1 opção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_ONE_ITEM', "Choose 1 option");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_ONE_ITEM', "Elige 1 opción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_UNTIL_ONE_ITEM', "Escolha até 1 opção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_UNTIL_ONE_ITEM', "Choose up to 1 option");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_UNTIL_ONE_ITEM', "Elige hasta 1 opción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CHOOSE_UP_TO', "Escolha até");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CHOOSE_UP_TO', "Choose up to");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CHOOSE_UP_TO', "Elige hasta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CLEAN', "Limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CLEAN', "Clean");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CLEAN', "Limpio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CLICK_HERE_CLOSE', "TOQUE AQUI PARA ENCERRAR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CLICK_HERE_CLOSE', "TOUCH HERE TO CLOSE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CLICK_HERE_CLOSE', "TOQUE AQUÍ PARA CERRAR");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CONFIRM', "Confirmar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CONFIRM', "Confirm");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CONFIRM', "Confirmar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CONFIRM_CANCEL_ORDER', "Deseja mesmo ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CONFIRM_CANCEL_ORDER', "Do you really want to ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CONFIRM_CANCEL_ORDER', "¿Realmente quieres ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CONFIRM_ORDER', "Confirmar pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CONFIRM_ORDER', "Confirm order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CONFIRM_ORDER', "Confirmar pedido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CONTINUE_ORDER', "Continuar comprando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CONTINUE_ORDER', "Continue shopping");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CONTINUE_ORDER', "Seguir comprando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_COUPON', "Que tal um cupom?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_COUPON', "How about a coupon?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_COUPON', "¿Qué tal un cupón?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_COUPON_NEXT_SCREEN', "Os descontos serão visualizados na tela de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_COUPON_NEXT_SCREEN', "Coupon discounts will be displayed on the payment screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_COUPON_NEXT_SCREEN', "Los cupones de descuento se mostrarán en la pantalla de pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_COUPON_SUCCESS', "Cupom adicionado com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_COUPON_SUCCESS', "Coupon successfully added!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_COUPON_SUCCESS', "¡Cupón añadido con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CPF_LOYALTY', "Digite seu CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CPF_LOYALTY', "Enter your CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CPF_LOYALTY', "Ingresa su CPF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CUSTOM', "Finalize a personalização ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CUSTOM', "Finish customization ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CUSTOM', "Terminar la personalización ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CUSTOM_1', "antes de adicionar o produto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CUSTOM_1', "before adding the product");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CUSTOM_1', "antes de agregar el producto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_CUSTOM_COMBO', "Você poderá personalizar sua escolha a seguir.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_CUSTOM_COMBO', "You can customize your choice below.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_CUSTOM_COMBO', "Puede personalizar su elección a continuación.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_DEVELOP', "Desenvolvido por:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_DEVELOP', "Developed by:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_DEVELOP', "Desarrollado por:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_DISCOUNTS', "Descontos:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_DISCOUNTS', "Discounts:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_DISCOUNTS', "Descuentos:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_DISCOUNT_ERROR', "Erro ao adicionar cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_DISCOUNT_ERROR', "Error adding coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_DISCOUNT_ERROR', "Error al agregar cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_DOCUMENT', "Incluir CPF/CNPJ na nota?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_DOCUMENT', "Include CPF/CNPJ in the receipt?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_DOCUMENT', "¿Incluir CPF/CNPJ en el recibo?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_EAT_IN', "Comer aqui");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_EAT_IN', "Eat in");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_EAT_IN', "Coma aquí");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_EDIT', "Editar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_EDIT', "Edit");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_EDIT', "Editar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_EMPTY_BAG', "Sua sacola está vazia");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_EMPTY_BAG', "Empty bag");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_EMPTY_BAG', "Su pedido está vacío");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_CPF_EMPTY', "Para prosseguir com a identificação, informe o seu CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_CPF_EMPTY', "To proceed with the identification, inform your CPF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_CPF_EMPTY', "Para proceder con la identificación, informe a su CPF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_CPF_LOYALTY', "Para prosseguir com a identificação, você deve informar um CPF válido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_CPF_LOYALTY', "To proceed with the identification, you must inform a valid CPF.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_CPF_LOYALTY', "Para proceder con la identificación, debe informar a un CPF válido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_FISCAL', "Erro ao fiscalizar venda. A venda será cancelada e os pagamento estornados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_FISCAL', "Error checking sale. The sale will be canceled and payments reversed.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_FISCAL', "Error al verificar la venta. La venta será cancelada y los pagos revertidos.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_LOYALTY', "Para prosseguir com a identificação, informe o seu número de fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_LOYALTY', "To proceed with identification, enter your loyalty number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_LOYALTY', "Para proceder con la identificación, introduce tu número de fidelización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_NAVIGATION', "Não foram encontradas navegações para esse totem.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_NAVIGATION', "No navigations were found for this totem.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_NAVIGATION', "No se encontraron navegaciones para este tótem.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_PAYMENT', "Erro ao inserir pagamento.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_PAYMENT', "Error adding payment.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_PAYMENT', "Error al inserir el pago.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_TEL_EMPTY', "Para prosseguir com a identificação, informe o seu telefone");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_TEL_EMPTY', "To proceed with identification, enter your phone number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_TEL_EMPTY', "Para proceder con la identificación, introduzca su número de teléfono");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ERROR_TEL_LOYALTY', "Para prosseguir com a identificação, você deve informar um número de telefone válido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ERROR_TEL_LOYALTY', "To proceed with the identification, you must inform a valid phone number.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ERROR_TEL_LOYALTY', "Para proceder con la identificación, debe informar a un número de teléfono válido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_EWALLET', "Carteiras digitais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_EWALLET', "Digital wallet");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_EWALLET', "Billeteras  Digitales");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FINALIZED', "Muito obrigado, volte sempre!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FINALIZED', "Thank you very much, come back often!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FINALIZED', "¡Muchas gracias, vuelve a menudo!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FINALIZED_NAME', ". Até mais!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FINALIZED_NAME', ". Until later!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FINALIZED_NAME', ". ¡Hasta luego!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FINALIZED_NAME_1', "Agradecemos pela sua compra, ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FINALIZED_NAME_1', "Thank you for your purchase,");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FINALIZED_NAME_1', "Gracias por su compra,");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FLAGS', "Bandeiras Aceitas:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FLAGS', "Cards:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FLAGS', "Banderas aceptadas:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FLLOW_US', "Siga-nos nas redes sociais");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FLLOW_US', "Follow us on social networks");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FLLOW_US', "Síguenos en redes sociales");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_FOLLOW_EWALLET', "Siga as instruções do aplicativo escolhido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_FOLLOW_EWALLET', "Follow de instructions on the chosen app");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_FOLLOW_EWALLET', "Siga las instrucciones de la aplicación elegida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_GO_CASHIER', "Por gentileza dirija-se ao caixa.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_GO_CASHIER', "Please go to the cashier.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_GO_CASHIER', "Por favor ve al cajero.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_HELLO', "Olá, ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_HELLO', "Hello, ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_HELLO', "Hola, ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_HELLO_WELCOME', "Olá, seja bem vindo(a)!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_HELLO_WELCOME', "Hello welcome! ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_HELLO_WELCOME', "Hola bienvenido(a)! ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ID_STATUS', "TOTEM {} {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ID_STATUS', "TOTEM {} {}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ID_STATUS', "TÓTEM {} {}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_INSERT_COUPON', "Inserir cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_INSERT_COUPON', "Insert coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_INSERT_COUPON', "Insertar cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_INVALID_DOCUMENT', "Documento inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_INVALID_DOCUMENT', "Invalid document");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_INVALID_DOCUMENT', "Documento inválido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_LOYALTY', "Digite seu número de fidelidade");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_LOYALTY', "Enter your loyalty number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_LOYALTY', "Ingresa su numero de fidelidade");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_LOYALTY_TITLE', "APROVEITE CUPONS EXCLUSIVOS!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_LOYALTY_TITLE', "ENJOY EXCLUSIVE COUPONS!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_LOYALTY_TITLE', "¡DISFRUTA DE CUPONES EXCLUSIVOS!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_MANDATORY', "Obrigatório");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_MANDATORY', "Required");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_MANDATORY', "Obligatorio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_MONEY_PAYMENT', "PAGAMENTO EM DINHEIRO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_MONEY_PAYMENT', "CASH PAYMENT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_MONEY_PAYMENT', "PAGO EN DINERO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_MY_COUPON', "Meus cupons");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_MY_COUPON', "My coupons");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_MY_COUPON', "Mis cupones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_NEXT', "Avançar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_NEXT', "Next");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_NEXT', "Próximo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_NOT_INFORM', "Não informar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_NOT_INFORM', "Do not inform");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_NOT_INFORM', "No informar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_NOT_LOYALTY', "Não conseguimos identificar o seu cadastro. Prossiga com a sua compra sem se identificar, ou se preferir, procure um dos nossos atendentes.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_NOT_LOYALTY', "We were unable to identify your registration. Proceed with your purchase without identifying yourself, or if you prefer, look for one of our attendants.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_NOT_LOYALTY', "No pudimos identificar su registro. Realiza tu compra sin identificarte, o si lo prefieres busca uno de nuestros asistentes.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_NUMBER', "Totem nº:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_NUMBER', "Totem No.:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_NUMBER', "Número de tótem:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ONE_ITEM', "item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ONE_ITEM', "item");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ONE_ITEM', "ítem");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ONLY_MONEY', "Totem funcionando somente com pagamento em dinheiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ONLY_MONEY', "Totem working only with cash payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ONLY_MONEY', "Totem trabajando solo con pago en efectivo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_OPTION', "opção");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_OPTION', "option");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_OPTION', "opción");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_OPTIONS', "opções");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_OPTIONS', "options");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_OPTIONS', "opciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ORDER_NUMBER', "Nº do pedido: ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ORDER_NUMBER', "Order number: ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ORDER_NUMBER', "Nº del pedido: ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_ORDER_SUGGESTION', "Combina com seu pedido:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_ORDER_SUGGESTION', "Match your order:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_ORDER_SUGGESTION', "Coincidir con su orden:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAGER_ERROR', "É necessário informar um número válido.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAGER_ERROR', "You must enter a valid number.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAGER_ERROR', "Debes ingresar un número válido.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAGER_TEXT', "INFORME O NÚMERO DO SEU PAGER:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAGER_TEXT', "ENTER YOUR PAGER NUMBER:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAGER_TEXT', "INGRESE SU NÚMERO DE LOCALIZADOR:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAGER_TITLE', "SEJA NOTIFICADO QUANDO O SEU PEDIDO ESTIVER PRONTO!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAGER_TITLE', "GET NOTIFIED WHEN YOUR ORDER IS READY!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAGER_TITLE', "¡RECIBE UNA NOTIFICACIÓN CUANDO TU PEDIDO ESTÉ LISTO!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAY', "Pagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAY', "Pay");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAY', "Pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAYMENT', "Pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAYMENT', "Payment");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAYMENT', "Pago");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PAYMENT_TYPE', "Escolha uma forma de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PAYMENT_TYPE', "Choose your payment method");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PAYMENT_TYPE', "Seleccione una forma de pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PERSONALIZE', "Personalize o seu produto:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PERSONALIZE', "Customize your product:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PERSONALIZE', "Personaliza tu producto:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_PINPAD', "Siga as instruções da máquina de cartão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_PINPAD', "Follow de card machine instructions");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_PINPAD', "Siga las instrucciones de la máquina de tarjetas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_REMOVE_COUPON', "Remover cupom");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_REMOVE_COUPON', "Remove coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_REMOVE_COUPON', "Eliminar cupón");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_REMOVE_COUPON_NEXT_SCREEN', "A remoção do desconto será exibida na tela de pagamento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_REMOVE_COUPON_NEXT_SCREEN', "Removing the discount will be displayed on the payment screen");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_REMOVE_COUPON_NEXT_SCREEN', "La eliminación del descuento se mostrará en la pantalla de pago.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SALE_TYPE', "Onde prefere comer?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SALE_TYPE', "Where you rather eat?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SALE_TYPE', "¿Dónde prefieres comer?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SELECT_EWALLET', "Selecione a carteira digital");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SELECT_EWALLET', "Select digital wallet");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SELECT_EWALLET', "Seleccionar Billetero Digital");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SPLIT_BILL', "Rachar a conta");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SPLIT_BILL', "Split bill?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SPLIT_BILL', "Dividir la cuenta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SPLIT_BILL_AMOUNT', "Quanto deseja pagar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SPLIT_BILL_AMOUNT', "How much do you want to pay?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SPLIT_BILL_AMOUNT', "¿Cuanto quieres pagar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SPLIT_BILL_ERROR', "Valor inválido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SPLIT_BILL_ERROR', "Invalid value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SPLIT_BILL_ERROR', "Valor no válido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_START_ORDER', "Toque para começar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_START_ORDER', "Press to start");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_START_ORDER', "Presione para comenzar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SUGGESTION_CUSTOM', "Você poderá personalizar sua escolha a seguir.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SUGGESTION_CUSTOM', "You will be able to customize your choice below.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SUGGESTION_CUSTOM', "Podrá personalizar su elección a continuación.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_SUMMARY', "Tudo certo com o pedido?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_SUMMARY', "Everything ok with the order?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_SUMMARY', "Todo bien con el pedido?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TAKE_OUT', "Levar para viagem");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TAKE_OUT', "Take out");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TAKE_OUT', "Tomar para ir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TEL_LOYALTY', "Digite seu número de telefone");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TEL_LOYALTY', "Enter your phone number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TEL_LOYALTY', "Ingresa su número de teléfono");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TERMINAL', "TOTEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TERMINAL', "TOTEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TERMINAL', "TÓTEM");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TEXT_MONEY', "PROSSIGA ATÉ O CAIXA PARA PAGAR SEU PEDIDO AO ATENDENTE.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TEXT_MONEY', "PROCEED TO THE CASHIER TO PAY YOUR ORDER TO THE ATTENDANT.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TEXT_MONEY', "PASE AL CAJERO PARA PAGAR SU PEDIDO AL ENCARGADO.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_THANKS', "MUITO OBRIGADO,");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_THANKS', "THANK YOU SO MUCH,");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_THANKS', "MUCHAS GRACIAS,");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_THANKS_BACK', "VOLTE SEMPRE!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_THANKS_BACK', "COME BACK ALWAYS!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_THANKS_BACK', "VUELVE SIEMPRE!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_THE', " os ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_THE', " the ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_THE', " los ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TO', "a");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TO', "to");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TO', "a");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TYPE_DISCOUNT', "Digite o cupom de desconto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TYPE_DISCOUNT', "Enter the discount coupon");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TYPE_DISCOUNT', "Introduce el cupón de descuento");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TYPE_DOCUMENT', "Informar CPF/CNPJ para nota");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TYPE_DOCUMENT', "Inform CPF/CNPJ for receipt");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TYPE_DOCUMENT', "Informar CPF/CNPJ para el recibo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TYPE_NAME', "DIGITE O SEU NOME:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TYPE_NAME', "ENTER YOUR NAME:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TYPE_NAME', "INTRODUZCA SU NOMBRE:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_TYPE_PAYMENT', "Forma de pagemento não cadastrado neste totem.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_TYPE_PAYMENT', "Form of payment not registered in this totem.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_TYPE_PAYMENT', "Forma de pago no registrada en este tótem.");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_WANT_CONTINUE', "Deseja prosseguir?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_WANT_CONTINUE', "Do you want to proceed?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_WANT_CONTINUE', "Quieres proceder?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_WELCOME', ", seja bem vindo(a)!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_WELCOME', ", welcome! ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_WELCOME', ", bienvenido(a)! ");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_YOUR_NAME', "QUAL O SEU NOME?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_YOUR_NAME', "WHAT IS YOUR NAME");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_YOUR_NAME', "¿CUAL ES TU NOMBRE?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOTEM_YOUR_SERVICE', " SEU ATENDIMENTO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOTEM_YOUR_SERVICE', " YOUR SERVICE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOTEM_YOUR_SERVICE', " SU SERVICIO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TOUCH_PRODUCT_TO_START_ORDER', "Selecione um produto para iniciar seu pedido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TOUCH_PRODUCT_TO_START_ORDER', "Select a product to start your order");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TOUCH_PRODUCT_TO_START_ORDER', "Seleccione un artículo para comenzar a ordenar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TO_PAY', "A pagar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TO_PAY', "To pay");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TO_PAY', "Pagar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TRANSFER', "Transferir");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TRANSFER', "Transfer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TRANSFER', "Transferir");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TRANSFERS_REPORT', "Relatório de Sangria e Suprimento");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TRANSFERS_REPORT', "Cash Paid In and Out Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TRANSFERS_REPORT', "Reporte de Ingreso y Retiro de Dinero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TRANSFER_TO_WHICH_OPERATOR', "Para qual garçom?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TRANSFER_TO_WHICH_OPERATOR', "To which operator?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TRANSFER_TO_WHICH_OPERATOR', "¿A qué Mesero?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TRY_AGAIN', "Tentar Novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TRY_AGAIN', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TRY_AGAIN', "Intente nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TRY_AGAIN_REFUND', "Tentar Novamente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TRY_AGAIN_REFUND', "Try again");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TRY_AGAIN_REFUND', "Intentar nuevamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE', "Tipo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE', "Type");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE', "Tipo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_BARCODE', "Código de Barras");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_BARCODE', "Barcode");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_BARCODE', "Código de Barras");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_DATE', "Digite a data ou pressione 'OK' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_DATE', "Enter the date or press 'OK' for current date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_DATE', "Escriba la fecha actual o presione 'OK' para la fecha actual");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_FINAL_DATE', "Digite a data final ou pressione 'OK' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_FINAL_DATE', "Type a final date or press ok to current date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_FINAL_DATE', "Escriba fecha final");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_INITIAL_DATE', "Digite a data inicial ou pressione 'OK' para data atual");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_INITIAL_DATE', "Type a initial date or press ok to current date");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_INITIAL_DATE', "Escriba fecha inicial");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_LABEL', "Tipo");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_LABEL', "Type");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_LABEL', "Tipo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_OPERATOR_ID', "Digite o número do Operador ou pressione 'OK' para todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_OPERATOR_ID', "Enter the operator number or press 'OK' for all");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_OPERATOR_ID', "Escriba el número de Mesero o presione 'OK' para todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_OPERATOR_ID_MESSAGE', "Digite o número do Operador");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_OPERATOR_ID_MESSAGE', "Inform operator number");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_OPERATOR_ID_MESSAGE', "Ingresar número de mesero");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_REPORT_POS', "Digite o número do PDV ou pressione 'OK' para todos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_REPORT_POS', "Enter the POS number or press 'OK' for all");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_REPORT_POS', "Escriba el número de TPV o presione 'OK' para todos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_SKIM_NUMBER', "Entre com o número completo do envelope:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_SKIM_NUMBER', "Inform skim number:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_SKIM_NUMBER', "Ingrese con el número de sobre completo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_TAB_ID', "Digite o número da Comanda:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_TAB_ID', "Enter the Pager number:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_TAB_ID', "Ingrese el número de comanda:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_THE_DRAWER_VALUE', "Insira o valor presente na gaveta:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_THE_DRAWER_VALUE', "Inform drawer value:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_THE_DRAWER_VALUE', "Ingrese el valor de la gaveta");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_WEIGHT_QTY', "Digite a quantidade em quilogramas:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_WEIGHT_QTY', "Inform the weight:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_WEIGHT_QTY', "Informar el peso");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'TYPE_YOUR_DOC_NUM', "Anote aqui seu CPF/CNPJ:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'TYPE_YOUR_DOC_NUM', "Write down your document here:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'TYPE_YOUR_DOC_NUM', "Escriba su document aquí :");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNABLE_PRINT_COUPON', "Impossivel imprimir ultimo cupom:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNABLE_PRINT_COUPON', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNABLE_PRINT_COUPON', "Imposible imprimir último ticket:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNAVAILABLE', "Indisponível");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNAVAILABLE', "Unavailable");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNAVAILABLE', "No Disponible");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNDEFINED', "Desconhecido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNDEFINED', "Unknown");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNDEFINED', "Desconocido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNDEFINED_REASON', "Motivo indefinido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNDEFINED_REASON', "Undefined reason");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNDEFINED_REASON', "Motivo indefinido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNDO', "DESFAZER");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNDO', "UNDO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNDO', "DESHACER");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNINFORMED', "NAO INFORMADO");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNINFORMED', "UNINFORMED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNINFORMED', "NO INFORMADO");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNITS', "Unidades");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNITS', "Units");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNITS', "Unidades");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNIT_PRICE', "Valor Unitário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNIT_PRICE', "Unit Price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNIT_PRICE', "Precio Unitario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNKNOWN_ERROR', "Erro desconhecido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNKNOWN_ERROR', "Unknow error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNKNOWN_ERROR', "Error desconocido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNKNOWN_FISCAL_ERROR', "Erro fiscal desconhecido");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNKNOWN_FISCAL_ERROR', "Unknown tax error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNKNOWN_FISCAL_ERROR', "Error fiscal desconocido");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNKNOWN_VERSION', "Versão Desconhecida");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNKNOWN_VERSION', "Unknown Version");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNKNOWN_VERSION', "Versión Desconocida");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNLINK', "Separar Mesas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNLINK', "Split tables");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNLINK', "Separar Mesas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNLINK_TABLE_CONFIRMATION', "Tem certeza que deseja separar a mesa?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNLINK_TABLE_CONFIRMATION', "Are you sure you want to split the tables?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNLINK_TABLE_CONFIRMATION', "¿Está seguro de separar las mesas?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNMAPPED_STATUS', "??? Unmaped ???");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNMAPPED_STATUS', "??? Unmaped ???");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNMAPPED_STATUS', "??? Unmaped ???");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNNAMED_USER', "Usuário sem nome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNNAMED_USER', "Unnamed user");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNNAMED_USER', "Usuario sin nombre");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNPRICED', "Sem preço");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNPRICED', "Without price");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNPRICED', "Sin precio");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNTOTALIZE', "Destotalizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNTOTALIZE', "Untotalize");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNTOTALIZE', "Destotalizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UNZIPPING_CHROME', "Descompactando o Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UNZIPPING_CHROME', "Unzipping Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UNZIPPING_CHROME', "Descomprimiendo Chrome");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE', "Atualizar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE', "Update");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE', "Actualizar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES', "Atualizar Binários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES', "Update Binaries");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES', "Actualizar Binarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_EXCEPTION', "Ocorreu um erro ao atualizar os binários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_EXCEPTION', "Error updating binaries");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_EXCEPTION', "Error al actualizar binarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_INVALIDDOWNLOADUPDATE', "Erro com os pacotes baixados dos binários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_INVALIDDOWNLOADUPDATE', "$PENDING_TRADUCTION_UPDATE_BINARIES_INVALIDDOWNLOADUPDATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_INVALIDDOWNLOADUPDATE', "$PENDING_TRADUCTION_UPDATE_BINARIES_INVALIDDOWNLOADUPDATE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_MESSAGE', "Binários atualizados com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_MESSAGE', "Binaries updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_MESSAGE', "Binarios actualizados correctamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_MESSAGE_QUESTION', "A atualização dos binários irá reiniciar o sistema da loja quando concluída. \\Tem certeza que deseja atualizar os Binários?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_MESSAGE_QUESTION', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_MESSAGE_QUESTION', "La actualización de los binarios va a reiniciar el sistema de la tienda cuando sea concluída. \\¿Está seguro que desea actualizar los Binarios?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_NOUPDATEFOUND', "Nenhuma atualização de binário encontrada.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_NOUPDATEFOUND', "$PENDING_TRADUCTION_UPDATE_BINARIES_NOUPDATEFOUND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_NOUPDATEFOUND', "$PENDING_TRADUCTION_UPDATE_BINARIES_NOUPDATEFOUND");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_BINARIES_ROLLBACKERROR', "Ocorreu um erro ao recuperar o backup após erro na atualização.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_BINARIES_ROLLBACKERROR', "Error recovering backup");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_BINARIES_ROLLBACKERROR', "Error al recuperar respaldo");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_CHROME', "Atualizar Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_CHROME', "Update Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_CHROME', "Actualizar Chrome");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_CHROME_MESSAGE', "Chrome atualizado com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_CHROME_MESSAGE', "Chrome successfully updated!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_CHROME_MESSAGE', "¡Chrome se actualizó con éxito!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_CHROME_MESSAGE_QUESTION', "A atualização do chrome irá reiniciar o sistema da loja quando concluída. \\Tem certeza que deseja atualizar o Chrome?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_CHROME_MESSAGE_QUESTION', "The chrome update will restart the store system when complete. \\Are you sure you want to update Chrome?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_CHROME_MESSAGE_QUESTION', "La actualización de Chrome reiniciará el sistema de la tienda cuando se complete. \\¿Está seguro de que desea actualizar Chrome?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_DEVICES_SUCCESS', "Dispositivos atualizados com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_DEVICES_SUCCESS', "Devices updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_DEVICES_SUCCESS', "Dispositivos actualizados con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_DISABLED', "Atualização não habilitada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_DISABLED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_DISABLED', "Actualización no habilitada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS', "Atualizar Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS', "Update Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS', "Actualizar Loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_ERRORDAYISOPEN', "Necessário fechar o dia para concluir a operação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_ERRORDAYISOPEN', "Need to close the day to complete the operation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_ERRORDAYISOPEN', "Necesidad de cerrar el día para completar la operación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_ERROREXTRACTINGLOADERS', "Erro ao extrair os loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_ERROREXTRACTINGLOADERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_ERROREXTRACTINGLOADERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_ERROREXTRACTINGLOADERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_ERROREXTRACTINGLOADERS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_ERRORGENESISDATABASENOTFOUND', "Banco de dados Genesis não encontrado, faça o dowload de cardápio primeiro");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_ERRORGENESISDATABASENOTFOUND', "Genesis database not found, please download menu first");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_ERRORGENESISDATABASENOTFOUND', "Base de datos de Génesis no encontrada, primero descargue el menú");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_ERRORUPDATINGDATABASES', "Erro ao atualizar os pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_ERRORUPDATINGDATABASES', "Error updating payments");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_ERRORUPDATINGDATABASES', "Error al actualizar los pagos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_ERRORVALIDATINGOPENEDUSERS', "Necessário fechar todos os usuários abertos para concluir a operação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_ERRORVALIDATINGOPENEDUSERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_ERRORVALIDATINGOPENEDUSERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_ERRORVALIDATINGOPENEDUSERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_ERRORVALIDATINGOPENEDUSERS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_MESSAGE', "Loaders atualizados com sucesso!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_MESSAGE', "Loaders updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_MESSAGE', "Cargadores actualizados correctamente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_MESSAGE_QUESTION', "A atualização dos loaders irá reiniciar o sistema da loja quando concluída. \\Tem certeza que deseja atualizar os Loaders?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_MESSAGE_QUESTION', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_MESSAGE_QUESTION', "La actualización de los loaders va a reiniciar el sistema de la tienda cuando sea concluída. \\¿Está seguro que desea actualizar los Loaders?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_NEED_TO_CLOSE_OPENED_USERS', "Os operadores abertos precisam ser fechados para atualizar os loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_NEED_TO_CLOSE_OPENED_USERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_NEED_TO_CLOSE_OPENED_USERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_NEED_TO_CLOSE_OPENED_USERS', "Los operadores abiertos deben ser cerrados para actualizar los loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_LOADERS_NEED_TO_CLOSE_TABLE_OPENED_USERS', "Os operadores de mesa abertos precisam ser fechados para atualizar os loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_LOADERS_NEED_TO_CLOSE_TABLE_OPENED_USERS', "$PENDING_TRADUCTION_UPDATE_LOADERS_NEED_TO_CLOSE_TABLE_OPENED_USERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_LOADERS_NEED_TO_CLOSE_TABLE_OPENED_USERS', "Los operadores de mesa abiertos deben ser cerrados para actualizar los loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MEDIA_DISABLED', "Atualização de mídia não habilitada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MEDIA_DISABLED', "PENDING_TRADUCTION_UPDATE_MEDIA_DISABLED");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MEDIA_DISABLED', "PENDING_TRADUCTION_UPDATE_MEDIA_DISABLED");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MEDIA_MESSAGE', "Mídias atualizadas com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MEDIA_MESSAGE', "Media updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MEDIA_MESSAGE', "Actualización de Media exitosa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MEDIA_NOTALLIMAGESDOWNLOAD', "Algumas mídias não tiveram o download realizado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MEDIA_NOTALLIMAGESDOWNLOAD', "Some media did not download");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MEDIA_NOTALLIMAGESDOWNLOAD', "Algunos medios no se descargaron");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MEDIA_NOUPDATEFOUND', "Nenhuma atualização de mídia encontrada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MEDIA_NOUPDATEFOUND', "PENDING_TRADUCTION_UPDATE_MEDIA_NOUPDATEFOUND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MEDIA_NOUPDATEFOUND', "PENDING_TRADUCTION_UPDATE_MEDIA_NOUPDATEFOUND");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_ERRORCHECKINGPACKAGEINTEGRITY', "Erro de integridade no pacote baixado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_ERRORCHECKINGPACKAGEINTEGRITY', "Downloaded package integrity error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_ERRORCHECKINGPACKAGEINTEGRITY', "Error de integridad del paquete descargado");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_ERRORDOWNLOADINGUPDATEPACKAGE', "Ocorreu um erro ao realizar o download do Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_ERRORDOWNLOADINGUPDATEPACKAGE', "PENDING_TRADUCTION_UPDATE_MENU_ERRORDOWNLOADINGUPDATEPACKAGE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_ERRORDOWNLOADINGUPDATEPACKAGE', "PENDING_TRADUCTION_UPDATE_MENU_ERRORDOWNLOADINGUPDATEPACKAGE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_ERRORINTEGRITYDATABASE', "Erro de integridade no banco de dados baixado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_ERRORINTEGRITYDATABASE', "Downloaded database integrity error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_ERRORINTEGRITYDATABASE', "Error de integridad de la base de datos descargada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_ERRORRETRIEVINGPENDINGUPDATE', "Ocorreu um erro ao realizar o download do Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_ERRORRETRIEVINGPENDINGUPDATE', "PENDING_TRADUCTION_UPDATE_MENU_ERRORRETRIEVINGPENDINGUPDATE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_ERRORRETRIEVINGPENDINGUPDATE', "PENDING_TRADUCTION_UPDATE_MENU_ERRORRETRIEVINGPENDINGUPDATE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_ERROR_PROCEED', "Houve um erro ao realizar download do cardápio. \\Deseja seguir com a abertura do dia?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_ERROR_PROCEED', "There was an error while downloading the menu. \\Do you want to proceed to open the day?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_ERROR_PROCEED', "PENDING_TRADUCTION_UPDATE_MENU_ERROR");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_EXCEPTION', "Ocorreu um erro durante a atualização");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_EXCEPTION', "Error in update");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_EXCEPTION', "Ocurrió un error durante la actualización");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_MESSAGE', "Recomendamos a atualização do cardápio sem nenhum pedido ativo no Restaurante.\\A atualização de cardápio irá reiniciar o sistema da loja quando concluída.\\Confirma a atualização do cardápio?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_MESSAGE', "We recommend menu update with no active orders.\\Menu update will restart the store when it's done.\\Confirm update?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_MESSAGE', "Recomendamos actualizar el menú sin pedidos activos.\\La Actualización del Menú reiniciará la tienda cuando termine.\\¿Confirmar actualización?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_NOUPDATEFOUND', "Nenhuma atualização de cardápio encontrada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_NOUPDATEFOUND', "PENDING_TRADUCTION_UPDATE_MENU_NOUPDATEFOUND");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_NOUPDATEFOUND', "PENDING_TRADUCTION_UPDATE_MENU_NOUPDATEFOUND");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_VERSION_AVAILABLE', "Atualização {0} disponível.\Deseja aplicar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_VERSION_AVAILABLE', "Update {0} available.\Apply?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_VERSION_AVAILABLE', "Actualización {0} disponible.\¿Desea Aplicar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_MENU_VERSION_PENDING', "A atualização {0} de cardápio está pendente.\\Ela será aplicada e o sistema será reiniciado.\\Deseja continuar?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_MENU_VERSION_PENDING', "PENDING_TRADUCTION_UPDATE_MENU_VERSION_PENDING");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_MENU_VERSION_PENDING', "La actualización {0} del menú está pendiente.\\La actualización será aplicada y el sistema se reiniciará.\\¿Desea continuar?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_USERS', "Atualizar Usuários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_USERS', "Update Users");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_USERS', "Actualizar Usuarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_USERS_ERRORRETRIEVINGUSERS', "Ocorreu um erro ao realizar a atualização dos usuários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_USERS_ERRORRETRIEVINGUSERS', "PENDING_TRADUCTION_UPDATE_USERS_ERRORRETRIEVINGUSERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_USERS_ERRORRETRIEVINGUSERS', "PENDING_TRADUCTION_UPDATE_USERS_ERRORRETRIEVINGUSERS");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_USERS_MESSAGE', "Usuários atualizados com sucesso");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_USERS_MESSAGE', "Users updated successfully");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_USERS_MESSAGE', "Usuarios actualizados con éxito");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATE_VTT', "ATUALIZAR VTT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATE_VTT', "UPDATE VTT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATE_VTT', "ACTUALIZAR VTT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_APACHE_HTTPD_CONF', "Atualizando arquivo httpd.conf do Apache");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_APACHE_HTTPD_CONF', "Updating Apache's httpd.conf file");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_APACHE_HTTPD_CONF', "Actualización del archivo httpd.conf de Apache");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_BINARIES', "Atualizando binários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_BINARIES', "UPDATING_BINARIES");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_BINARIES', "Actualizando binarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_CHROME', "Atualizando o Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_CHROME', "Updating Chrome");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_CHROME', "Actualización de Chrome");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_CONFIGURED_ALERTS_PLEASE_WAIT', "Atualizando configurações de alertas\\Por favor aguarde...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_CONFIGURED_ALERTS_PLEASE_WAIT', "Updating alert settings\\Please wait...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_CONFIGURED_ALERTS_PLEASE_WAIT', "Actualizando configuración de alerta\\Por favor espere...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_LOADERS', "Atualizando Loaders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_LOADERS', "UPDATING_LOADERS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_LOADERS', "Actualizando Loaders");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_NAVIGATIONS', "Atualizando navegações");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_NAVIGATIONS', "Updating Navigations");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_NAVIGATIONS', "Actualización de navegaciones");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_PAYMENTS', "Atualizando pagamentos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_PAYMENTS', "Updating payments");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_PAYMENTS', "Actualización de pagos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_POS_MENU', "Atualizando Menu no PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_POS_MENU', "Updating Menu at the POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_POS_MENU', "Actualizando Menú en el TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_POS_USERS', "Atualizando usuários no PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_POS_USERS', "Updating users at the POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_POS_USERS', "Actualizando usuarios en el TPV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_START_STOP_FILES', "Atualizando arquivos start/stop");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_START_STOP_FILES', "Updating start/stop files");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_START_STOP_FILES', "Actualización de archivos de inicio/detención");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_SYSTEM', "Atualizando o Sistema");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_SYSTEM', "UPDATING_SYSTEM");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_SYSTEM', "Actualizando el Sistema");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'UPDATING_USER_ROLES', "Atualizando funções dos usuários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'UPDATING_USER_ROLES', "Updating user roles");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'UPDATING_USER_ROLES', "$PENDING_TRADUCTION_UPDATING_USER_ROLES");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER', "Usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER', "Usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USERS', "Usuários");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USERS', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USERS', "Usuarios");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_AUTHENTICATION', "Autenticação do Usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_AUTHENTICATION', "User Authentication");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_AUTHENTICATION', "Autenticación de Usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_AUTHENTICATION_WITH_PARAMETER', "Autenticação: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_AUTHENTICATION_WITH_PARAMETER', "Authentication: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_AUTHENTICATION_WITH_PARAMETER', "Autenticación: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_CANCELLATION', "Cancelamento do usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_CANCELLATION', "User Cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_CANCELLATION', "Cancelación de usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_CANNOT_BE_ACTIVATED', "Este usuário não pode ser ativado!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_CANNOT_BE_ACTIVATED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_CANNOT_BE_ACTIVATED', "¡Este usuario no puede ser activado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_CANNOT_BE_CHANGED', "Este usuário não pode ser alterado!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_CANNOT_BE_CHANGED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_CANNOT_BE_CHANGED', "¡Este usuario no puede ser alterado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_CANNOT_BE_DEACTIVATED', "Este usuário não pode ser desativado!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_CANNOT_BE_DEACTIVATED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_CANNOT_BE_DEACTIVATED', "¡Este usuario no puede ser desactivado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_CANNOT_BE_REMOVED', "Esse usuário não pode ser removido!");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_CANNOT_BE_REMOVED', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_CANNOT_BE_REMOVED', "¡Este usuario no puede ser eliminado!");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_FISCAL_CANCELLATION', "Cancelamento fiscal do usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_FISCAL_CANCELLATION', "User tax cancellation");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_FISCAL_CANCELLATION', "Cancelación de impuestos de usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_ID_ALREADY', "Id de usuário já existente");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_ID_ALREADY', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_ID_ALREADY', "ID de usuario ya existente");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_IS_LOGIN_IN_POS', "Operador está logando no PDV ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_IS_LOGIN_IN_POS', "Operator logging into POS ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_IS_LOGIN_IN_POS', "Operador está siendo logado en el TDV ({0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_LOGGED_IN_ANOTHER_POS_TYPE', "Usuário logado em outro tipo de PDV ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_LOGGED_IN_ANOTHER_POS_TYPE', "Operator logged in another type of POS ({0})");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_LOGGED_IN_ANOTHER_POS_TYPE', "El usuario inició sesión en otro tipo de TPV ({0})");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_LOGGED_IN_THIS_POS', "Existe um usuário logado neste PDV");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_LOGGED_IN_THIS_POS', "There is an operator logged in this POS");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_LOGGED_IN_THIS_POS', "Otro Mesero registrado actualmente en este TDV");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_NOT_FOUND', "Não existem usuários para essa Loja");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_NOT_FOUND', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_NOT_FOUND', "No existen usuarios para esta tienda");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_NOT_OPENED', "Usuário não está aberto");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_NOT_OPENED', "User not opened");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_NOT_OPENED', "Usuario no abierto");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'USER_REQUESTED_CANCELLATION', "Cancelamento solicitado pelo usuário");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'USER_REQUESTED_CANCELLATION', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'USER_REQUESTED_CANCELLATION', "Cancelamiento solicitado por el usuario");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VALIDATION_ERROR', "Erro de validação");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VALIDATION_ERROR', "validation error");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VALIDATION_ERROR', "Error de validacion");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VALUE', "Valor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VALUE', "Value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VALUE', "Valor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VALUE_DELIVERY_REPORT', "Valor");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VALUE_DELIVERY_REPORT', "Value");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VALUE_DELIVERY_REPORT', "Valor");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VERSION', "Versão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VERSION', "Version");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VERSION', "Versión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VERSION_MENU', "Cardápio");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VERSION_MENU', "Menu");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VERSION_MENU', "Menu");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOIDED_LINES_REPORT', "Relatório de Linhas Canceladas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOIDED_LINES_REPORT', "Voided Lines Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOIDED_LINES_REPORT', "Reporte de Líneas Anuladas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOIDED_REPORT', "Relatório de Vendas Canceladas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOIDED_REPORT', "Cancelled Sales Report");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOIDED_REPORT', "Reporte de Ventas Canceladas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOID_DELIVERY_ORDER', "Cancelar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOID_DELIVERY_ORDER', "Cancel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOID_DELIVERY_ORDER', "Cancelar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOID_SALE', "Cancelar Pedidos Fechados");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOID_SALE', "Cancel Closed Orders");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOID_SALE', "Cancelar Cuentas Cerradas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOUCHER', "Voucher");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOUCHER', "Voucher");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOUCHER', "Voucher");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'VOUCHER_CARD', "Vale Refeição");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'VOUCHER_CARD', "Voucher");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'VOUCHER_CARD', "Tarjeta de Alimentación");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAITING_FOR_DELIVERY', "Aguardando entrega");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAITING_FOR_DELIVERY', "Waiting for delivery");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAITING_FOR_DELIVERY', "Esperando para la entrega");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAITING_FOR_OPERATOR_CONFIRMATION', "Aguardando confirmação do operador...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAITING_FOR_OPERATOR_CONFIRMATION', "Awaiting operator confirmation...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAITING_FOR_OPERATOR_CONFIRMATION', "En espera de confirmación del operador...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAITING_SITEF_ADD_PAYMENT', "Aguardando comunicação com TEF Integrado");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAITING_SITEF_ADD_PAYMENT', "Waiting for TEF");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAITING_SITEF_ADD_PAYMENT', "Esperando TEF");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAITING_TO_BE_SEATED', "Aguardando");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAITING_TO_BE_SEATED', "Waiting");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAITING_TO_BE_SEATED', "Esperando");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAIT_MESSAGE', "Aguarde...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAIT_MESSAGE', "Wait...");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAIT_MESSAGE', "Espere...");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WAIT_SCALE_WEIGHT', "Aguarde o peso da balança");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WAIT_SCALE_WEIGHT', "Waiting for scale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WAIT_SCALE_WEIGHT', "Espere a la balanza");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WANT_TO_CLOSE_THE_SYSTEM', "Você tem certeza que deseja fechar o sistema?\\ OBS: Essa operação pode demorar alguns segundos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WANT_TO_CLOSE_THE_SYSTEM', "$PENDING_TRADUCTION");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WANT_TO_CLOSE_THE_SYSTEM', "¿Está seguro que desea cerrar el sistema?\\ OBS: Esta operación puede demorar algunos segundos");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WANT_TO_PRINT_CUSTOMER_RECEIPT', "Deseja imprimir o comprovante do cliente?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WANT_TO_PRINT_CUSTOMER_RECEIPT', "Want to print the customer receipt?");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WANT_TO_PRINT_CUSTOMER_RECEIPT', "¿Desea imprimir el recibo del cliente?");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WARNINGS', "Alertas");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WARNINGS', "Warnings");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WARNINGS', "Alertas");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WARNING_LOW_PAPER', "Impressora com pouco papel");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WARNING_LOW_PAPER', "low paper printer");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WARNING_LOW_PAPER', "impresora de papel baja");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WARNING_REPORT_ITEMS_CONFERENCE', "Os valores totais podem ser diferentes devido a eventuais descontos");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WARNING_REPORT_ITEMS_CONFERENCE', "PENDING_TRADUCTION_WARNING_REPORT_ITEMS_CONFERENCE");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WARNING_REPORT_ITEMS_CONFERENCE', "PENDING_TRADUCTION_WARNING_REPORT_ITEMS_CONFERENCE");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WEIGHT_SYMBOL', "KG");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WEIGHT_SYMBOL', "KG");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WEIGHT_SYMBOL', "KG");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WELCOME_CLIENT_LOYALTY_INFO', "Cliente: {0}\\CPF: {1}\\Telefone: {2}\\Mensagem: {3}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WELCOME_CLIENT_LOYALTY_INFO', "Client: {0}\\Document: {1}\\Telephone: {2}\\Message: {3}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WELCOME_CLIENT_LOYALTY_INFO', "Cliente: {0}\\Documento: {1}\\Teléfono: {2}\\Mensaje: {3}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WIFI_ACCESS_CODE', "CODIGO DE ACESSO: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WIFI_ACCESS_CODE', "ACCESS CODE: {0}");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WIFI_ACCESS_CODE', "CÓDIGO DE ACCESO: {0}");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WIFI_PRINTER_MESSAGE', "WI-FI GRATIS POR 30min");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WIFI_PRINTER_MESSAGE', "FREE WI-FI FOR 30min");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WIFI_PRINTER_MESSAGE', "WI-FI GRATIS POR 30mins");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WIHOUT_DISCOUNTS_TO_CLEAN', "Não existem descontos para limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WIHOUT_DISCOUNTS_TO_CLEAN', "No discounts to clear");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WIHOUT_DISCOUNTS_TO_CLEAN', "No existen Descuentos para borrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT', "(Sem)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT', "(Without)");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT', "(Sin)");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT_ANY_LOYALTY_BENEFITS_TO_SHOW', "Não possui benefícios para listar ou já adicionou todos os benefícios disponíveis");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT_ANY_LOYALTY_BENEFITS_TO_SHOW', "Has no benefits to list or has already added all available benefits");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT_ANY_LOYALTY_BENEFITS_TO_SHOW', "No tiene beneficios para listar o ya agregó todos los beneficios disponibles");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT_DOCUMENT', "Sem CPF/CNPJ");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT_DOCUMENT', "PENDING_TRADUCTION_WITHOUT_DOCUMENT");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT_DOCUMENT', "PENDING_TRADUCTION_WITHOUT_DOCUMENT");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT_ORDER_PICKLIST_TO_REPRINT', "Pedido sem PickLists para reimpressão");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT_ORDER_PICKLIST_TO_REPRINT', "Order without PickLists for reprint");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT_ORDER_PICKLIST_TO_REPRINT', "Pedido sin PickLists para reimpresión");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT_PERMISSION_LEVEL_TO_SEE_TABLE', "Sem nível de permissão suficiente para ver a mesa");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT_PERMISSION_LEVEL_TO_SEE_TABLE', "No permission to see the table");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT_PERMISSION_LEVEL_TO_SEE_TABLE', "Nivel de permiso insuficiente para ver la mesa");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WITHOUT_TENDER_TO_CLEAR', "Não existem pagamentos para limpar");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WITHOUT_TENDER_TO_CLEAR', "PENDING_TRADUCTION_WITHOUT_TENDER_TO_CLEAR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WITHOUT_TENDER_TO_CLEAR', "No existen pagos para borrar");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'WRONG_SALE', "Venda Errada");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'WRONG_SALE', "Wrong Sale");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'WRONG_SALE', "Venta Errada");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'YES', "Sim");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'YES', "YES");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'YES', "Sí");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'YOU_WILL_BE_CALLED', "VOCE SERA CHAMADO POR:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'YOU_WILL_BE_CALLED', "YOU WILL BE CALLED BY:");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'YOU_WILL_BE_CALLED', "SERÁS LLAMADO POR:");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'ZIP_CODE', "CEP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'ZIP_CODE', "CEP");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'ZIP_CODE', "CEP");

INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (4, 'Z_REDUCTION_ERROR', "Erro obtendo informações da Redução Z. Verifique a impressora.");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (1, 'Z_REDUCTION_ERROR', "$PENDING_TRADUCTION_Z_REDUCTION_ERROR");
INSERT INTO Messages(LangCode, MessageKey, Message) VALUES (6, 'Z_REDUCTION_ERROR', "Error obteniendo informaciones de Reducción Z. Verifique la impresora.");

COMMIT TRANSACTION;
